@@ -36,7 +36,7 @@ public:
     {
         if (connected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "deconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] deconnect " << this << endl;
             x->deconnect(linkX, reuse);
         }
     }
@@ -44,7 +44,7 @@ public:
     {
         if (deconnected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "reconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] reconnect " << this << endl;
             assert(linkX->prev == NULL && linkX->next == NULL);
             x->getConstrs()->push_back(linkX, true);
         }
@@ -60,7 +60,7 @@ public:
         if (v != x)
             return x;
         else
-            exit(EXIT_FAILURE);
+            throw InternalError();
     }
 
     int getIndex(Variable* var) const FINAL
@@ -141,7 +141,7 @@ public:
     {
         if (connected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "deconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] deconnect " << this << endl;
             x->deconnect(linkX, reuse);
             y->deconnect(linkY, reuse);
         }
@@ -150,7 +150,7 @@ public:
     {
         if (deconnected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "reconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] reconnect " << this << endl;
             assert(linkX->prev == NULL && linkX->next == NULL);
             x->getConstrs()->push_back(linkX, true);
             assert(linkY->prev == NULL && linkY->next == NULL);
@@ -169,7 +169,7 @@ public:
         else if (v == y)
             return x;
         else
-            exit(EXIT_FAILURE);
+            throw InternalError();
     }
 
     int getIndex(Variable* var) const FINAL
@@ -239,6 +239,10 @@ public:
         , linkY(NULL)
         , linkZ(NULL)
     {
+#ifdef NO_STORE_BINARY_COSTS
+        cerr << "Sorry, only binary cost functions!" << endl;
+        throw BadConfiguration();
+#endif
         assert(xx != yy);
         assert(xx != zz);
         assert(yy != zz);
@@ -272,7 +276,7 @@ public:
     {
         if (connected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "deconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] deconnect " << this << endl;
             x->deconnect(linkX, reuse);
             y->deconnect(linkY, reuse);
             z->deconnect(linkZ, reuse);
@@ -282,7 +286,7 @@ public:
     {
         if (deconnected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "reconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] reconnect " << this << endl;
             assert(linkX->prev == NULL && linkX->next == NULL);
             //			if (linkX->content.constr->isTriangle()) x->getTriangles()->push_back(linkX, true);
             //			else
@@ -313,7 +317,7 @@ public:
             return z;
             break;
         default:
-            exit(EXIT_FAILURE);
+            throw InternalError();
         }
     }
 
@@ -332,7 +336,7 @@ public:
         else if ((y == v2) && (z == v1))
             return x;
         else
-            exit(EXIT_FAILURE);
+            throw InternalError();
     }
 
     int getIndex(Variable* var) const FINAL
@@ -361,7 +365,7 @@ public:
             return min(x->wcspIndex, y->wcspIndex);
             break;
         default:
-            exit(EXIT_FAILURE);
+            throw InternalError();
         }
     }
     int getSmallestVarIndexInScope() FINAL
@@ -394,7 +398,7 @@ public:
             return min(x->getDACOrder(), y->getDACOrder());
             break;
         default:
-            exit(EXIT_FAILURE);
+            throw InternalError();
         }
     }
     Variable* getDACVar(int varCtrIndex) const FINAL
@@ -408,7 +412,7 @@ public:
             case 2:
                 return z;
             default:
-                exit(EXIT_FAILURE);
+                throw InternalError();
             }
         } else { // varCtrIndex >= 1
             switch (dacvar) {
@@ -419,7 +423,7 @@ public:
             case 2:
                 return ((varCtrIndex == 1) == (x->getDACOrder() < y->getDACOrder())) ? x : y;
             default:
-                exit(EXIT_FAILURE);
+                throw InternalError();
             }
         }
     }
@@ -457,6 +461,7 @@ extern int cmpDAC(const void* var1, const void* var2);
 class AbstractNaryConstraint : public Constraint {
 protected:
     int arity_;
+    unsigned int maxInitDomSize;
 
     EnumeratedVariable** scope;
     EnumeratedVariable** scope_dac; // scope sorted by increasing DAC order
@@ -471,13 +476,21 @@ public:
     AbstractNaryConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int arity_in)
         : Constraint(wcsp)
         , arity_(arity_in)
+        , maxInitDomSize(0)
     {
+#if defined(NO_STORE_BINARY_COSTS) || defined(NO_STORE_TERNARY_COSTS)
+        cerr << "Sorry, no " << arity_in << "-ary cost functions!" << endl;
+        throw BadConfiguration();
+#endif
         scope = new EnumeratedVariable*[arity_];
         scope_dac = new EnumeratedVariable*[arity_];
         links = new DLink<ConstraintLink>*[arity_];
 
         for (int i = 0; i < arity_; i++) {
             EnumeratedVariable* var = scope_in[i];
+            if (var->getDomainInitSize() > maxInitDomSize) {
+                maxInitDomSize = var->getDomainInitSize();
+            }
             scope_inv[var->wcspIndex] = i;
             scope[i] = var;
             scope_dac[i] = var;
@@ -489,6 +502,7 @@ public:
 
     AbstractNaryConstraint(WCSP* wcsp)
         : Constraint(wcsp)
+        , maxInitDomSize(0)
     {
     }
 
@@ -496,6 +510,9 @@ public:
     {
         delete[] scope;
         delete[] scope_dac;
+        for (int i = 0; i < arity_; i++) {
+            delete links[i];
+        }
         delete[] links;
     }
 
@@ -546,7 +563,7 @@ public:
     {
         if (connected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "deconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] deconnect " << this << endl;
             for (int i = 0; i < arity_; i++)
                 deconnect(i, reuse);
         }
@@ -556,7 +573,7 @@ public:
     {
         if (deconnected()) {
             if (ToulBar2::verbose >= 3)
-                cout << "reconnect " << this << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] reconnect " << this << endl;
             for (int i = 0; i < arity_; i++) {
                 assert(links[i]->prev == NULL && links[i]->next == NULL);
                 scope[i]->getConstrs()->push_back(links[i], true);
@@ -566,7 +583,8 @@ public:
 
     virtual Cost eval(const Tuple& t)
     {
-        cout << "dummy eval on: " << *this << endl;
+        if (ToulBar2::verbose >= 0)
+            cout << "dummy eval on: " << *this << endl;
         return MIN_COST;
     }
     //    virtual void insertTuple( Tuple t, Cost c, EnumeratedVariable** scope_in ) { }
