@@ -1,6 +1,6 @@
 /** \file tb2binconstr.hpp
  *  \brief Binary constraint applied on variables with enumerated domains.
- *
+ *  \warning EAC is not applied on duplicated binary constraints
  */
 
 #ifndef TB2BINCONSTR_HPP_
@@ -33,8 +33,11 @@ protected:
     unsigned int sizeY;
     vector<StoreCost> deltaCostsX;
     vector<StoreCost> deltaCostsY;
+#ifdef NO_STORE_BINARY_COSTS
+    vector<Cost> costs;
+#else
     vector<StoreCost> costs;
-
+#endif
     vector<Value> supportX;
     vector<Value> supportY;
 
@@ -84,7 +87,7 @@ public:
         unsigned int ix = x->toIndex(vx);
         unsigned int iy = y->toIndex(vy);
         Cost res = costs[ix * sizeY + iy];
-        //if (res >= wcsp->getUb() || res - deltaCostsX[ix] - deltaCostsY[iy] + wcsp->getLb() >= wcsp->getUb()) return wcsp->getUb();
+        // if (res >= wcsp->getUb() || res - deltaCostsX[ix] - deltaCostsY[iy] + wcsp->getLb() >= wcsp->getUb()) return wcsp->getUb();
         res -= deltaCostsX[ix] + deltaCostsY[iy];
         assert(res >= MIN_COST);
         return res;
@@ -123,13 +126,43 @@ public:
         return minCost;
     }
 
+    vector<Cost> getDeltaCostsX()
+    {
+        vector<Cost> deltas;
+        for (Cost cost : deltaCostsX) {
+            deltas.push_back(cost);
+        }
+        return deltas;
+    }
+    vector<Cost> getDeltaCostsY()
+    {
+        vector<Cost> deltas;
+        for (Cost cost : deltaCostsY) {
+            deltas.push_back(cost);
+        }
+        return deltas;
+    }
+    void setDeltaCostsX(const vector<Cost>& update)
+    {
+        assert(update.size() == sizeX);
+        for (unsigned int a = 0; a < sizeX; a++) {
+            deltaCostsX[a] = update[a];
+        }
+    }
+    void setDeltaCostsY(const vector<Cost>& update)
+    {
+        assert(update.size() == sizeY);
+        for (unsigned int b = 0; b < sizeY; b++) {
+            deltaCostsY[b] = update[b];
+        }
+    }
     Cost getCost(EnumeratedVariable* xx, EnumeratedVariable* yy, Value vx, Value vy)
     {
         unsigned int vindex[2];
         vindex[getIndex(xx)] = xx->toIndex(vx);
         vindex[getIndex(yy)] = yy->toIndex(vy);
         Cost res = costs[vindex[0] * sizeY + vindex[1]];
-        //if (res >= wcsp->getUb() || res - deltaCostsX[vindex[0]] - deltaCostsY[vindex[1]] + wcsp->getLb() >= wcsp->getUb()) return wcsp->getUb();
+        // if (res >= wcsp->getUb() || res - deltaCostsX[vindex[0]] - deltaCostsY[vindex[1]] + wcsp->getLb() >= wcsp->getUb()) return wcsp->getUb();
         res -= deltaCostsX[vindex[0]] + deltaCostsY[vindex[1]];
         assert(res >= MIN_COST);
         return res;
@@ -144,7 +177,13 @@ public:
 
     void addcost(Value vx, Value vy, Cost mincost)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "addcost(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] addcost(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
         assert(mincost >= MIN_COST || !LUBTEST(getCost(vx, vy), -mincost) || ToulBar2::isZ); // Warning! negative costs can be added temporally by variable elimination on the fly
         unsigned int ix = x->toIndex(vx);
         unsigned int iy = y->toIndex(vy);
@@ -153,7 +192,13 @@ public:
 
     void addcost(EnumeratedVariable* xin, EnumeratedVariable* yin, Value vx, Value vy, Cost mincost)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "addcost(C" << xin->getName() << "," << yin->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] addcost(C" << xin->getName() << "," << yin->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
         assert(mincost >= MIN_COST || !LUBTEST(getCost(xin, yin, vx, vy), -mincost));
         if (xin == x) {
             costs[x->toIndex(vx) * sizeY + y->toIndex(vy)] += mincost;
@@ -164,6 +209,14 @@ public:
 
     void setCost(Cost c)
     {
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(std::all_of(deltaCostsX.begin(), deltaCostsX.end(), [](Cost c) { return c == MIN_COST; }));
+        assert(std::all_of(deltaCostsY.begin(), deltaCostsY.end(), [](Cost c) { return c == MIN_COST; }));
         for (unsigned int a = 0; a < sizeX; a++)
             for (unsigned int b = 0; b < sizeY; b++)
                 costs[a * sizeY + b] = c;
@@ -171,7 +224,15 @@ public:
 
     void setcost(EnumeratedVariable* xin, EnumeratedVariable* yin, Value vx, Value vy, Cost mincost)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "setcost(C" << xin->getName() << "," << yin->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] setcost(C" << xin->getName() << "," << yin->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+        assert(std::all_of(deltaCostsX.begin(), deltaCostsX.end(), [](Cost c) { return c == MIN_COST; }));
+        assert(std::all_of(deltaCostsY.begin(), deltaCostsY.end(), [](Cost c) { return c == MIN_COST; }));
         if (xin == x)
             costs[x->toIndex(vx) * sizeY + y->toIndex(vy)] = mincost;
         else
@@ -180,13 +241,27 @@ public:
 
     void setcost(Value vx, Value vy, Cost mincost)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "setcost(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] setcost(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << vx << "," << vy << "), " << mincost << ")" << endl), true));
+        assert(std::all_of(deltaCostsX.begin(), deltaCostsX.end(), [](Cost c) { return c == MIN_COST; }));
+        assert(std::all_of(deltaCostsY.begin(), deltaCostsY.end(), [](Cost c) { return c == MIN_COST; }));
         costs[x->toIndex(vx) * sizeY + y->toIndex(vy)] = mincost;
     }
 
     void addCosts(EnumeratedVariable* xin, EnumeratedVariable* yin, vector<Cost>& costsin)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "add binary cost vector to (C" << getVar(0)->getName() << "," << getVar(1)->getName() << ") " << costsin[0] << "," << costsin[1] << "," << costsin[2] << "," << costsin[3] << " ..." << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] add binary cost vector to (C" << getVar(0)->getName() << "," << getVar(1)->getName() << ") " << costsin[0] << "," << costsin[1] << "," << costsin[2] << "," << costsin[3] << " ..." << endl), true));
         assert(costsin.size() <= costs.size());
         unsigned int ix, iy;
         for (EnumeratedVariable::iterator iterx = x->begin(); iterx != x->end(); ++iterx) {
@@ -203,7 +278,13 @@ public:
 
     void addCosts(BinaryConstraint* xy)
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "add binary cost function to (C" << getVar(0)->getName() << "," << getVar(1)->getName() << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] add binary cost function to C(" << getVar(0)->getName() << "," << getVar(1)->getName() << ")" << endl), true));
         assert(((x == xy->x) && (y == xy->y)) || ((x == xy->y) && (y == xy->x)));
         unsigned int ix, iy;
         for (EnumeratedVariable::iterator iterx = x->begin(); iterx != x->end(); ++iterx) {
@@ -211,7 +292,7 @@ public:
                 ix = x->toIndex(*iterx);
                 iy = y->toIndex(*itery);
                 Cost c = costs[ix * sizeY + iy];
-                //if(costs[ix * sizeY + iy] < wcsp->getUb()) //BUG with BTD: ub is only local, deltaCosts should be considered
+                // if(costs[ix * sizeY + iy] < wcsp->getUb()) //BUG with BTD: ub is only local, deltaCosts should be considered
                 {
                     costs[ix * sizeY + iy] = c + xy->getCost(x, y, *iterx, *itery);
                 }
@@ -221,7 +302,13 @@ public:
 
     void clearCosts()
     {
-        assert(ToulBar2::verbose < 4 || ((cout << "clear cost (C" << getVar(0)->getName() << "," << getVar(1)->getName() << ")" << endl), true));
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] clearcosts(C" << getVar(0)->getName() << "," << getVar(1)->getName() << ")" << endl), true));
         for (unsigned int i = 0; i < sizeX; i++)
             deltaCostsX[i] = MIN_COST;
         for (unsigned int j = 0; j < sizeY; j++)
@@ -233,6 +320,28 @@ public:
         }
     }
 
+    void clearFiniteCosts()
+    {
+#ifdef NO_STORE_BINARY_COSTS
+        if (Store::getDepth() > 0) {
+            cerr << "Cannot modify finite costs in binary cost functions during search!" << endl;
+            throw BadConfiguration();
+        }
+#endif
+        assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] clearfinitecosts(C" << getVar(0)->getName() << "," << getVar(1)->getName() << ")" << endl), true));
+        for (unsigned int i = 0; i < sizeX; i++)
+            deltaCostsX[i] = MIN_COST;
+        for (unsigned int j = 0; j < sizeY; j++)
+            deltaCostsY[j] = MIN_COST;
+        for (unsigned int i = 0; i < sizeX; i++) {
+            for (unsigned int j = 0; j < sizeY; j++) {
+                if (!CUT(costs[i * sizeY + j], wcsp->getUb())) {
+                    costs[i * sizeY + j] = MIN_COST;
+                }
+            }
+        }
+    }
+
     Cost getMaxFiniteCost()
     {
         Cost ub = wcsp->getUb();
@@ -249,17 +358,23 @@ public:
 
     void setInfiniteCost(Cost ub)
     {
-        Cost mult_ub = ((ub < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, ub * MEDIUM_COST)) : ub);
+        bool modified = false;
+        Cost mult_ub = ((wcsp->getUb() < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, wcsp->getUb() * MEDIUM_COST)) : wcsp->getUb());
         for (EnumeratedVariable::iterator iterx = x->begin(); iterx != x->end(); ++iterx) {
             unsigned int ix = x->toIndex(*iterx);
             for (EnumeratedVariable::iterator itery = y->begin(); itery != y->end(); ++itery) {
                 unsigned int iy = y->toIndex(*itery);
-                Cost cost = costs[ix * sizeY + iy];
+                Cost cost = costs[ix * sizeY + iy] + x->getCost(*iterx) + y->getCost(*itery);
                 Cost delta = deltaCostsX[ix] + deltaCostsY[iy];
-                if (CUT(cost - delta, ub))
+                if (CUT(cost - delta, ub)) {
                     costs[ix * sizeY + iy] = mult_ub + delta;
+                    modified = true;
+                }
             }
         }
+        if (modified) {
+            propagate();
+        }
     }
 
     Cost evalsubstr(const Tuple& s, Constraint* ctr) FINAL
@@ -346,7 +461,8 @@ public:
     {
         Value v0 = x->toValue(t[0]);
         Value v1 = y->toValue(t[1]);
-        setcost(v0, v1, c);
+        Cost oldc = getCost(v0, v1);
+        addcost(v0, v1, c - oldc);
     }
 
     void addtoTuple(const Tuple& t, Cost c) FINAL
@@ -356,18 +472,6 @@ public:
         addcost(v0, v1, c);
     }
 
-    //    void setTuple( unsigned int* t, Cost c )  {
-    //        Value v0 = x->toValue(t[0]);
-    //        Value v1 = y->toValue(t[1]);
-    //        setcost( v0, v1, c );
-    //    }
-    //
-    //    void addtoTuple( unsigned int* t, Cost c )  {
-    //        Value v0 = x->toValue(t[0]);
-    //        Value v1 = y->toValue(t[1]);
-    //        addcost( v0, v1, c );
-    //    }
-
     void fillElimConstr(EnumeratedVariable* xin, EnumeratedVariable* yin, Constraint* from1, Constraint* from2)
     {
         x = xin;
@@ -384,8 +488,12 @@ public:
             supportY.resize(sizeY);
         if (max(sizeX, sizeY) > trwsM.size())
             trwsM.resize(max(sizeX, sizeY), MIN_COST);
-        if ((unsigned long)sizeX * (unsigned long)sizeY > costs.size())
+        if ((size_t)sizeX * (size_t)sizeY > costs.size())
+#ifdef NO_STORE_BINARY_COSTS
+            costs.resize((size_t)sizeX * (size_t)sizeY, MIN_COST);
+#else
             costs.resize((size_t)sizeX * (size_t)sizeY, StoreCost(MIN_COST));
+#endif
         linkX->removed = true;
         linkY->removed = true;
         linkX->content.constr = this;
@@ -393,7 +501,7 @@ public:
         linkX->content.scopeIndex = 0;
         linkY->content.scopeIndex = 1;
         setDACScopeIndex();
-        resetConflictWeight();
+        resetConflictWeight(); // warning! it is done before having updated costs!!
         elimFrom(from1, from2);
     }
 
@@ -414,6 +522,8 @@ public:
 
     void propagate()
     {
+        if (ToulBar2::dumpWCSP % 2) // do not propagate if problem is dumped before preprocessing
+            return;
         if (x->assigned()) {
             assign(0);
             return;
@@ -477,8 +587,8 @@ public:
                 findFullSupportY();
         }
     }
-    //Trick! instead of doing remove(index) now, let AC queue do the job.
-    //So several incdec events on the same constraint can be merged into one AC event
+    // Trick! instead of doing remove(index) now, let AC queue do the job.
+    // So several incdec events on the same constraint can be merged into one AC event
     void increase(int index)
     {
         if (index == 0)
@@ -620,6 +730,7 @@ public:
     {
         int count = 0;
         double sum = 0;
+        double tight = -1;
         Cost* costs = new Cost[x->getDomainSize() * y->getDomainSize()];
         for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
             for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
@@ -656,7 +767,17 @@ public:
     void dump(ostream& os, bool original = true);
     void dump_CFN(ostream& os, bool original = true);
     Long size() const FINAL { return (Long)sizeX * sizeY; }
-    Long space() const FINAL { return (Long)sizeof(StoreCost) * sizeX * sizeY; }
+#ifdef NO_STORE_BINARY_COSTS
+    Long space() const FINAL
+    {
+        return (Long)sizeof(Cost) * sizeX * sizeY;
+    }
+#else
+    Long space() const FINAL
+    {
+        return (Long)sizeof(StoreCost) * sizeX * sizeY;
+    }
+#endif
 
     friend struct Functor_getCost;
     friend struct Functor_getCostReverse;
@@ -682,7 +803,7 @@ void BinaryConstraint::findSupport(T getCost, EnumeratedVariable* x, EnumeratedV
     assert(connected());
     wcsp->revise(this);
     if (ToulBar2::verbose >= 3)
-        cout << "findSupport C" << x->getName() << "," << y->getName() << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] findSupport C" << x->getName() << "," << y->getName() << endl;
     bool supportBroken = false;
     for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
         unsigned int xindex = x->toIndex(*iterX);
@@ -718,7 +839,7 @@ void BinaryConstraint::findFullSupport(T getCost, EnumeratedVariable* x, Enumera
     assert(connected());
     wcsp->revise(this);
     if (ToulBar2::verbose >= 3)
-        cout << "findFullSupport C" << x->getName() << "," << y->getName() << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] findFullSupport C" << x->getName() << "," << y->getName() << endl;
     bool supportBroken = false;
     for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
         unsigned int xindex = x->toIndex(*iterX);
