@@ -5,7 +5,9 @@
 #include "tb2ternaryconstr.hpp"
 #include "tb2enumvar.hpp"
 #include "tb2wcsp.hpp"
-#include <numeric>
+
+// TODO: avoid enumeration on all variables if they are already assigned and removed (using backtrackable domain data structure).
+// in order to speed-up propagate function if the support variable has a zero unary cost
 
 // warning! we assume binary variables
 class WeightedClause : public AbstractNaryConstraint {
@@ -27,8 +29,9 @@ class WeightedClause : public AbstractNaryConstraint {
         assert(lb <= cost);
         Constraint::projectLB(c);
     }
+    // Extends unaries on values that satisfy the clause inside the clause and project to c0
     void extend(Cost c)
-    {
+    { // extension of unaries before projecting to lb:
         for (int i = 0; i < arity_; i++) {
             EnumeratedVariable* x = scope[i];
             if (x->unassigned()) {
@@ -43,6 +46,7 @@ class WeightedClause : public AbstractNaryConstraint {
         }
         projectLB(c);
     }
+    // assignment of varIndex satisfies the clause
     void satisfied(int varIndex)
     {
         nonassigned = 0;
@@ -52,10 +56,10 @@ class WeightedClause : public AbstractNaryConstraint {
         for (int i = 0; i < arity_; i++) {
             EnumeratedVariable* x = scope[i];
             assert(deconnected(i));
-            if (i != varIndex) {
+            if (i != varIndex) { // the assigned satisfying variable is left as is. This accounts for the increase of the global lb in extend
                 Cost c = deltaCosts[i];
                 Value v = getClause(i);
-                if (c > MIN_COST) {
+                if (c > MIN_COST) { // we extended, now we give back
                     deltaCosts[i] = MIN_COST;
                     if (x->unassigned()) {
                         if (!CUT(c + wcsp->getLb(), wcsp->getUb())) {
@@ -113,7 +117,7 @@ public:
         return getDomainSizeProduct();
     }
 
-    void reconnect()
+    void reconnect() override
     {
         if (deconnected()) {
             nonassigned = arity_;
@@ -122,19 +126,26 @@ public:
     }
     int getNonAssigned() const { return nonassigned; }
 
-    Long getConflictWeight() const { return Constraint::getConflictWeight(); }
-    Long getConflictWeight(int varIndex) const
+    Long getConflictWeight() const override { return Constraint::getConflictWeight(); }
+    Long getConflictWeight(int varIndex) const override
     {
         assert(varIndex >= 0);
         assert(varIndex < arity_);
         return conflictWeights[varIndex] + Constraint::getConflictWeight();
     }
-    void incConflictWeight(Constraint* from)
+    void incConflictWeight(Constraint* from) override
     {
-        //assert(fromElim1==NULL);
-        //assert(fromElim2==NULL);
+        assert(from != NULL);
         if (from == this) {
-            Constraint::incConflictWeight(1);
+            if (deconnected() || nonassigned == arity_) {
+                Constraint::incConflictWeight(1);
+            } else {
+                for (int i = 0; i < arity_; i++) {
+                    if (connected(i)) {
+                        conflictWeights[i]++;
+                    }
+                }
+            }
         } else if (deconnected()) {
             for (int i = 0; i < from->arity(); i++) {
                 int index = getIndex(from->getVar(i));
@@ -145,14 +156,13 @@ public:
             }
         }
     }
-
-    void resetConflictWeight()
+    void resetConflictWeight() override
     {
         conflictWeights.assign(conflictWeights.size(), 0);
         Constraint::resetConflictWeight();
     }
 
-    bool universal()
+    bool universal() override
     {
         if (cost != MIN_COST || lb != MIN_COST)
             return false;
@@ -162,7 +172,7 @@ public:
         return true;
     }
 
-    Cost eval(const Tuple& s)
+    Cost eval(const Tuple& s) override
     {
         if (lb == MIN_COST && tuple[support] != s[support]) {
             assert(accumulate(deltaCosts.begin(), deltaCosts.end(), -lb) == MIN_COST);
@@ -215,9 +225,9 @@ public:
         return eval(evalTuple);
     }
 
-    double computeTightness() { return 1.0 * cost / getDomainSizeProduct(); }
+    double computeTightness() override { return 1.0 * cost / getDomainSizeProduct(); }
 
-    pair<pair<Cost, Cost>, pair<Cost, Cost>> getMaxCost(int index, Value a, Value b)
+    pair<pair<Cost, Cost>, pair<Cost, Cost>> getMaxCost(int index, Value a, Value b) override
     {
         Cost sumdelta = ((lb > MIN_COST) ? accumulate(deltaCosts.begin(), deltaCosts.end(), -lb) : MIN_COST);
         bool supporta = (getClause(index) == a);
@@ -226,14 +236,14 @@ public:
         return make_pair(make_pair(maxcosta, maxcosta), make_pair(maxcostb, maxcostb));
     }
 
-    void first()
+    void first() override
     {
         zeros = all_of(deltaCosts.begin(), deltaCosts.end(), [](Cost c) { return c == MIN_COST; });
         done = false;
         if (!zeros)
             firstlex();
     }
-    bool next(Tuple& t, Cost& c)
+    bool next(Tuple& t, Cost& c) override
     {
         if (!zeros)
             return nextlex(t, c);
@@ -245,7 +255,7 @@ public:
         return true;
     }
 
-    Cost getMaxFiniteCost()
+    Cost getMaxFiniteCost() override
     {
         Cost sumdelta = ((lb > MIN_COST) ? accumulate(deltaCosts.begin(), deltaCosts.end(), -lb) : MIN_COST);
         if (CUT(sumdelta, wcsp->getUb()))
@@ -255,14 +265,14 @@ public:
         else
             return max(sumdelta, cost - lb);
     }
-    void setInfiniteCost(Cost ub)
+    void setInfiniteCost(Cost ub) override
     {
-        Cost mult_ub = ((ub < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, ub * MEDIUM_COST)) : ub);
+        Cost mult_ub = ((wcsp->getUb() < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, wcsp->getUb() * MEDIUM_COST)) : wcsp->getUb());
         if (CUT(cost, ub))
             cost = mult_ub;
     }
 
-    void assign(int varIndex)
+    void assign(int varIndex) override
     {
         if (connected(varIndex)) {
             deconnect(varIndex);
@@ -271,6 +281,7 @@ public:
 
             if (scope[varIndex]->getValue() == getClause(varIndex)) {
                 deconnect();
+                wcsp->revise(this);
                 satisfied(varIndex);
                 return;
             }
@@ -286,8 +297,10 @@ public:
     }
 
     // propagates the minimum between the remaining clause weight and unary costs of all literals to the problem lower bound
-    void propagate()
+    void propagate() override
     {
+        if (ToulBar2::dumpWCSP % 2) // do not propagate if problem is dumped before preprocessing
+            return;
         Cost mincost = (connected() && scope[support]->unassigned()) ? scope[support]->getCost(getClause(support)) : MAX_COST;
         for (int i = 0; connected() && i < arity_; i++) {
             EnumeratedVariable* x = scope[i];
@@ -302,11 +315,12 @@ public:
             }
         }
         if (connected() && mincost < MAX_COST && mincost > MIN_COST && cost > lb) {
+            wcsp->revise(this);
             extend(min(cost - lb, mincost));
         }
     };
 
-    bool verify()
+    bool verify() override
     {
         Tuple t;
         Cost c;
@@ -317,10 +331,10 @@ public:
         }
         return false;
     }
-    void increase(int index) {}
-    void decrease(int index) {}
-    void remove(int index) {}
-    void projectFromZero(int index)
+    void increase(int index) override {}
+    void decrease(int index) override {}
+    void remove(int index) override {}
+    void projectFromZero(int index) override
     {
         if (index == support && cost > lb)
             propagate();
@@ -369,7 +383,7 @@ public:
         return result;
     }
 
-    void print(ostream& os)
+    void print(ostream& os) override
     {
         os << endl
            << this << " clause(";
@@ -401,7 +415,7 @@ public:
         os << " unassigned: " << (int)nonassigned << "/" << unassigned_ << endl;
     }
 
-    void dump(ostream& os, bool original = true)
+    void dump(ostream& os, bool original = true) override
     {
         Cost maxdelta = MIN_COST;
         for (vector<StoreCost>::iterator it = deltaCosts.begin(); it != deltaCosts.end(); ++it) {
@@ -416,7 +430,7 @@ public:
             if (maxdelta == MIN_COST) {
                 os << " " << 0 << " " << 1 << endl;
                 for (int i = 0; i < arity_; i++) {
-                    os << tuple[i] << " ";
+                    os << scope[i]->toValue(tuple[i]) << " ";
                 }
                 os << cost << endl;
             } else {
@@ -426,7 +440,7 @@ public:
                 firstlex();
                 while (nextlex(t, c)) {
                     for (int i = 0; i < arity_; i++) {
-                        os << t[i] << " ";
+                        os << scope[i]->toValue(t[i]) << " ";
                     }
                     os << c << endl;
                 }
@@ -459,7 +473,7 @@ public:
         }
     }
 
-    void dump_CFN(ostream& os, bool original = true)
+    void dump_CFN(ostream& os, bool original = true) override
     {
         bool printed = false;
         os << "\"F_";
@@ -484,10 +498,10 @@ public:
             for (int i = 0; i < arity_; i++) {
                 if (printed)
                     os << ",";
-                os << scope[i]->getName();
+                os << "\"" << scope[i]->getName() << "\"";
                 printed = true;
             }
-            os << "],\"defaultcost\":" << wcsp->Cost2RDCost(MIN_COST) << ",\n\"costs\":[";
+            os << "],\"defaultcost\":" << wcsp->DCost2Decimal(wcsp->Cost2RDCost(MIN_COST)) << ",\n\"costs\":[";
 
             if (maxdelta == MIN_COST) {
                 printed = false;
@@ -497,7 +511,7 @@ public:
                     os << ((scope[i]->isValueNames()) ? scope[i]->getValueName(tuple[i]) : std::to_string(tuple[i]));
                     printed = true;
                 }
-                os << "," << wcsp->Cost2RDCost(cost);
+                os << "," << wcsp->DCost2Decimal(wcsp->Cost2RDCost(cost));
             } else {
                 Tuple t;
                 Cost c;
@@ -510,8 +524,8 @@ public:
                             os << ",";
                         os << ((scope[i]->isValueNames()) ? scope[i]->getValueName(t[i]) : std::to_string(t[i]));
                         printed = true;
-                   }
-                   os << "," << wcsp->Cost2RDCost(c);
+                    }
+                    os << "," << wcsp->DCost2Decimal(wcsp->Cost2RDCost(c));
                 }
             }
         } else {
@@ -528,10 +542,10 @@ public:
                 if (scope[i]->unassigned()) {
                     if (printed)
                         os << ",";
-                    os << scope[i]->getName();
+                    os << "\"" << scope[i]->getName() << "\"";
                     printed = true;
                 }
-            os << "],\"defaultcost\":" << wcsp->Cost2RDCost(MIN_COST) << ",\n\"costs\":[";
+            os << "],\"defaultcost\":" << wcsp->DCost2Decimal(wcsp->Cost2RDCost(MIN_COST)) << ",\n\"costs\":[";
 
             if (maxdelta == MIN_COST) {
                 printed = false;
@@ -543,7 +557,7 @@ public:
                         printed = true;
                     }
                 }
-                os << "," << wcsp->Cost2RDCost(min(wcsp->getUb(), cost));
+                os << "," << ((wcsp->getUb() > cost) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(cost)) : "inf");
             } else {
                 Tuple t;
                 Cost c;
@@ -559,7 +573,7 @@ public:
                             printed = true;
                         }
                     }
-                    os << "," << wcsp->Cost2RDCost(min(wcsp->getUb(), c));
+                    os << "," << ((wcsp->getUb() > c) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(c)) : "inf");
                 }
             }
         }
