@@ -10,8 +10,6 @@
 #include "tb2clqcover.hpp"
 #include "search/tb2clusters.hpp"
 
-//#define PROPAGATE_CLIQUE_WITH_BINARIES
-
 int CliqueConstraint::nextid{ 0 };
 
 CliqueConstraint::CliqueConstraint(WCSP* wcsp, EnumeratedVariable** scope_in,
@@ -53,7 +51,7 @@ CliqueConstraint::CliqueConstraint(WCSP* wcsp, EnumeratedVariable** scope_in,
     id = nextid++;
     if (rhs != 1) {
         cout << "Unsupported: rhs == " << rhs << "\n";
-        exit(1);
+        throw WrongFileFormat();
     }
 }
 
@@ -96,6 +94,8 @@ std::ostream& CliqueConstraint::printstate(std::ostream& os)
 
 void CliqueConstraint::propagate()
 {
+    if (ToulBar2::dumpWCSP % 2) // do not propagate if problem is dumped before preprocessing
+        return;
     if (bc.empty()) {
         assert(Store::getDepth() == 0);
         initialize_binary();
@@ -447,7 +447,7 @@ Cost CliqueConstraint::get_zero_cost(int idx) // TODO remember last support and
         });
 }
 
-Cost CliqueConstraint::get_binary_zero_cost(int idx, int jdx) //TODO: avoid iterate if last support still valid and zero cost
+Cost CliqueConstraint::get_binary_zero_cost(int idx, int jdx) // TODO: avoid iterate if last support still valid and zero cost
 {
     EnumeratedVariable* x = scope[idx];
     EnumeratedVariable* y = scope[jdx];
@@ -568,7 +568,7 @@ BinaryConstraint* CliqueConstraint::project_binary_cost(int idx, int jdx, Cost c
             Value jval = y->toValue(jvalindex);
             if (!y->canbe(jval))
                 continue;
-            cons->addcost(x, y, ival, jval, c); //TODO: update deltas for tree decomposition-based methods
+            cons->addcost(x, y, ival, jval, c); // TODO: update deltas for tree decomposition-based methods
         }
     }
     return cons;
@@ -616,7 +616,7 @@ void CliqueConstraint::read(istream& is)
     }
     if (rhs != 1) {
         cerr << "Unsupported: rhs == " << rhs << " and should be set to one.\n";
-        exit(1);
+        throw WrongFileFormat();
     }
 }
 
