@@ -26,11 +26,11 @@ EnumeratedVariable::EnumeratedVariable(WCSP* w, string n, Value iinf, Value isup
     init();
 }
 
-EnumeratedVariable::EnumeratedVariable(WCSP* w, string n, Value* d, int dsize)
-    : Variable(w, n, min(d, dsize), max(d, dsize))
-    , domain(d, dsize)
+EnumeratedVariable::EnumeratedVariable(WCSP* w, string n, vector<Value>& dom)
+    : Variable(w, n, *min_element(dom.begin(), dom.end()), *max_element(dom.begin(), dom.end()))
+    , domain(dom)
     , deltaCost(MIN_COST)
-    , support(min(d, dsize))
+    , support(*min_element(dom.begin(), dom.end()))
     , watchForIncrease(false)
     , watchForDecrease(false)
 {
@@ -41,7 +41,7 @@ void EnumeratedVariable::init()
 {
     if (Store::getDepth() > 0) {
         cerr << "You cannot create a variable during the search!" << endl;
-        exit(EXIT_FAILURE);
+        throw InternalError();
     }
 
     costs = vector<StoreCost>(getDomainInitSize(), StoreCost(MIN_COST));
@@ -64,6 +64,13 @@ void EnumeratedVariable::init()
     queueDEE();
 }
 
+void EnumeratedVariable::getDomain(set<Value>& array)
+{
+    for (iterator iter = begin(); iter != end(); ++iter) {
+        array.insert(*iter);
+    }
+}
+
 void EnumeratedVariable::getDomain(Value* array)
 {
     for (iterator iter = begin(); iter != end(); ++iter) {
@@ -107,7 +114,13 @@ void EnumeratedVariable::print(ostream& os)
         for (iterator iter = begin(); iter != end(); ++iter) {
             os << " " << getCost(*iter);
         }
-        os << " > s:" << support;
+        os << " >";
+    }
+    if (ToulBar2::verbose >= 8) {
+        os << " /" << getDeltaCost();
+    }
+    if (unassigned()) {
+        os << " s:" << support;
         if (ToulBar2::FullEAC && isFullEAC()) {
             os << "!";
         }
@@ -190,7 +203,7 @@ void EnumeratedVariable::projectSupCost(Cost cost)
 
 void EnumeratedVariable::extend(Value value, Cost cost)
 {
-    assert(ToulBar2::verbose < 4 || ((cout << "extend " << getName() << " (" << value << ") -= " << cost << endl), true));
+    assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] extend " << getName() << " (" << value << ") -= " << cost << endl), true));
     assert(cost >= MIN_COST);
     assert(CUT(costs[toIndex(value)], cost));
     costs[toIndex(value)] -= cost;
@@ -259,7 +272,7 @@ void EnumeratedVariable::propagateNC()
 {
     wcsp->revise(NULL);
     if (ToulBar2::verbose >= 3)
-        cout << "propagateNC for " << getName() << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] propagateNC for " << getName() << endl;
     Value maxcostvalue = getSup() + 1;
     Cost maxcost = MIN_COST;
     bool supportBroken = false;
@@ -408,7 +421,7 @@ bool EnumeratedVariable::isEAC(Value a)
             if (!(*iter).constr->isEAC((*iter).scopeIndex, a)) {
 #ifndef NDEBUG
                 if (ToulBar2::verbose >= 4) {
-                    cout << getName() << "(" << a << ") is not EAC due to constraint " << *(*iter).constr << endl;
+                    cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] " << getName() << "(" << a << ") is not EAC due to constraint " << *(*iter).constr << endl;
                     if ((*iter).constr->isTernary()) {
                         TernaryConstraint* c = (TernaryConstraint*)(*iter).constr;
                         if (c->xy->connected())
@@ -428,13 +441,13 @@ bool EnumeratedVariable::isEAC(Value a)
         setSupport(a);
 #ifndef NDEBUG
         if (ToulBar2::verbose >= 4)
-            cout << getName() << "(" << a << ") is EAC!" << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] " << getName() << "(" << a << ") is EAC!" << endl;
 #endif
         return true;
     }
 #ifndef NDEBUG
     if (ToulBar2::verbose >= 4)
-        cout << getName() << "(" << a << ") is not EAC due to unary cost " << getCost(a) << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] " << getName() << "(" << a << ") is not EAC due to unary cost " << getCost(a) << endl;
 #endif
     return false;
 }
@@ -504,7 +517,7 @@ void EnumeratedVariable::propagateEAC()
 
 void EnumeratedVariable::propagateDEE(Value a, Value b, bool dee)
 {
-    //cout << "check DEE for " << *this << " " << a << " -> " << b << " " << dee << endl;
+    // cout << "check DEE for " << *this << " " << a << " -> " << b << " " << dee << endl;
     if (a == b)
         return;
     Cost costa = getCost(a);
@@ -577,28 +590,28 @@ void EnumeratedVariable::propagateDEE(Value a, Value b, bool dee)
     assert(totaldiffcosta <= costb || totaldiffcostb <= costa);
     if (totalmaxcosta == MIN_COST) {
         if (ToulBar2::verbose >= 2)
-            cout << "DEE " << *this << " (" << a << "," << totalmaxcosta << ") -> (*,*)" << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] DEE " << *this << " (" << a << "," << totalmaxcosta << ") -> (*,*)" << endl;
         wcsp->incNbDEE(getDomainSize() - 1);
         assign(a);
         return;
     }
     if (totalmaxcostb == MIN_COST) {
         if (ToulBar2::verbose >= 2)
-            cout << "DEE " << *this << " (" << b << "," << totalmaxcostb << ") -> (*,*)" << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] DEE " << *this << " (" << b << "," << totalmaxcostb << ") -> (*,*)" << endl;
         wcsp->incNbDEE(getDomainSize() - 1);
         assign(b);
         return;
     }
     if (totaldiffcosta <= costb) {
         if (ToulBar2::verbose >= 2)
-            cout << "DEE " << *this << " (" << a << "," << totaldiffcosta << ") -> (" << b << "," << costb << ")" << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] DEE " << *this << " (" << a << "," << totaldiffcosta << ") -> (" << b << "," << costb << ")" << endl;
         wcsp->incNbDEE();
         remove(b);
         if (assigned())
             return;
     } else {
         if (ToulBar2::verbose >= 2)
-            cout << "DEE " << *this << " (" << b << "," << totaldiffcostb << ") -> (" << a << "," << costa << ")" << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] DEE " << *this << " (" << b << "," << totaldiffcostb << ") -> (" << a << "," << costa << ")" << endl;
         wcsp->incNbDEE();
         remove(a);
         if (assigned())
@@ -616,7 +629,7 @@ void EnumeratedVariable::propagateDEE(Value a, Value b, bool dee)
         assert(unassigned());
         if (getCost(*iter) >= totalmaxcost) {
             if (ToulBar2::verbose >= 2)
-                cout << "DEE " << *this << " (" << ((totalmaxcosta < totalmaxcostb) ? a : b) << "," << totalmaxcost << ") -> (" << *iter << "," << getCost(*iter) << ")" << endl;
+                cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] DEE " << *this << " (" << ((totalmaxcosta < totalmaxcostb) ? a : b) << "," << totalmaxcost << ") -> (" << *iter << "," << getCost(*iter) << ")" << endl;
             wcsp->incNbDEE();
             remove(*iter);
         }
@@ -680,7 +693,7 @@ bool EnumeratedVariable::verifyDEE()
 void EnumeratedVariable::increaseFast(Value newInf)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "increase " << getName() << " " << inf << " -> " << newInf << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] increase " << getName() << " " << inf << " -> " << newInf << endl;
     assert(!wcsp->getIsPartOfOptimalSolution() || ((wcsp->getTreeDec()) ? wcsp->getTreeDec()->getRoot()->getUb() : wcsp->getUb()) <= ToulBar2::verifiedOptimum || wcsp->getBestValue(wcspIndex) >= newInf);
     if (newInf > inf) {
         if (newInf > sup) {
@@ -709,7 +722,7 @@ void EnumeratedVariable::increaseFast(Value newInf)
 void EnumeratedVariable::increase(Value newInf, bool isDecision)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "increase " << getName() << " " << inf << " -> " << newInf << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] increase " << getName() << " " << inf << " -> " << newInf << endl;
 #ifndef NDEBUG
     if (isDecision && wcsp->getIsPartOfOptimalSolution() && wcsp->getBestValue(wcspIndex) < newInf)
         wcsp->setIsPartOfOptimalSolution(false);
@@ -744,7 +757,7 @@ void EnumeratedVariable::increase(Value newInf, bool isDecision)
 void EnumeratedVariable::decreaseFast(Value newSup)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "decrease " << getName() << " " << sup << " -> " << newSup << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] decrease " << getName() << " " << sup << " -> " << newSup << endl;
     assert(!wcsp->getIsPartOfOptimalSolution() || ((wcsp->getTreeDec()) ? wcsp->getTreeDec()->getRoot()->getUb() : wcsp->getUb()) <= ToulBar2::verifiedOptimum || wcsp->getBestValue(wcspIndex) <= newSup);
     if (newSup < sup) {
         if (newSup < inf) {
@@ -773,7 +786,7 @@ void EnumeratedVariable::decreaseFast(Value newSup)
 void EnumeratedVariable::decrease(Value newSup, bool isDecision)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "decrease " << getName() << " " << sup << " -> " << newSup << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] decrease " << getName() << " " << sup << " -> " << newSup << endl;
 #ifndef NDEBUG
     if (isDecision && wcsp->getIsPartOfOptimalSolution() && wcsp->getBestValue(wcspIndex) > newSup)
         wcsp->setIsPartOfOptimalSolution(false);
@@ -808,7 +821,7 @@ void EnumeratedVariable::decrease(Value newSup, bool isDecision)
 void EnumeratedVariable::removeFast(Value value)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "remove " << *this << " <> " << value << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] remove " << *this << " <> " << value << endl;
     assert(!wcsp->getIsPartOfOptimalSolution() || ((wcsp->getTreeDec()) ? wcsp->getTreeDec()->getRoot()->getUb() : wcsp->getUb()) <= ToulBar2::verifiedOptimum || wcsp->getBestValue(wcspIndex) != value);
     if (value == inf)
         increaseFast(value + 1);
@@ -829,7 +842,7 @@ void EnumeratedVariable::removeFast(Value value)
 void EnumeratedVariable::remove(Value value, bool isDecision)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "remove " << *this << " <> " << value << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] remove " << *this << " <> " << value << endl;
 #ifndef NDEBUG
     if (isDecision && wcsp->getIsPartOfOptimalSolution() && wcsp->getBestValue(wcspIndex) == value)
         wcsp->setIsPartOfOptimalSolution(false);
@@ -865,10 +878,17 @@ void EnumeratedVariable::assignWhenEliminated(Value newValue)
     maxCost = MIN_COST;
 }
 
+void EnumeratedVariable::restoreInitialDomainWhenEliminated()
+{
+    assert(assigned());
+    inf = *(domain.begin());
+    sup = *(domain.rbegin());
+}
+
 void EnumeratedVariable::assign(Value newValue, bool isDecision)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "assign " << *this << " -> " << newValue << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] assign " << *this << " -> " << newValue << endl;
 #ifndef NDEBUG
     if (isDecision && wcsp->getIsPartOfOptimalSolution() && wcsp->getBestValue(wcspIndex) != newValue)
         wcsp->setIsPartOfOptimalSolution(false);
@@ -904,7 +924,7 @@ void EnumeratedVariable::assign(Value newValue, bool isDecision)
 void EnumeratedVariable::assignLS(Value newValue, ConstraintSet& delayedCtrs, bool force)
 {
     if (ToulBar2::verbose >= 2)
-        cout << "assignLS " << *this << " -> " << newValue << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] assignLS " << *this << " -> " << newValue << endl;
 #ifndef NDEBUG
     if (wcsp->getIsPartOfOptimalSolution() && wcsp->getBestValue(wcspIndex) != newValue)
         wcsp->setIsPartOfOptimalSolution(false);
@@ -933,25 +953,48 @@ void EnumeratedVariable::assignLS(Value newValue, ConstraintSet& delayedCtrs, bo
     }
 }
 
-// eliminates the current (this) variable that participates
-// in a single binary constraint ctr
-bool EnumeratedVariable::elimVar(BinaryConstraint* ctr)
+void EnumeratedVariable::shrink()
 {
-    assert(getDegree() == 1);
+    if (toIndex(inf) > 0) {
+        for (int i = 0; i < sup - inf + 1; i++) {
+            costs[i] = costs[toIndex(inf + i)];
+            valueNames[i] = valueNames[toIndex(inf + i)];
+        }
+    }
+    costs.resize(sup - inf + 1);
+    valueNames.resize(sup - inf + 1);
+    domain.shrink(inf, sup);
+}
+
+// eliminates the current (this) variable that participates
+// in a single binary constraint ctr (plus a duplicated one if any)
+bool EnumeratedVariable::elimVar(BinaryConstraint* ctr, BinaryConstraint* ctr_duplicate)
+{
+    assert((getDegree() == 1 && !ctr_duplicate) || (getDegree() == 2 && ctr_duplicate && wcsp->getTreeDec()));
 
     EnumeratedVariable* x = (EnumeratedVariable*)ctr->getVarDiffFrom(this);
+    if (x->getName().rfind(HIDDEN_VAR_TAG_HVE_PRE, 0) == 0) {
+        return false;
+    }
 
     TreeDecomposition* td = wcsp->getTreeDec();
-    if (td && cluster != ctr->getCluster())
+    if (td && !td->isSameCluster(cluster, ctr->getCluster())) {
         return false;
+    }
 
-    if (ToulBar2::verbose >= 2)
+    if (ToulBar2::verbose >= 2) {
         cout << "   elim linked to one binary " << ctr << endl;
+        if (ctr_duplicate) {
+            cout << "   and a second duplicated one " << ctr_duplicate << endl;
+        }
+    }
 
     // deconnect first to be sure the current var is not involved in future propagation
     ctr->deconnect();
+    if (ctr_duplicate)
+        ctr_duplicate->deconnect();
     // to be done before propagation
-    WCSP::elimInfo ei = { this, x, NULL, ctr, NULL, NULL, NULL };
+    WCSP::elimInfo ei = { this, x, ctr_duplicate ? x : NULL, ctr, ctr_duplicate, NULL, NULL };
     wcsp->elimInfos[wcsp->getElimOrder()] = ei;
     wcsp->elimOrderInc();
 
@@ -965,7 +1008,7 @@ bool EnumeratedVariable::elimVar(BinaryConstraint* ctr)
     for (iterator iter1 = x->begin(); iter1 != x->end(); ++iter1) {
         Cost mincost = MAX_COST;
         for (iterator iter = begin(); iter != end(); ++iter) {
-            Cost curcost = getCost(*iter) + getBinaryCost(ctr, *iter, *iter1);
+            Cost curcost = getCost(*iter) + getBinaryCost(ctr, *iter, *iter1) + ((ctr_duplicate) ? getBinaryCost(ctr_duplicate, *iter, *iter1) : MIN_COST);
             if (ToulBar2::isZ)
                 mincost = wcsp->LogSumExp(mincost, curcost);
             else if (curcost < mincost)
@@ -1012,14 +1055,23 @@ bool EnumeratedVariable::elimVar(BinaryConstraint* ctr)
 bool EnumeratedVariable::elimVar(ConstraintLink xylink, ConstraintLink xzlink)
 {
     EnumeratedVariable* y = (EnumeratedVariable*)wcsp->getVar(xylink.constr->getSmallestVarIndexInScope(xylink.scopeIndex));
+    if (y->getName().rfind(HIDDEN_VAR_TAG_HVE_PRE, 0) == 0) {
+        return false;
+    }
     EnumeratedVariable* z = (EnumeratedVariable*)wcsp->getVar(xzlink.constr->getSmallestVarIndexInScope(xzlink.scopeIndex));
+    if (z->getName().rfind(HIDDEN_VAR_TAG_HVE_PRE, 0) == 0) {
+        return false;
+    }
+    assert(y != z);
 
     TreeDecomposition* td = wcsp->getTreeDec();
     if (td) {
         if (y->isSep() && z->isSep())
             return false;
-        if ((cluster != xylink.constr->getCluster()) || (cluster != xzlink.constr->getCluster()) || (xylink.constr->getCluster() != xzlink.constr->getCluster()))
+        if (!td->isSameCluster(cluster, xylink.constr->getCluster()) || !td->isSameCluster(cluster, xzlink.constr->getCluster())) {
+            assert(!td->isSameCluster(xylink.constr->getCluster(), xzlink.constr->getCluster()));
             return false;
+        }
     }
 
     assert(getDegree() == 2);
@@ -1028,7 +1080,7 @@ bool EnumeratedVariable::elimVar(ConstraintLink xylink, ConstraintLink xzlink)
 
     BinaryConstraint* yz = y->getConstr(z);
 
-    if (td && yz && (getCluster() != yz->getCluster())) {
+    if (td && yz && !td->isSameCluster(getCluster(), yz->getCluster())) {
         BinaryConstraint* yz_ = y->getConstr(z, getCluster());
         if (yz_) {
             yz = yz_;
@@ -1069,9 +1121,10 @@ bool EnumeratedVariable::elimVar(ConstraintLink xylink, ConstraintLink xzlink)
     }
 
     if (yz) {
-        if (td && yz->getCluster() != cluster) {
+        if (td && !td->isSameCluster(yz->getCluster(), cluster)) {
             yz = yznew;
             yz->reconnect();
+            yz->setCluster(getCluster());
             yz->setDuplicate();
         } else {
             yz->addCosts(yznew);
@@ -1080,11 +1133,12 @@ bool EnumeratedVariable::elimVar(ConstraintLink xylink, ConstraintLink xzlink)
         }
     } else {
         yz = yznew;
+        yz->setCluster(getCluster());
         yz->reconnect();
     }
 
-    if (td)
-        yz->setCluster(getCluster());
+    assert(!td || td->isSameCluster(yz->getCluster(), cluster));
+
     // to be done before propagation
     WCSP::elimInfo ei = { this, y, z, (BinaryConstraint*)xylink.constr, (BinaryConstraint*)xzlink.constr, NULL, NULL };
     wcsp->elimInfos[wcsp->getElimOrder()] = ei;
@@ -1123,11 +1177,11 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
     }
 
     if (n3links > 1)
-        return false;
+        return false; // TODO: continue if it is a duplicated ternary with the same scope as xyz
 
-    if (wcsp->getTreeDec()) {
-        if (cluster != xyz->getCluster())
-            return false;
+    TreeDecomposition* td = wcsp->getTreeDec();
+    if (td && (!td->isSameCluster(cluster, xyz->getCluster()) || (n2links > 0 && !td->isSameCluster(cluster, links[0].constr->getCluster())) || (n2links > 1 && !td->isSameCluster(cluster, links[1].constr->getCluster())))) {
+        return false;
     }
 
     for (int i = 0; i < n2links; i++) {
@@ -1139,11 +1193,19 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
     xyz->deconnect();
     if (n2links > 0)
         links[0].constr->deconnect();
-    if (n2links > 1)
+    if (n2links > 1) {
+        assert(links[0].constr != links[1].constr);
         links[1].constr->deconnect();
+    }
 
     EnumeratedVariable* y = (EnumeratedVariable*)yz->getVar(0);
+    if (y->getName().rfind(HIDDEN_VAR_TAG_HVE_PRE, 0) == 0) {
+        return false;
+    }
     EnumeratedVariable* z = (EnumeratedVariable*)yz->getVar(1);
+    if (z->getName().rfind(HIDDEN_VAR_TAG_HVE_PRE, 0) == 0) {
+        return false;
+    }
 
     bool flag_rev = false;
     if (n2links > 0) {
@@ -1152,6 +1214,13 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
             flag_rev = true;
         }
     }
+    bool duplicated = false;
+    if (n2links > 1) {
+        if (links[0].constr->getSmallestVarIndexInScope(links[0].scopeIndex) == links[1].constr->getSmallestVarIndexInScope(links[1].scopeIndex)) {
+            assert(links[0].constr->isDuplicate() || links[1].constr->isDuplicate());
+            duplicated = true;
+        }
+    }
 
     Cost negcost = MIN_COST;
     for (iterator itery = y->begin(); itery != y->end(); ++itery) {
@@ -1166,8 +1235,13 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
                         curcost += getBinaryCost(links[0], *iter, *itery);
                     }
                     if (n2links > 1) {
-                        assert(links[1].constr->getIndex(z) >= 0);
-                        curcost += getBinaryCost(links[1], *iter, *iterz);
+                        if (duplicated) {
+                            assert(links[1].constr->getIndex(y) >= 0);
+                            curcost += getBinaryCost(links[1], *iter, *itery);
+                        } else {
+                            assert(links[1].constr->getIndex(z) >= 0);
+                            curcost += getBinaryCost(links[1], *iter, *iterz);
+                        }
                     }
                 } else {
                     if (n2links > 0) {
@@ -1175,8 +1249,13 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
                         curcost += getBinaryCost(links[0], *iter, *iterz);
                     }
                     if (n2links > 1) {
-                        assert(links[1].constr->getIndex(y) >= 0);
-                        curcost += getBinaryCost(links[1], *iter, *itery);
+                        if (duplicated) {
+                            assert(links[1].constr->getIndex(z) >= 0);
+                            curcost += getBinaryCost(links[1], *iter, *iterz);
+                        } else {
+                            assert(links[1].constr->getIndex(y) >= 0);
+                            curcost += getBinaryCost(links[1], *iter, *itery);
+                        }
                     }
                 }
                 if (ToulBar2::isZ)
@@ -1202,6 +1281,8 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
     if (y->unassigned() && z->unassigned())
         yz->reconnect();
 
+    assert(!td || td->isSameCluster(yz->getCluster(), cluster));
+
     // to be done before propagation
     WCSP::elimInfo ei = { this, y, z, (BinaryConstraint*)links[(flag_rev) ? 1 : 0].constr, (BinaryConstraint*)links[(flag_rev) ? 0 : 1].constr, xyz, NULL };
     wcsp->elimInfos[wcsp->getElimOrder()] = ei;
@@ -1212,18 +1293,18 @@ bool EnumeratedVariable::elimVar(TernaryConstraint* xyz)
 
 void EnumeratedVariable::eliminate()
 {
-    if (isSep_)
+    if (isSep())
         return;
     if (ToulBar2::nbDecisionVars > 0 && wcspIndex < ToulBar2::nbDecisionVars)
         return;
-    if (ToulBar2::allSolutions && ToulBar2::btdMode != 1 && getName()[0] != IMPLICIT_VAR_TAG[0])
+    if (ToulBar2::allSolutions && ToulBar2::btdMode != 1 && getName().rfind(IMPLICIT_VAR_TAG, 0) != 0)
         return;
-    if (ToulBar2::divNbSol > 1 && getName()[0] != IMPLICIT_VAR_TAG[0] && Store::getDepth() == 0)
+    if (ToulBar2::divNbSol > 1 && getName().rfind(IMPLICIT_VAR_TAG, 0) != 0 && Store::getDepth() == 0) // do not eliminate decision variables nor diversity variables before generating diversity constraints
         return;
 
     assert(!wcsp->getTreeDec() || wcsp->getTreeDec()->getCluster(cluster)->isActive());
 
-    if (ToulBar2::elimDegree_preprocessing_ >= 0 && (getDegree() <= min(1, ToulBar2::elimDegree_preprocessing_) || getTrueDegree() <= ToulBar2::elimDegree_preprocessing_)) {
+    if (ToulBar2::elimDegree_preprocessing_ >= 0 && (getDegree() <= min(1, ToulBar2::elimDegree_preprocessing_) || getTrueDegree() <= ToulBar2::elimDegree_preprocessing_) && !isGlobal()) {
         if (ToulBar2::elimSpaceMaxMB && (Double)wcsp->elimSpace + (sizeof(tValue) * (getTrueDegree() + 1) + sizeof(Cost)) * getMaxElimSize() > (Double)ToulBar2::elimSpaceMaxMB * 1024. * 1024.) {
             if (ToulBar2::verbose >= 1)
                 cout << "Generic variable elimination of " << getName() << " stopped (" << (Double)wcsp->elimSpace / 1024. / 1024. << " + " << (Double)(sizeof(tValue) * (getTrueDegree() + 1) + sizeof(Cost)) * getMaxElimSize() / 1024. / 1024. << " >= " << ToulBar2::elimSpaceMaxMB << " MB)" << endl;
@@ -1259,7 +1340,9 @@ void EnumeratedVariable::eliminate()
             TernaryConstraint* ternCtr = existTernary();
 
             if (ternCtr) {
+#ifndef NO_STORE_BINARY_COSTS
                 if (!elimVar(ternCtr))
+#endif
                     return;
             } else {
                 if (getDegree() > 2)
@@ -1268,16 +1351,26 @@ void EnumeratedVariable::eliminate()
                 ConstraintLink xylink = *constrs.begin();
                 ConstraintLink xzlink = { NULL, 0 };
 
-                if (xylink.constr->arity() > 2 || !(xylink.constr->extension()))
+                if (!xylink.constr->isBinary())
                     return;
 
                 if (getDegree() == 2) {
                     xzlink = *constrs.rbegin();
-                    if (xzlink.constr->arity() > 2 || !(xzlink.constr->extension()))
-                        return;
-
-                    if (!elimVar(xylink, xzlink))
+                    if (!xzlink.constr->isBinary())
                         return;
+                    assert(xylink.constr != xzlink.constr);
+                    bool duplicated = (xylink.constr->getSmallestVarIndexInScope(xylink.scopeIndex) == xzlink.constr->getSmallestVarIndexInScope(xzlink.scopeIndex));
+                    if (duplicated) {
+                        BinaryConstraint* xy = (BinaryConstraint*)xylink.constr;
+                        BinaryConstraint* xy_duplicate = (BinaryConstraint*)xzlink.constr;
+                        if (!elimVar(xy, xy_duplicate))
+                            return;
+                    } else {
+#ifndef NO_STORE_BINARY_COSTS
+                        if (!elimVar(xylink, xzlink))
+#endif
+                            return;
+                    }
                 } else {
                     BinaryConstraint* xy = (BinaryConstraint*)xylink.constr;
                     if (!elimVar(xy))
@@ -1299,7 +1392,7 @@ void EnumeratedVariable::eliminate()
     }
     assert(getDegree() == 0);
     if (ToulBar2::verbose >= 2)
-        cout << "Eliminate End of var " << getName() << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] Eliminate End of var " << getName() << endl;
     assert(getCost(support) == MIN_COST); // it is ensured by previous calls to findSupport
     assign(support); // warning! dummy assigned value
 }
@@ -1325,7 +1418,7 @@ void EnumeratedVariable::permuteDomain(Value a, Value b)
             bctr->permute(this, a, b);
         } else {
             cerr << "Sorry! cannot sort domains with non-binary cost functions!" << endl;
-            exit(EXIT_FAILURE);
+            throw BadConfiguration();
         }
     }
 }
@@ -1412,14 +1505,14 @@ bool EnumeratedVariable::canbeMerged(EnumeratedVariable* x)
 {
     if (ToulBar2::nbDecisionVars > 0 && wcspIndex < ToulBar2::nbDecisionVars)
         return false;
-    if (ToulBar2::allSolutions && getName()[0] != IMPLICIT_VAR_TAG[0] && (ToulBar2::elimDegree >= 0 || ToulBar2::elimDegree_preprocessing >= 0))
+    if (ToulBar2::allSolutions && getName().rfind(IMPLICIT_VAR_TAG, 0) != 0 && (ToulBar2::elimDegree >= 0 || ToulBar2::elimDegree_preprocessing >= 0))
         return false;
-    if (ToulBar2::divNbSol > 1 && getName()[0] != IMPLICIT_VAR_TAG[0] && Store::getDepth() == 0)
+    if (ToulBar2::divNbSol > 1 && getName().rfind(IMPLICIT_VAR_TAG, 0) != 0 && Store::getDepth() == 0) // do not eliminate decision variables nor diversity variables before generating diversity constraints
         return false;
     double mult = (1.0 * x->getDomainSize()) / getDomainSize();
     for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
         Constraint* ctr = (*iter).constr;
-        if (ctr->isGlobal() || ctr->isSep() || (!ctr->extension() && Store::getDepth() > 0))
+        if (ctr->isGlobal() || ctr->isSep() || (!ctr->extension() && (Store::getDepth() > 0 || ctr->size() > 1))) // avoid functional elimination if diverse solutions or not a clause
             return false;
         if (mult > 1.1 && (mult * ctr->size() > MAX_NB_TUPLES))
             return false;
@@ -1467,7 +1560,7 @@ void EnumeratedVariable::mergeTo(BinaryConstraint* xy, map<Value, Value>& functi
         switch (scopeSize) {
         case 0: {
             cerr << "Error: empty scope from " << *ctr << " when merging functional variable " << *this << " to variable " << *x << endl;
-            exit(EXIT_FAILURE);
+            throw InternalError();
             break;
         }
         case 1: {
@@ -1490,8 +1583,6 @@ void EnumeratedVariable::mergeTo(BinaryConstraint* xy, map<Value, Value>& functi
             break;
         }
         case 2: {
-            assert(wcsp->unassigned(scopeIndex[0]));
-            assert(wcsp->unassigned(scopeIndex[1]));
             EnumeratedVariable* u = (EnumeratedVariable*)wcsp->getVar(scopeIndex[0]);
             EnumeratedVariable* v = (EnumeratedVariable*)wcsp->getVar(scopeIndex[1]);
             assert(x == u || x == v);
@@ -1520,10 +1611,6 @@ void EnumeratedVariable::mergeTo(BinaryConstraint* xy, map<Value, Value>& functi
             break;
         }
         case 3: {
-            assert(wcsp->unassigned(scopeIndex[0]));
-            assert(wcsp->unassigned(scopeIndex[1]));
-            assert(wcsp->unassigned(scopeIndex[2]));
-
             EnumeratedVariable* u = (EnumeratedVariable*)wcsp->getVar(scopeIndex[0]);
             EnumeratedVariable* v = (EnumeratedVariable*)wcsp->getVar(scopeIndex[1]);
             EnumeratedVariable* w = (EnumeratedVariable*)wcsp->getVar(scopeIndex[2]);
@@ -1545,7 +1632,7 @@ void EnumeratedVariable::mergeTo(BinaryConstraint* xy, map<Value, Value>& functi
                         Cost cost = ctr->evalsubstr(oldtuple, ctr);
                         if (cost > MIN_COST) {
                             empty = false;
-                            costs[u->toIndex(*iterU) * v->getDomainInitSize() * w->getDomainInitSize() + v->toIndex(*iterV) * w->getDomainInitSize() + w->toIndex(*iterW)] = cost;
+                            costs[(size_t)u->toIndex(*iterU) * (size_t)v->getDomainInitSize() * (size_t)w->getDomainInitSize() + (size_t)v->toIndex(*iterV) * (size_t)w->getDomainInitSize() + (size_t)w->toIndex(*iterW)] = cost;
                         }
                     }
                 }
@@ -1660,13 +1747,16 @@ void EnumeratedVariable::mergeTo(BinaryConstraint* xy, map<Value, Value>& functi
     elimVar(xy);
     assert(getDegree() == 0);
     assert(getCost(support) == MIN_COST);
+    for (iterator iter = begin(); iter != end(); ++iter) { // clear all unary costs because they are already projected on variable x
+        costs[toIndex(*iter)] = deltaCost;
+    }
     assign(support); // warning! dummy assigned value
 }
 
 bool EnumeratedVariable::verify()
 {
     TreeDecomposition* td = wcsp->getTreeDec();
-    if (!td)
+    if (!td || (ToulBar2::heuristicFreedom && Store::getDepth() > 0))
         return true;
     for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
         Constraint* ctr1 = (*iter).constr;
