@@ -16,7 +16,7 @@ NaryConstraint::NaryConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int ar
         unsigned int domsize = scope_in[i]->getDomainInitSize();
         if (domsize > (unsigned int)std::numeric_limits<tValue>::max()) {
             cerr << "Nary constraints overflow. Extend tValue type range." << endl;
-            exit(EXIT_FAILURE);
+            throw BadConfiguration();
         }
     }
 
@@ -66,7 +66,7 @@ void NaryConstraint::assign(int varIndex)
             return;
         }
 
-        if (nonassigned <= NARYPROJECTIONSIZE) {
+        if (nonassigned <= NARYPROJECTIONSIZE && (nonassigned < 3 || maxInitDomSize <= NARYPROJECTION3MAXDOMSIZE)) {
             //	  cout << "Assign var " << *getVar(varIndex) << "  in  " << *this;
             deconnect();
             projectNary();
@@ -78,7 +78,7 @@ void NaryConstraint::assign(int varIndex)
 }
 
 // NOT USED
-//void NaryConstraint::projectFromZero(int index)
+// void NaryConstraint::projectFromZero(int index)
 //{
 //    int i;
 //    int nsup = 0;
@@ -116,7 +116,7 @@ void NaryConstraint::assign(int varIndex)
 //}
 
 // NOT USED
-//void NaryConstraint::starrule(const Tuple& t, Cost minc)
+// void NaryConstraint::starrule(const Tuple& t, Cost minc)
 //{
 //    int i;
 //    for(i=0;i<arity_;i++) {
@@ -363,7 +363,7 @@ bool NaryConstraint::separability(EnumeratedVariable* vx, EnumeratedVariable* vz
 
     scope_in[a - 1] = vz; //.push_back( beta );
     ordre[a - 2] = a - 1;
-    //if(it_values[ordre[a-2]] != vz->end()) ++it_values[ordre[a-2]];
+    // if(it_values[ordre[a-2]] != vz->end()) ++it_values[ordre[a-2]];
     if (ToulBar2::verbose >= 1)
         cout << vz->getName() << " ] ? "; // << endl;
 
@@ -372,7 +372,7 @@ bool NaryConstraint::separability(EnumeratedVariable* vx, EnumeratedVariable* vz
         while (sev && it_values[ordre[0]] != scope_in[ordre[0]]->end()) {
             if (it_values[ordre[a - 2]] == itvzfirst /*vz->begin()*/)
                 ++it_values[ordre[a - 2]];
-            //if(it_values[ordre[a-2]] != vz->end()){
+            // if(it_values[ordre[a-2]] != vz->end()){
             for (int j = 0; j < a; j++) {
                 t[j] = scope_in[j]->toIndex(*it_values[j]); // PROBLEME CAR it_values[ordre[a-2]] = vz->end()
                 t1[j] = scope_in[j]->toIndex(*it_values[j]);
@@ -417,7 +417,7 @@ bool NaryConstraint::separability(EnumeratedVariable* vx, EnumeratedVariable* vz
                 ++it_values[ordre[i]];
                 finished = it_values[ordre[i]] != var->end();
                 if (!finished) {
-                    //it_values[ordre[i]] = var->begin();
+                    // it_values[ordre[i]] = var->begin();
                     if (i > 0) {
                         it_values[ordre[i]] = var->begin();
                         if (i == a - 1)
@@ -581,7 +581,7 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
             t[j] = scope_in[j]->toIndex(*it_values[j]);
             tZ[j - 1] = scope_in[j]->toIndex(*it_values[j]);
         }
-        //t[0] = scope_in[0]->toIndex(*it_values[0]);
+        // t[0] = scope_in[0]->toIndex(*it_values[0]);
         EnumeratedVariable::iterator it = scope_in[0]->begin();
         do {
             t[0] = scope_in[0]->toIndex(*it);
@@ -589,7 +589,7 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
             cX = naryx->evalsubstr(t, naryx);
             diffcost = squareminus(cost, cX, wcsp->getUb());
             ++it;
-            //cout << t[0] << endl;
+            // cout << t[0] << endl;
         } while (it != scope_in[0]->end() && cost >= wcsp->getUb() && cX >= wcsp->getUb());
 
         cost = eval(t, scope_in);
@@ -602,6 +602,7 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
 
             cout << "  " << diffcost << endl;
         }
+        assert(diffcost >= MIN_COST);
         naryz->setTuple(tZ, diffcost);
         int finished = false;
         i = a - 1;
@@ -681,7 +682,7 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
 }
 
 // THIS CODE IS NEVER USED!!!
-//void NaryConstraint::permute( EnumeratedVariable** scope_in )
+// void NaryConstraint::permute( EnumeratedVariable** scope_in )
 //{
 //    TUPLES* pf_old = pf;
 //    pf = new TUPLES;
@@ -714,7 +715,7 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
 
 // for adding a tuple in f
 // scope_in contains the order of the values in Tuple tin
-//void NaryConstraint::setTuple( const Tuple& tin, Cost c, EnumeratedVariable** scope_in )
+// void NaryConstraint::setTuple( const Tuple& tin, Cost c, EnumeratedVariable** scope_in )
 //{
 //    assert(scope_in);
 //    Tuple t(tin);
@@ -726,17 +727,17 @@ void NaryConstraint::separate(EnumeratedVariable* vx, EnumeratedVariable* vz)
 //    else costs[getCostsIndex(t)] = c;
 //}
 
-//void NaryConstraint::addtoTuple( const Tuple& tin, Cost c, EnumeratedVariable** scope_in )
+// void NaryConstraint::addtoTuple( const Tuple& tin, Cost c, EnumeratedVariable** scope_in )
 //{
-//    assert(scope_in);
-//    Tuple t(tin);
-//    for(int i = 0; i < arity_; i++) {
-//        int pos = getIndex(scope_in[i]);
-//        t[pos] = tin[i];
-//    }
-//    if (pf) (*pf)[t] += c;
-//    else costs[getCostsIndex(t)] += c;
-//}
+//     assert(scope_in);
+//     Tuple t(tin);
+//     for(int i = 0; i < arity_; i++) {
+//         int pos = getIndex(scope_in[i]);
+//         t[pos] = tin[i];
+//     }
+//     if (pf) (*pf)[t] += c;
+//     else costs[getCostsIndex(t)] += c;
+// }
 
 void NaryConstraint::addtoTuples(EnumeratedVariable* x, Value v, Cost costi)
 {
@@ -791,16 +792,16 @@ void NaryConstraint::addtoTuples(Cost costi)
 
 void NaryConstraint::setInfiniteCost(Cost ub)
 {
-    Cost mult_ub = ((ub < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, ub * MEDIUM_COST)) : ub);
+    Cost mult_ub = ((wcsp->getUb() < (MAX_COST / MEDIUM_COST)) ? (max(LARGE_COST, wcsp->getUb() * MEDIUM_COST)) : wcsp->getUb());
     if (pf) {
         for (TUPLES::iterator it = pf->begin(); it != pf->end(); ++it) {
-            Cost c = it->second;
+            Cost c = it->second; // TODO: add corresponding unary costs
             if (CUT(c, ub))
                 it->second = mult_ub;
         }
     } else {
         for (ptrdiff_t idx = 0; idx < costSize; idx++) {
-            Cost c = costs[idx];
+            Cost c = costs[idx]; // TODO: add corresponding unary costs
             if (CUT(c, ub))
                 costs[idx] = mult_ub;
         }
@@ -867,8 +868,8 @@ void NaryConstraint::insertSum(const Tuple& t1, Cost c1, Constraint* ctr1, const
 }
 
 // THIS CODE IS NEVER USED!!!
-//TODO: make it compatible with table representation rather than Map
-//void NaryConstraint::sum( NaryConstraint* nary )
+// TODO: make it compatible with table representation rather than Map
+// void NaryConstraint::sum( NaryConstraint* nary )
 //{
 //    deconnect(true);
 //
@@ -1104,7 +1105,7 @@ void NaryConstraint::project(EnumeratedVariable* x)
         DLink<ConstraintLink>* linkx = links[xindex];
         links[xindex] = links[arity_ - 1];
         links[arity_ - 1] = linkx;
-        //swap of scope array
+        // swap of scope array
         scope[xindex] = scope[arity_ - 1];
         scope[arity_ - 1] = x;
         // update of links indexs
@@ -1112,6 +1113,17 @@ void NaryConstraint::project(EnumeratedVariable* x)
         links[arity_ - 1]->content.scopeIndex = arity_ - 1;
         scope_inv[scope[xindex]->wcspIndex] = xindex;
     }
+
+    // shift left scope_dac array starting at position of variable x in scope_dac
+    int dacindex = 0;
+    while (dacindex < arity_ - 1 && scope_dac[dacindex] != x) {
+        dacindex++;
+    }
+    assert(dacindex == arity_ - 1 || scope_dac[dacindex] == x);
+    for (; dacindex < arity_ - 1; dacindex++) {
+        scope_dac[dacindex] = scope_dac[dacindex + 1];
+    }
+
     if (x->unassigned()) {
         x->deconnect(links[arity_ - 1]);
         nonassigned = nonassigned - 1;
@@ -1121,49 +1133,57 @@ void NaryConstraint::project(EnumeratedVariable* x)
     evalTuple.resize(arity_);
 }
 
-// THIS CODE IS NEVER USED!!!
-//TODO: make it compatible with table representation rather than Map
 // Projects out all variables except x,y,z
 // and gives the result at fproj
-//void NaryConstraint::projectxyz( EnumeratedVariable* x,
-//        EnumeratedVariable* y,
-//        EnumeratedVariable* z,
-//        TUPLES& fproj)
-//{
-//    assert(CUT(default_cost,wcsp->getUb()));
-//
-//    Tuple stxyz(3,0);
-//    Tuple txyz(stxyz);
-//    Tuple t;
-//    Cost c;
-//    TUPLES::iterator  itproj;
-//
-//    // compute in one pass of all tuples the projection
-//    first();
-//    while(next(t,c)) {
-//        txyz[0] = t[ getIndex(x) ];
-//        txyz[1] = t[ getIndex(y) ];
-//        txyz[2] = t[ getIndex(z) ];
-//
-//        itproj = fproj.find(txyz);
-//        if(itproj != fproj.end()) {
-//            if(c < itproj->second) fproj[txyz] = c;
-//        } else {
-//            fproj[txyz] = c;
-//        }
-//    }
-//
-//    // finially we substract the projection from the initial function
-//    first();
-//    while(next(t,c)) {
-//        txyz[0] = t[ getIndex(x) ];
-//        txyz[1] = t[ getIndex(y) ];
-//        txyz[2] = t[ getIndex(z) ];
-//        itproj = fproj.find(txyz);
-//        if(itproj != fproj.end()) { assert(CUT(c, itproj->second)); (*pf)[t] -= itproj->second; }
-//        else assert(false);
-//    }
-//}
+void NaryConstraint::projectxyz(EnumeratedVariable* x,
+    EnumeratedVariable* y,
+    EnumeratedVariable* z,
+    TUPLES& fproj)
+{
+    assert(CUT(default_cost, wcsp->getUb()));
+    static Tuple t;
+    Tuple txyz(3, 0);
+    Cost c;
+    TUPLES::iterator itproj;
+
+    // compute in one pass of all tuples the projection
+    first();
+    while (next(t, c)) {
+        if (c == default_cost)
+            continue;
+        txyz[0] = t[getIndex(x)];
+        txyz[1] = t[getIndex(y)];
+        txyz[2] = t[getIndex(z)];
+        itproj = fproj.find(txyz);
+        if (itproj != fproj.end()) {
+            if (c < itproj->second)
+                fproj[txyz] = c;
+        } else {
+            fproj[txyz] = c;
+        }
+    }
+
+    // finally we subtract the projection from the initial function
+    first();
+    while (next(t, c)) {
+        if (c == default_cost)
+            continue;
+        txyz[0] = t[getIndex(x)];
+        txyz[1] = t[getIndex(y)];
+        txyz[2] = t[getIndex(z)];
+        itproj = fproj.find(txyz);
+        if (itproj != fproj.end()) {
+            assert(CUT(c, itproj->second));
+            //        if (!CUT(c + wcsp->getLb(), wcsp->getUb())) {
+            if (pf)
+                (*pf)[t] -= itproj->second;
+            else
+                costs[getCostsIndex(t)] -= itproj->second;
+            //        }
+        } else
+            assert(false);
+    }
+}
 
 // Projects out all variables except x,y
 // and gives the result at fproj
@@ -1214,45 +1234,43 @@ void NaryConstraint::projectxy(EnumeratedVariable* x,
     }
 }
 
-// THIS CODE IS NEVER USED!!!
-//TODO: make it compatible with table representation rather than Map
-//void NaryConstraint::preproject3()
-//{
-//    assert(connected());
-//    assert(CUT(default_cost,wcsp->getUb()));
-//
-//    for(int i = 0; i < arity_ - 2; i++) {
-//        EnumeratedVariable* x = scope[i];
-//        EnumeratedVariable* y = scope[i+1];
-//        EnumeratedVariable* z = scope[i+2];
-//
-//        TUPLES fproj;
-//        projectxyz(x,y,z,fproj);
-//
-//        Tuple t;
-//        vector<Cost> xyz;
-//        unsigned int a,b,c;
-//        unsigned int sizex = x->getDomainInitSize();
-//        unsigned int sizey = y->getDomainInitSize();
-//        unsigned int sizez = z->getDomainInitSize();
-//
-//        for (a = 0; a < sizex; a++)
-//            for (b = 0; b < sizey; b++)
-//                for (c = 0; c < sizez; c++) xyz.push_back(default_cost);
-//
-//        TUPLES::iterator it =  fproj.begin();
-//        while(it != fproj.end()) {
-//            t = it->first;
-//            a = t[0];
-//            b = t[1];
-//            c = t[2];
-//            xyz[ a * sizey * sizez + b * sizez + c ]	= it->second;
-//            it++;
-//        }
-//        if(fproj.size() > 0 || default_cost > MIN_COST) wcsp->postTernaryConstraint(x->wcspIndex, y->wcspIndex, z->wcspIndex,xyz);
-//        if (deconnected()) return;
-//    }
-//}
+void NaryConstraint::preproject3(TernaryConstraint* ctr)
+{
+    EnumeratedVariable* x = (EnumeratedVariable*)ctr->getVar(0);
+    assert(getIndex(x) >= 0);
+    EnumeratedVariable* y = (EnumeratedVariable*)ctr->getVar(1);
+    assert(getIndex(y) >= 0);
+    EnumeratedVariable* z = (EnumeratedVariable*)ctr->getVar(2);
+    assert(getIndex(z) >= 0);
+
+    TUPLES fproj;
+    projectxyz(x, y, z, fproj);
+
+    unsigned int sizex = x->getDomainInitSize();
+    unsigned int sizey = y->getDomainInitSize();
+    unsigned int sizez = z->getDomainInitSize();
+
+    vector<Cost> xyz;
+    for (unsigned int a = 0; a < sizex; a++)
+        for (unsigned int b = 0; b < sizey; b++)
+            for (unsigned int c = 0; c < sizez; c++)
+                xyz.push_back(default_cost);
+
+    TUPLES::iterator it = fproj.begin();
+    while (it != fproj.end()) {
+        unsigned int a = it->first[0];
+        unsigned int b = it->first[1];
+        unsigned int c = it->first[2];
+        xyz[a * sizey * sizez + b * sizez + c] = it->second;
+        it++;
+    }
+    if (fproj.size() > 0 || default_cost > MIN_COST) {
+        if (ToulBar2::verbose >= 3) {
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] preproject nary " << this << " to ternary (" << x->getName() << "," << y->getName() << "," << z->getName() << ")" << endl;
+        }
+        wcsp->postTernaryConstraint(x->wcspIndex, y->wcspIndex, z->wcspIndex, xyz);
+    }
+}
 
 inline bool cmp_pairvars(pair<EnumeratedVariable*, EnumeratedVariable*> pv1, pair<EnumeratedVariable*, EnumeratedVariable*> pv2)
 {
@@ -1264,13 +1282,10 @@ void NaryConstraint::preprojectall2()
     assert(connected());
     assert(CUT(default_cost, wcsp->getUb()));
 
-    TSCOPE scopeinv;
-    getScope(scopeinv);
-    for (TSCOPE::iterator it1 = scopeinv.begin(); it1 != scopeinv.end(); ++it1) {
-        TSCOPE::iterator it2 = it1;
-        for (++it2; it2 != scopeinv.end(); ++it2) {
-            EnumeratedVariable* x = (EnumeratedVariable*)wcsp->getVar((*it1).first);
-            EnumeratedVariable* y = (EnumeratedVariable*)wcsp->getVar((*it2).first);
+    for (int i = 0; i < arity_; i++) {
+        for (int j = i + 1; j < arity_; j++) {
+            EnumeratedVariable* x = scope_dac[i]; // (EnumeratedVariable*)wcsp->getVar((*it1).first);
+            EnumeratedVariable* y = scope_dac[j]; // (EnumeratedVariable*)wcsp->getVar((*it2).first);
 
             TUPLES fproj;
             projectxy(x, y, fproj);
@@ -1293,6 +1308,9 @@ void NaryConstraint::preprojectall2()
                 it++;
             }
             if (fproj.size() > 0 || default_cost > MIN_COST) {
+                if (ToulBar2::verbose >= 3) {
+                    cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] preproject nary " << this << " to binary (" << x->getName() << "," << y->getName() << ")" << endl;
+                }
                 wcsp->postBinaryConstraint(x->wcspIndex, y->wcspIndex, xy);
             }
             if (deconnected())
@@ -1305,6 +1323,7 @@ double NaryConstraint::computeTightness()
 {
     int count = 0;
     double sum = 0;
+    double tight = -1;
     Cost* costs_ = new Cost[size()];
     if (pf) {
         TUPLES::iterator it = pf->begin();
@@ -1374,10 +1393,10 @@ void NaryConstraint::print(ostream& os)
     //	assert(nonassigned == unassigned_); // not valid when used with assignLS
 
     /*TSCOPE::iterator it = scope_inv.begin();
-	while(it != scope_inv.end()) {
-		os << "(" << it->first << ",idx: " << it->second << ") ";
-		++it;
-	}*/
+        while(it != scope_inv.end()) {
+                os << "(" << it->first << ",idx: " << it->second << ") ";
+                ++it;
+        }*/
     os << endl;
 
     if (ToulBar2::verbose >= 4) {
@@ -1402,7 +1421,24 @@ void NaryConstraint::print(ostream& os)
         } else {
             os << "costs: [";
             for (ptrdiff_t idx = 0; idx < costSize; idx++) {
-                os << " " << costs[idx];
+                if (ToulBar2::verbose >= 8) {
+                    Long tvalues = idx;
+                    Long product = costSize;
+                    os << "(" << idx << ",[";
+                    for (int i = 0; i < arity_; i++) {
+                        product /= scope[i]->getDomainInitSize();
+                        os << scope[i]->toValue(tvalues / product);
+                        tvalues %= product;
+                        if (i < arity_ - 1)
+                            os << ",";
+                    }
+                    assert(product == 1);
+                    os << "]," << costs[idx] << ")";
+                    if (idx < costSize - 1)
+                        os << ",";
+                } else {
+                    os << " " << costs[idx];
+                }
             }
             os << "] " << endl;
         }
@@ -1423,7 +1459,7 @@ void NaryConstraint::dump(ostream& os, bool original)
                 Cost c = it->second;
                 it++;
                 for (unsigned int i = 0; i < t.size(); i++) {
-                    os << t[i] << " ";
+                    os << scope[i]->toValue(t[i]) << " ";
                 }
                 os << c << endl;
             }
@@ -1432,7 +1468,7 @@ void NaryConstraint::dump(ostream& os, bool original)
             vector<unsigned int> t(a, 0);
             for (ptrdiff_t idx = 0; idx < costSize; idx++) {
                 for (int i = 0; i < a; i++) {
-                    os << t[i] << " ";
+                    os << scope[i]->toValue(t[i]) << " ";
                 }
                 os << costs[idx] << endl;
                 int i = a - 1;
@@ -1488,10 +1524,10 @@ void NaryConstraint::dump_CFN(ostream& os, bool original)
         for (int i = 0; i < arity_; i++) {
             if (printed)
                 os << ",";
-            os << scope[i]->getName();
+            os << "\"" << scope[i]->getName() << "\"";
             printed = true;
         }
-        os << "],\"defaultcost\":" << wcsp->Cost2RDCost(default_cost) << ",\n\"costs\":[";
+        os << "],\"defaultcost\":" << wcsp->DCost2Decimal(wcsp->Cost2RDCost(default_cost)) << ",\n\"costs\":[";
         if (pf) {
             TUPLES::iterator it = pf->begin();
             printed = false;
@@ -1506,7 +1542,7 @@ void NaryConstraint::dump_CFN(ostream& os, bool original)
                     os << ((scope[i]->isValueNames()) ? scope[i]->getValueName(t[i]) : std::to_string(t[i]));
                     printed = true;
                 }
-                os << "," << wcsp->Cost2RDCost(c);
+                os << "," << wcsp->DCost2Decimal(wcsp->Cost2RDCost(c));
             }
         } else {
             int a = arity_;
@@ -1517,10 +1553,10 @@ void NaryConstraint::dump_CFN(ostream& os, bool original)
                 for (int i = 0; i < a; i++) {
                     if (printed)
                         os << ",";
-                    printed = true;
                     os << ((scope[i]->isValueNames()) ? scope[i]->getValueName(t[i]) : std::to_string(t[i]));
+                    printed = true;
                 }
-                os << "," << wcsp->Cost2RDCost(costs[idx]);
+                os << "," << wcsp->DCost2Decimal(wcsp->Cost2RDCost(costs[idx]));
                 int i = a - 1;
                 while (i >= 0 && t[i] == scope[i]->getDomainInitSize() - 1) {
                     t[i] = 0;
@@ -1544,25 +1580,26 @@ void NaryConstraint::dump_CFN(ostream& os, bool original)
             if (scope[i]->unassigned()) {
                 if (printed)
                     os << ",";
-                os << scope[i]->getCurrentVarId();
+                os << "\"" << scope[i]->getName() << "\"";
                 printed = true;
             }
-        os << "],\"defaultcost\":" << wcsp->Cost2RDCost(default_cost) << ",\n\"costs\":[";
+        os << "],\"defaultcost\":" << ((wcsp->getUb() > default_cost) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(default_cost)) : "inf") << ",\n\"costs\":[";
 
         Tuple t;
         Cost cost;
         first();
-        printed = false;
         while (next(t, cost)) {
             os << endl;
+            printed = false;
             for (unsigned int i = 0; i < t.size(); i++) {
-                if (printed)
-                    os << ",";
-                printed = true;
-                if (scope[i]->unassigned())
+                if (scope[i]->unassigned()) {
+                    if (printed)
+                        os << ",";
                     os << scope[i]->toCurrentIndex(scope[i]->toValue(t[i]));
+                    printed = true;
+                }
             }
-            os << "," << ((original) ? wcsp->Cost2RDCost(cost) : wcsp->Cost2RDCost(min(wcsp->getUb(), cost)));
+            os << "," << ((wcsp->getUb() > cost) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(cost)) : "inf");
         }
     }
     os << "]},\n";
