@@ -8,7 +8,7 @@
 
 /*
  * Constructors and misc.
- * 
+ *
  */
 
 TernaryConstraint::TernaryConstraint(WCSP* wcsp,
@@ -28,7 +28,7 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
     , functionalY(true)
     , functionalZ(true)
 {
-    assert(tab.size() == sizeX * sizeY * sizeZ);
+    assert(tab.size() == (size_t)sizeX * (size_t)sizeY * (size_t)sizeZ);
     deltaCostsX = vector<StoreCost>(sizeX, StoreCost(MIN_COST));
     deltaCostsY = vector<StoreCost>(sizeY, StoreCost(MIN_COST));
     deltaCostsZ = vector<StoreCost>(sizeZ, StoreCost(MIN_COST));
@@ -40,13 +40,10 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
     supportY = vector<pair<Value, Value>>(sizeY, make_pair(x->getInf(), z->getInf()));
     supportZ = vector<pair<Value, Value>>(sizeZ, make_pair(x->getInf(), y->getInf()));
 
-    //    costs = vector<StoreCost>(sizeX*sizeY*sizeZ,StoreCost(MIN_COST,storeCost));
-
-    for (unsigned int a = 0; a < x->getDomainInitSize(); a++) {
-        for (unsigned int b = 0; b < y->getDomainInitSize(); b++) {
-            for (unsigned int c = 0; c < z->getDomainInitSize(); c++) {
-                Cost cost = tab[a * sizeY * sizeZ + b * sizeZ + c];
-                //		  costs[a * sizeY * sizeZ + b * sizeZ + c] = cost;
+    for (unsigned int a = 0; a < sizeX; a++) {
+        for (unsigned int b = 0; b < sizeY; b++) {
+            for (unsigned int c = 0; c < sizeZ; c++) {
+                Cost cost = tab[(size_t)a * (size_t)sizeY * (size_t)sizeZ + (size_t)b * (size_t)sizeZ + (size_t)c];
                 if (!CUT(cost, top) && x->canbe(x->toValue(a)) && y->canbe(y->toValue(b)) && z->canbe(z->toValue(c))) {
                     if (functionalX) {
                         if (functionX[b * sizeZ + c] == WRONG_VAL)
@@ -71,12 +68,14 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
         }
     }
 
-    xy = xy_;
-    xz = xz_;
-    yz = yz_;
+    setBinaries(xy_, xz_, yz_);
 
     if (functionalX) {
+#ifdef NO_STORE_TERNARY_COSTS
+        costsYZ = vector<Cost>((size_t)sizeY * (size_t)sizeZ, MIN_COST);
+#else
         costsYZ = vector<StoreCost>((size_t)sizeY * (size_t)sizeZ, StoreCost(MIN_COST));
+#endif
         for (unsigned int b = 0; b < y->getDomainInitSize(); b++) {
             for (unsigned int c = 0; c < z->getDomainInitSize(); c++) {
                 if (functionX[b * sizeZ + c] != WRONG_VAL)
@@ -85,11 +84,15 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
         }
         //    	costs.free_all();
     } else {
+#ifdef NO_STORE_TERNARY_COSTS
+        costs = vector<Cost>((size_t)sizeX * (size_t)sizeY * (size_t)sizeZ, MIN_COST);
+#else
         costs = vector<StoreCost>((size_t)sizeX * (size_t)sizeY * (size_t)sizeZ, StoreCost(MIN_COST));
+#endif
         for (unsigned int a = 0; a < x->getDomainInitSize(); a++) {
             for (unsigned int b = 0; b < y->getDomainInitSize(); b++) {
                 for (unsigned int c = 0; c < z->getDomainInitSize(); c++) {
-                    costs[a * sizeY * sizeZ + b * sizeZ + c] = tab[a * sizeY * sizeZ + b * sizeZ + c];
+                    costs[(size_t)a * sizeY * sizeZ + (size_t)b * sizeZ + c] = tab[(size_t)a * sizeY * sizeZ + (size_t)b * sizeZ + c];
                 }
             }
         }
@@ -97,81 +100,81 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
 
 // Uncomment the following code if toulbar2 is used within numberjack
 #ifdef NUMBERJACK
-    vector<int>& vecX = wcsp->getListSuccessors()->at(xx->wcspIndex);
-    vector<int>& vecY = wcsp->getListSuccessors()->at(yy->wcspIndex);
-    vector<int>& vecZ = wcsp->getListSuccessors()->at(zz->wcspIndex);
-    // If variables xx,yy,zz are not yet involved by a decomposable global cost function and there is a functional dependency between them
+    vector<int>& vecX = wcsp->getListSuccessors()->at(x->wcspIndex);
+    vector<int>& vecY = wcsp->getListSuccessors()->at(y->wcspIndex);
+    vector<int>& vecZ = wcsp->getListSuccessors()->at(z->wcspIndex);
+    // If variables x,y,z are not yet involved by a decomposable global cost function and there is a functional dependency between them
     // then suggests a good Berge DAC ordering:
-    if ((std::find(vecX.begin(), vecX.end(), yy->wcspIndex) == vecX.end()) && (std::find(vecX.begin(), vecX.end(), zz->wcspIndex) == vecX.end()) && (std::find(vecY.begin(), vecY.end(), xx->wcspIndex) == vecY.end()) && (std::find(vecY.begin(), vecY.end(), zz->wcspIndex) == vecY.end()) && (std::find(vecZ.begin(), vecZ.end(), xx->wcspIndex) == vecZ.end()) && (std::find(vecZ.begin(), vecZ.end(), yy->wcspIndex) == vecZ.end())) {
+    if ((std::find(vecX.begin(), vecX.end(), y->wcspIndex) == vecX.end()) && (std::find(vecX.begin(), vecX.end(), z->wcspIndex) == vecX.end()) && (std::find(vecY.begin(), vecY.end(), x->wcspIndex) == vecY.end()) && (std::find(vecY.begin(), vecY.end(), z->wcspIndex) == vecY.end()) && (std::find(vecZ.begin(), vecZ.end(), x->wcspIndex) == vecZ.end()) && (std::find(vecZ.begin(), vecZ.end(), y->wcspIndex) == vecZ.end())) {
         switch (functionalX + functionalY + functionalZ) {
         case 1:
             if (functionalX) {
-                vecX.push_back(yy->wcspIndex);
-                vecX.push_back(zz->wcspIndex);
+                vecX.push_back(y->wcspIndex);
+                vecX.push_back(z->wcspIndex);
             } else if (functionalY) {
-                vecY.push_back(xx->wcspIndex);
-                vecY.push_back(zz->wcspIndex);
+                vecY.push_back(x->wcspIndex);
+                vecY.push_back(z->wcspIndex);
             } else if (functionalZ) {
-                vecZ.push_back(xx->wcspIndex);
-                vecZ.push_back(yy->wcspIndex);
+                vecZ.push_back(x->wcspIndex);
+                vecZ.push_back(y->wcspIndex);
             }
-            ToulBar2::Berge_Dec = 1;
+            ToulBar2::Berge_Dec = true;
             break;
         case 2:
             if (functionalX && functionalY) {
-                if (xx->wcspIndex < yy->wcspIndex) {
-                    vecX.push_back(zz->wcspIndex);
-                    vecZ.push_back(yy->wcspIndex);
+                if (x->wcspIndex < y->wcspIndex) {
+                    vecX.push_back(z->wcspIndex);
+                    vecZ.push_back(y->wcspIndex);
                 } else {
-                    vecY.push_back(zz->wcspIndex);
-                    vecZ.push_back(xx->wcspIndex);
+                    vecY.push_back(z->wcspIndex);
+                    vecZ.push_back(x->wcspIndex);
                 }
             } else if (functionalX && functionalZ) {
-                if (xx->wcspIndex < zz->wcspIndex) {
-                    vecX.push_back(yy->wcspIndex);
-                    vecY.push_back(zz->wcspIndex);
+                if (x->wcspIndex < z->wcspIndex) {
+                    vecX.push_back(y->wcspIndex);
+                    vecY.push_back(z->wcspIndex);
                 } else {
-                    vecZ.push_back(yy->wcspIndex);
-                    vecY.push_back(xx->wcspIndex);
+                    vecZ.push_back(y->wcspIndex);
+                    vecY.push_back(x->wcspIndex);
                 }
             } else if (functionalY && functionalZ) {
-                if (yy->wcspIndex < zz->wcspIndex) {
-                    vecY.push_back(xx->wcspIndex);
-                    vecX.push_back(zz->wcspIndex);
+                if (y->wcspIndex < z->wcspIndex) {
+                    vecY.push_back(x->wcspIndex);
+                    vecX.push_back(z->wcspIndex);
                 } else {
-                    vecZ.push_back(xx->wcspIndex);
-                    vecX.push_back(yy->wcspIndex);
+                    vecZ.push_back(x->wcspIndex);
+                    vecX.push_back(y->wcspIndex);
                 }
             }
-            ToulBar2::Berge_Dec = 1;
+            ToulBar2::Berge_Dec = true;
             break;
         case 3:
-            if (xx->wcspIndex < yy->wcspIndex && xx->wcspIndex < zz->wcspIndex) {
-                if (yy->wcspIndex < zz->wcspIndex) {
-                    vecY.push_back(xx->wcspIndex);
-                    vecX.push_back(zz->wcspIndex);
+            if (x->wcspIndex < y->wcspIndex && x->wcspIndex < z->wcspIndex) {
+                if (y->wcspIndex < z->wcspIndex) {
+                    vecY.push_back(x->wcspIndex);
+                    vecX.push_back(z->wcspIndex);
                 } else {
-                    vecZ.push_back(xx->wcspIndex);
-                    vecX.push_back(yy->wcspIndex);
+                    vecZ.push_back(x->wcspIndex);
+                    vecX.push_back(y->wcspIndex);
                 }
-            } else if (yy->wcspIndex < xx->wcspIndex && yy->wcspIndex < zz->wcspIndex) {
-                if (xx->wcspIndex < zz->wcspIndex) {
-                    vecX.push_back(yy->wcspIndex);
-                    vecY.push_back(zz->wcspIndex);
+            } else if (y->wcspIndex < x->wcspIndex && y->wcspIndex < z->wcspIndex) {
+                if (x->wcspIndex < z->wcspIndex) {
+                    vecX.push_back(y->wcspIndex);
+                    vecY.push_back(z->wcspIndex);
                 } else {
-                    vecZ.push_back(yy->wcspIndex);
-                    vecY.push_back(xx->wcspIndex);
+                    vecZ.push_back(y->wcspIndex);
+                    vecY.push_back(x->wcspIndex);
                 }
-            } else if (zz->wcspIndex < xx->wcspIndex && zz->wcspIndex < yy->wcspIndex) {
-                if (xx->wcspIndex < yy->wcspIndex) {
-                    vecX.push_back(zz->wcspIndex);
-                    vecZ.push_back(yy->wcspIndex);
+            } else if (z->wcspIndex < x->wcspIndex && z->wcspIndex < y->wcspIndex) {
+                if (x->wcspIndex < y->wcspIndex) {
+                    vecX.push_back(z->wcspIndex);
+                    vecZ.push_back(y->wcspIndex);
                 } else {
-                    vecY.push_back(zz->wcspIndex);
-                    vecZ.push_back(xx->wcspIndex);
+                    vecY.push_back(z->wcspIndex);
+                    vecZ.push_back(x->wcspIndex);
                 }
             }
-            ToulBar2::Berge_Dec = 1;
+            ToulBar2::Berge_Dec = true;
             break;
         default:
             break;
@@ -179,7 +182,23 @@ TernaryConstraint::TernaryConstraint(WCSP* wcsp,
     }
 #endif
 
-    propagate();
+    if (ToulBar2::bilevel >= 2) {
+        assert(ToulBar2::bilevel <= 3);
+        deconnect(true);
+        assert(xy->deconnected());
+        assert(xz->deconnected());
+        assert(yz->deconnected());
+        assert(wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].size() >= 3); // fresh new binary cost functions must have been automatically created and added
+        assert(yz->wcspIndex == wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].back());
+        wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].pop_back();
+        assert(xz->wcspIndex == wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].back());
+        wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].pop_back();
+        assert(xy->wcspIndex == wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].back());
+        wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].pop_back();
+        wcsp->delayedCtrBLP[ToulBar2::bilevel - 1].push_back(wcspIndex);
+    } else {
+        propagate();
+    }
 }
 
 TernaryConstraint::TernaryConstraint(WCSP* wcsp)
@@ -216,6 +235,7 @@ double TernaryConstraint::computeTightness()
 {
     int count = 0;
     double sum = 0;
+    double tight = -1;
     Cost* costs = new Cost[x->getDomainSize() * y->getDomainSize() * z->getDomainSize()];
     for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
         for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
@@ -242,10 +262,13 @@ void TernaryConstraint::print(ostream& os)
     os << this << " TernaryConstraint(" << x->getName() << ((functionalX) ? "!" : "") << "," << y->getName() << ((functionalY) ? "!" : "") << "," << z->getName() << ((functionalZ) ? "!" : "") << ")";
     if (ToulBar2::weightedDegree)
         os << "/" << getConflictWeight();
-    if (wcsp->getTreeDec())
-        cout << "   cluster: " << getCluster() << endl;
-    else
-        cout << endl;
+    if (wcsp->getTreeDec()) {
+        cout << "   cluster: " << getCluster();
+        if (ToulBar2::heuristicFreedom && getCluster() != -1) {
+            cout << "  freedom: " << wcsp->getTreeDec()->getCluster(getCluster())->getFreedom() << " isinTD: " << wcsp->getTreeDec()->getCluster(getCluster())->getIsCurrInTD();
+        }
+    }
+    cout << endl;
 
     if (ToulBar2::verbose >= 5) {
         for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
@@ -262,14 +285,37 @@ void TernaryConstraint::print(ostream& os)
 
 void TernaryConstraint::dump(ostream& os, bool original)
 {
-    os << "3 " << ((original) ? (x->wcspIndex) : x->getCurrentVarId()) << " " << ((original) ? (y->wcspIndex) : y->getCurrentVarId()) << " " << ((original) ? (z->wcspIndex) : z->getCurrentVarId()) << " " << MIN_COST << " " << x->getDomainSize() * y->getDomainSize() * z->getDomainSize() << endl;
+    unsigned int tuples = 0;
+    unsigned int tuplesNotTop = 0;
+    for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
+        for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
+            for (EnumeratedVariable::iterator iterZ = z->begin(); iterZ != z->end(); ++iterZ) {
+                Cost cost = getCost(*iterX, *iterY, *iterZ);
+                if (cost > MIN_COST) {
+                    tuples++;
+                }
+                if (cost != top) {
+                    tuplesNotTop++;
+                }
+            }
+        }
+    }
+    Cost defaultCost = MIN_COST;
+    if (tuplesNotTop < tuples) {
+        defaultCost = top;
+        tuples = tuplesNotTop;
+    }
+    os << "3 " << ((original) ? (x->wcspIndex) : x->getCurrentVarId()) << " " << ((original) ? (y->wcspIndex) : y->getCurrentVarId()) << " " << ((original) ? (z->wcspIndex) : z->getCurrentVarId()) << " " << defaultCost << " " << tuples << endl;
     int i = 0;
     for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX, i++) {
         int j = 0;
         for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY, j++) {
             int k = 0;
             for (EnumeratedVariable::iterator iterZ = z->begin(); iterZ != z->end(); ++iterZ, k++) {
-                os << ((original) ? x->toIndex(*iterX) : i) << " " << ((original) ? y->toIndex(*iterY) : j) << " " << ((original) ? z->toIndex(*iterZ) : k) << " " << ((original) ? getCost(*iterX, *iterY, *iterZ) : min(wcsp->getUb(), getCost(*iterX, *iterY, *iterZ))) << endl;
+                Cost cost = getCost(*iterX, *iterY, *iterZ);
+                if (cost != defaultCost) {
+                    os << ((original) ? *iterX : i) << " " << ((original) ? *iterY : j) << " " << ((original) ? *iterZ : k) << " " << ((original) ? cost : min(wcsp->getUb(), cost)) << endl;
+                }
             }
         }
     }
@@ -277,41 +323,60 @@ void TernaryConstraint::dump(ostream& os, bool original)
 
 void TernaryConstraint::dump_CFN(ostream& os, bool original)
 {
+    unsigned int tuples = 0;
+    unsigned int tuplesNotTop = 0;
+    for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX) {
+        for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY) {
+            for (EnumeratedVariable::iterator iterZ = z->begin(); iterZ != z->end(); ++iterZ) {
+                Cost cost = getCost(*iterX, *iterY, *iterZ);
+                if (cost > MIN_COST) {
+                    tuples++;
+                }
+                if (cost != top) {
+                    tuplesNotTop++;
+                }
+            }
+        }
+    }
+    Cost defaultCost = MIN_COST;
+    if (tuplesNotTop < tuples) {
+        defaultCost = top;
+        tuples = tuplesNotTop;
+    }
     bool printed = false;
     os << "\"F_" << ((original) ? (x->wcspIndex) : x->getCurrentVarId()) << "_"
        << ((original) ? (y->wcspIndex) : y->getCurrentVarId()) << "_"
-       << ((original) ? (z->wcspIndex) : z->getCurrentVarId()) << "\":{\"scope\":[";
-    os << x->getName() << ","
-       << y->getName() << ","
-       << z->getName() << "],";
-    os << "\"defaultcost\":" << MIN_COST << ",\n\"costs\":[\n";
+       << ((original) ? (z->wcspIndex) : z->getCurrentVarId()) << "\":{\"scope\":[\"";
+    os << x->getName() << "\",\""
+       << y->getName() << "\",\""
+       << z->getName() << "\"],";
+    os << "\"defaultcost\":" << defaultCost << ",\n\"costs\":[\n";
     int i = 0;
     for (EnumeratedVariable::iterator iterX = x->begin(); iterX != x->end(); ++iterX, i++) {
         int j = 0;
         for (EnumeratedVariable::iterator iterY = y->begin(); iterY != y->end(); ++iterY, j++) {
             int k = 0;
             for (EnumeratedVariable::iterator iterZ = z->begin(); iterZ != z->end(); ++iterZ, k++) {
-                if (printed)
-                    os << ",\n";
-                if (getCost(*iterX, *iterY, *iterZ) != MIN_COST) {
+                if (getCost(*iterX, *iterY, *iterZ) != defaultCost) {
+                    if (printed)
+                        os << ",\n";
                     os << ((original) ? x->toIndex(*iterX) : i) << "," << ((original) ? y->toIndex(*iterY) : j) << "," << ((original) ? z->toIndex(*iterZ) : k) << ","
-                       << ((original) ? wcsp->Cost2RDCost(getCost(*iterX, *iterY, *iterZ)) : wcsp->Cost2RDCost(min(wcsp->getUb(), getCost(*iterX, *iterY, *iterZ))));
+                       << ((original) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(getCost(*iterX, *iterY, *iterZ))) : ((wcsp->getUb() > getCost(*iterX, *iterY, *iterZ)) ? wcsp->DCost2Decimal(wcsp->Cost2RDCost(getCost(*iterX, *iterY, *iterZ))) : "inf"));
                     printed = true;
-                } else
-                    printed = false;
+                }
             }
         }
     }
-    os << "]}\n";
+    os << "]},\n";
 }
 
 /*
  * Propagation methods
- * 
+ *
  */
 bool TernaryConstraint::project(EnumeratedVariable* x, Value value, Cost cost, vector<StoreCost>& deltaCostsX)
 {
-    assert(ToulBar2::verbose < 4 || ((cout << "project(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << getVar(2)->getName() << ", (" << x->getName() << "," << value << "), " << cost << ")" << endl), true));
+    assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] project(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << getVar(2)->getName() << ", (" << x->getName() << "," << value << "), " << cost << ")" << endl), true));
 
     // hard ternary constraint costs are not changed
     if (!CUT(cost + wcsp->getLb(), wcsp->getUb())) {
@@ -333,7 +398,7 @@ bool TernaryConstraint::project(EnumeratedVariable* x, Value value, Cost cost, v
 
 void TernaryConstraint::extend(EnumeratedVariable* x, Value value, Cost cost, vector<StoreCost>& deltaCostsX)
 {
-    assert(ToulBar2::verbose < 4 || ((cout << "extend(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << getVar(2)->getName() << ", (" << x->getName() << "," << value << "), " << cost << ")" << endl), true));
+    assert(ToulBar2::verbose < 4 || ((cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] extend(C" << getVar(0)->getName() << "," << getVar(1)->getName() << "," << getVar(2)->getName() << ", (" << x->getName() << "," << value << "), " << cost << ")" << endl), true));
     TreeDecomposition* td = wcsp->getTreeDec();
     if (td)
         td->addDelta(cluster, x, value, -cost);
@@ -494,7 +559,7 @@ bool TernaryConstraint::separability(EnumeratedVariable* vy, EnumeratedVariable*
 void TernaryConstraint::separate(EnumeratedVariable* vy, EnumeratedVariable* vz)
 {
     Cost cost, minCost = MAX_COST;
-    //assert(separability(vy,vz));
+    // assert(separability(vy,vz));
     first(vy, vz);
     vector<Cost> costsZX((size_t)zvar->getDomainInitSize() * (size_t)xvar->getDomainInitSize(), MIN_COST);
     vector<Cost> costsXY((size_t)xvar->getDomainInitSize() * (size_t)yvar->getDomainInitSize(), MIN_COST);
@@ -545,6 +610,7 @@ void TernaryConstraint::separate(EnumeratedVariable* vy, EnumeratedVariable* vz)
             costsXY[xvar->toIndex(*itvx) * yvar->getDomainInitSize() + yvar->toIndex(*itvy)] = squareminus(cost, costzx, wcsp->getUb());
             if (ToulBar2::verbose >= 3)
                 cout << *itvx << " " << *itvy << " : " << squareminus(cost, costzx, wcsp->getUb()) << endl;
+            assert(squareminus(cost, costzx, wcsp->getUb()) >= MIN_COST);
             ++itvy;
         }
         ++itvx;
@@ -583,15 +649,16 @@ void TernaryConstraint::fillxy()
     TreeDecomposition* td = wcsp->getTreeDec();
     BinaryConstraint* xy_ = NULL;
     xy_ = x->getConstr(y);
-    if (td && xy_ && (getCluster() != xy_->getCluster())) {
+    if (td && xy_ && !td->isSameCluster(getCluster(), xy_->getCluster())) {
         BinaryConstraint* xy__ = x->getConstr(y, getCluster());
         if (xy__)
             xy_ = xy__; // we have found another constraint of the same cluster
     }
-    if (!xy_ || (xy_ && td && (getCluster() != xy_->getCluster()))) {
+    if (!xy_ || (xy_ && td && !td->isSameCluster(getCluster(), xy_->getCluster()))) {
         xy = wcsp->newBinaryConstr(x, y, this);
-        xy->setCluster(getCluster());
-        if (td && xy_ && (getCluster() != xy_->getCluster()))
+        if (td && !ToulBar2::approximateCountingBTD)
+            xy->setCluster(getCluster());
+        if (td && xy_ && !td->isSameCluster(getCluster(), xy_->getCluster()))
             xy->setDuplicate();
         wcsp->elimBinOrderInc();
     } else
@@ -605,19 +672,18 @@ void TernaryConstraint::fillxz()
     TreeDecomposition* td = wcsp->getTreeDec();
     BinaryConstraint* xz_ = NULL;
     xz_ = x->getConstr(z);
-    if (td && xz_ && (getCluster() != xz_->getCluster())) {
+    if (td && xz_ && !td->isSameCluster(getCluster(), xz_->getCluster())) {
         BinaryConstraint* xz__ = x->getConstr(z, getCluster());
         if (xz__)
             xz_ = xz__; // we have found another constraint of the same cluster
     }
-    if (!xz_ || (xz_ && td && getCluster() != xz_->getCluster())) {
+    if (!xz_ || (xz_ && td && !td->isSameCluster(getCluster(), xz_->getCluster()))) {
         xz = wcsp->newBinaryConstr(x, z, this);
-        xy->setCluster(getCluster());
-        if (td && xz_ && (getCluster() != xz_->getCluster()))
+        if (td && !ToulBar2::approximateCountingBTD)
+            xz->setCluster(getCluster());
+        if (td && xz_ && !td->isSameCluster(getCluster(), xz_->getCluster()))
             xz->setDuplicate();
         wcsp->elimBinOrderInc();
-        if (td)
-            xz->setCluster(getCluster());
     } else
         xz = xz_;
     if (xz->isDuplicate())
@@ -629,15 +695,16 @@ void TernaryConstraint::fillyz()
     TreeDecomposition* td = wcsp->getTreeDec();
     BinaryConstraint* yz_ = NULL;
     yz_ = y->getConstr(z);
-    if (td && yz_ && (getCluster() != yz_->getCluster())) {
+    if (td && yz_ && !td->isSameCluster(getCluster(), yz_->getCluster())) {
         BinaryConstraint* yz__ = y->getConstr(z, getCluster());
         if (yz__)
             yz_ = yz__;
     }
-    if (!yz_ || (yz_ && td && getCluster() != yz_->getCluster())) {
+    if (!yz_ || (yz_ && td && !td->isSameCluster(getCluster(), yz_->getCluster()))) {
         yz = wcsp->newBinaryConstr(y, z, this);
-        yz->setCluster(getCluster());
-        if (td && yz_ && (getCluster() != yz_->getCluster()))
+        if (td && !ToulBar2::approximateCountingBTD)
+            yz->setCluster(getCluster());
+        if (td && yz_ && !td->isSameCluster(getCluster(), yz_->getCluster()))
             yz->setDuplicate();
         wcsp->elimBinOrderInc();
     } else
@@ -652,8 +719,10 @@ void TernaryConstraint::fillElimConstrBinaries()
     fillxz();
     fillyz();
 
+    resetConflictWeight(); // if needed recompute tightness after having updated costs
+
     if (ToulBar2::verbose > 1)
-        cout << " fillElimConstrBinaries (" << x->wcspIndex << "," << y->wcspIndex << "," << z->wcspIndex << ")  ";
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] fillElimConstrBinaries (" << x->wcspIndex << "," << y->wcspIndex << "," << z->wcspIndex << ")  ";
 }
 
 void TernaryConstraint::setDuplicates()
@@ -668,7 +737,7 @@ void TernaryConstraint::setDuplicates()
         } else {
             wcsp->initElimConstr();
             xy = wcsp->newBinaryConstr(x, y);
-            xy->setCluster(cluster);
+            xy->setCluster(getCluster());
             xy->setDuplicate();
             wcsp->elimBinOrderInc();
             setDuplicate();
@@ -704,6 +773,7 @@ void TernaryConstraint::setDuplicates()
             setDuplicate();
         }
     }
+    assert(xy->getCluster() == getCluster() && xz->getCluster() == getCluster() && yz->getCluster() == getCluster());
 }
 
 bool TernaryConstraint::verify(EnumeratedVariable* x, EnumeratedVariable* y, EnumeratedVariable* z)
@@ -713,8 +783,10 @@ bool TernaryConstraint::verify(EnumeratedVariable* x, EnumeratedVariable* y, Enu
         for (EnumeratedVariable::iterator iterY = y->begin(); minCost > MIN_COST && iterY != y->end(); ++iterY) {
             for (EnumeratedVariable::iterator iterZ = z->begin(); minCost > MIN_COST && iterZ != z->end(); ++iterZ) {
                 Cost cost = getCost(x, y, z, *iterX, *iterY, *iterZ);
+#ifndef NO_STORE_TERNARY_COSTS
                 if (ToulBar2::LcLevel >= LC_DAC && getIndex(x) == getDACScopeIndex())
                     cost += y->getCost(*iterY) + z->getCost(*iterZ);
+#endif
                 GLB(&minCost, cost);
             }
         }
@@ -731,8 +803,18 @@ bool TernaryConstraint::verify()
     TreeDecomposition* td = wcsp->getTreeDec();
 
     if (td) {
-        if (cluster != xy->getCluster() || cluster != xz->getCluster() || cluster != yz->getCluster())
+        if (!td->isSameCluster(cluster, xy->getCluster()) || !td->isSameCluster(cluster, xz->getCluster()) || !td->isSameCluster(cluster, yz->getCluster())) {
+            if (ToulBar2::heuristicFreedom) {
+                cout << " different cluster assignment for ternary: " << cluster << "(" << td->getCluster(cluster)->getFreedom() << ") xy: " << xy->getCluster() << "(" << td->getCluster(xy->getCluster())->getFreedom() << ") xz: " << xz->getCluster() << "(" << td->getCluster(xz->getCluster())->getFreedom() << ") yz: " << yz->getCluster() << "(" << td->getCluster(yz->getCluster())->getFreedom() << ")" << endl;
+            } else {
+                cout << " different cluster assignment for ternary: " << cluster << " xy: " << xy->getCluster() << " xz: " << xz->getCluster() << " yz: " << yz->getCluster() << endl;
+            }
+            cout << *this;
+            cout << *xy;
+            cout << *xz;
+            cout << *yz;
             return false;
+        }
     }
 
     if (ToulBar2::LcLevel == LC_DAC) {
@@ -754,7 +836,12 @@ bool TernaryConstraint::verify()
     }
 }
 
-//Triangle::Triangle(WCSP *wcsp,
+bool TernaryConstraint::checkTreeDecomposition()
+{
+    return (!wcsp->getTreeDec() || (wcsp->getTreeDec()->isSameCluster(cluster, xy->getCluster()) && wcsp->getTreeDec()->isSameCluster(cluster, xz->getCluster()) && wcsp->getTreeDec()->isSameCluster(cluster, yz->getCluster())));
+}
+
+// Triangle::Triangle(WCSP *wcsp,
 //				  EnumeratedVariable *xx,
 //				  EnumeratedVariable *yy,
 //				  EnumeratedVariable *zz,
@@ -777,9 +864,9 @@ bool TernaryConstraint::verify()
 //
 //						  }
 
-//activate {
+// activate {
 //	xyz = wcsp->newTernaryConstr(x,y,z,this);
-//}
+// }
 
 /* Local Variables: */
 /* c-basic-offset: 4 */
