@@ -1,19 +1,20 @@
 /** \file tb2vac.cpp
  *  \brief VAC implementation for binary cost functions.
  *
- *      \defgroup VAC Virtual Arc Consistency enforcing
+ *  \defgroup VAC Virtual Arc Consistency enforcing
  *  The three phases of VAC are enforced in three different "Pass".
  *  Bool(P) is never built. Instead specific functions (getVACCost) booleanize the WCSP on the fly.
- *  The domain variables of Bool(P) are the original variable domains (saved and restored using trailing at each iteration) 
+ *  The domain variables of Bool(P) are the original variable domains (saved and restored using trailing at each iteration).
  *  All the counter data-structures (k) are timestamped to avoid clearing them at each iteration.
- *  \note Simultaneously AC (and potentially DAC, EAC) are maintained by proper queuing.
- *  \see <em> Soft Arc Consistency Revisited. </em> Cooper et al. Artificial Intelligence. 2010.
+ *
+ *  Note : Simultaneously AC (and potentially DAC, EAC) are maintained by proper queuing.
+ *
+ *  See : <em> Soft Arc Consistency Revisited. </em> Cooper et al. Artificial Intelligence. 2010.
  */
 
 #include <math.h> /* atan2 */
 #include "search/tb2clusters.hpp"
 #include "tb2vacutils.hpp"
-#define PI 3.14159265
 
 class tVACStat {
 public:
@@ -51,8 +52,8 @@ VACExtension::VACExtension(WCSP* w)
     , bneckCF(NULL)
     , bneckCost(MIN_COST)
 {
-    queueP = new stack<pair<int, int>>;
-    queueR = new stack<pair<int, int>>;
+    queueP = new stack<pair<int, Value>>;
+    queueR = new stack<pair<int, Value>>;
 }
 
 VACExtension::~VACExtension()
@@ -156,7 +157,7 @@ void VACExtension::nextScaleCost()
     if (!done)
         c = itThreshold / (UNIT_COST + UNIT_COST);
 
-    if (Store::getDepth() == 0) {
+    if (Store::getDepth() <= 1) {
         if (c < ToulBar2::costThresholdPre)
             c = MIN_COST;
     } else if (c < ToulBar2::costThreshold)
@@ -229,7 +230,7 @@ bool VACExtension::enqueueVAC(Cost threshold, Cost previousThreshold)
 
 bool VACExtension::propagate()
 {
-    if (Store::getDepth() >= abs(ToulBar2::vac)) {
+    if (!ToulBar2::RASPS && Store::getDepth() >= abs(ToulBar2::vac)) {
         inconsistentVariable = -1;
         return false;
     }
@@ -329,7 +330,7 @@ bool VACExtension::propagate()
                     double ratio = (ToulBar2::RASPSnbStrictACVariables == 0) ? 0.0000000001 : (((double)ToulBar2::RASPSnbStrictACVariables / (double)wcsp->numberOfVariables()) / (double)itThreshold);
                     if (ToulBar2::verbose >= 0) {
                         cout << std::fixed << std::setprecision(7);
-                        cout << "Threshold: " << itThreshold << " NbAssignedVar: " << ToulBar2::RASPSnbStrictACVariables << " Ratio: " << ratio << endl;
+                        cout << "Threshold: " << itThreshold << " NbAssignedVar: " << ToulBar2::RASPSnbStrictACVariables << " Ratio: " << ratio << " SumOfDomainSize: " << wcsp->getDomainSizeSum() << endl;
                         cout << std::fixed << std::setprecision(DECIMAL_POINT);
                     }
                     ToulBar2::RASPSitThresholds.push_back(std::make_pair(itThreshold, ratio));
@@ -390,10 +391,10 @@ bool VACExtension::propagate()
                                 wcsp->assignLS(variables, values, true); // option true: make sure already assigned variables are removed from Solver::unassignedVars
                             wcsp->propagate();
                             if (ToulBar2::useRASPS <= 1) {
-                                //cout << "call to recursiveSolve from VAC" << endl;
+                                // cout << "call to recursiveSolve from VAC" << endl;
                                 solver->recursiveSolve(wcsp->getLb()); // if a new solution is found, UB will be updated automatically
                             } else {
-                                //cout << "call to recursiveSolveLDS from VAC" << endl;
+                                // cout << "call to recursiveSolveLDS from VAC" << endl;
                                 solver->recursiveSolveLDS(ToulBar2::useRASPS - 1);
                             }
                         } catch (const Contradiction&) {
@@ -445,6 +446,37 @@ bool VACExtension::propagate()
                     }
                 }
 #endif
+                //                if (ToulBar2::preprocessTernaryRPC && Store::getDepth() <= 1) {
+                //                    double mintight = 1e20;
+                //                    Variable *var1 = NULL;
+                //                    Variable *var2 = NULL;
+                //                    for (unsigned int i = 0; i < wcsp->numberOfConstraints(); i++) {
+                //                        if (wcsp->getCtr(i)->connected() && wcsp->getCtr(i)->isBinary()) {
+                //                            wcsp->getCtr(i)->resetTightness();
+                //                            double tight = wcsp->getCtr(i)->getTightness();
+                //                            if (mintight > tight) {
+                //                                mintight = tight;
+                //                                var1 = wcsp->getCtr(i)->getVar(0);
+                //                                var2 = wcsp->getCtr(i)->getVar(1);
+                //                            }
+                //                        }
+                //                    }
+                //                    for (int i = 0; i < wcsp->getElimBinOrder(); i++) {
+                //                        if (wcsp->getCtr(-i -1)->connected()) {
+                //                            wcsp->getCtr(-i -1)->resetTightness();
+                //                            double tight = wcsp->getCtr(-i -1)->getTightness();
+                //                            if (mintight > tight) {
+                //                                mintight = tight;
+                //                                var1 = wcsp->getCtr(-i -1)->getVar(0);
+                //                                var2 = wcsp->getCtr(-i -1)->getVar(1);
+                //                            }
+                //                            mintight = MIN(mintight, tight);
+                //                        }
+                //                    }
+                //                    if (ToulBar2::verbose >= 0 && var1 && var2) {
+                //                        cout << "Minimum binary cost function tightness after VAC: " << mintight << " on variables " << var1->getName() << " and " << var2->getName() << endl;
+                //                    }
+                //                }
             }
 #ifndef INCREMENTALVAC
             Store::restore(storedepth);
@@ -472,9 +504,9 @@ bool VACExtension::propagate()
     if (ToulBar2::vacValueHeuristic && acSupportOK && isVAC()) {
         // update current unary support if possible && needed
         for (vector<tuple<VACVariable*, Value, bool>>::iterator iter = acSupport.begin(); iter != acSupport.end(); ++iter) {
-            VACVariable* x = get<0>(*iter);
-            Value val = get<1>(*iter);
-            bool vacintegral = get<2>(*iter);
+            VACVariable* x = std::get<0>(*iter);
+            Value val = std::get<1>(*iter);
+            bool vacintegral = std::get<2>(*iter);
             if (x->canbe(val)) {
                 if (x->getCost(val) == MIN_COST) {
                     if (ToulBar2::verbose > 0 && (x->getSupport() != val || (vacintegral && !x->isFullEAC())))
@@ -509,7 +541,7 @@ bool VACExtension::enforcePass1(VACVariable* xj, VACBinaryConstraint* cij)
         } else if (cij->revise(xi, v)) {
             wipeout = xi->removeVAC(v);
             xi->setKiller(v, xj->wcspIndex);
-            queueP->push(pair<int, int>(xi->wcspIndex, v));
+            queueP->push(pair<int, Value>(xi->wcspIndex, v));
             if (wipeout) {
                 inconsistentVariable = xi->wcspIndex;
                 return true;
@@ -533,6 +565,8 @@ void VACExtension::enforcePass1()
     VACBinaryConstraint* cij;
 
     while (!VAC.empty()) {
+        if (ToulBar2::interrupted)
+            throw TimeOut();
         xj = (VACVariable*)VAC.pop_first();
         for (EnumeratedVariable::iterator it = xj->begin(); it != xj->end(); ++it) {
             if (xj->getVACCost(*it) > MIN_COST)
@@ -541,7 +575,7 @@ void VACExtension::enforcePass1()
         for (ConstraintList::iterator itc = xj->getConstrs()->begin();
              itc != xj->getConstrs()->end(); ++itc) {
             Constraint* c = (*itc).constr;
-            if (c->isBinary()) {
+            if (c->isBinary() && !c->isDuplicate()) {
                 cij = (VACBinaryConstraint*)c;
                 if (enforcePass1(xj, cij))
                     return;
@@ -565,7 +599,7 @@ bool VACExtension::checkPass1() const
         for (ConstraintList::iterator iter = xi->getConstrs()->begin();
              iter != xj->getConstrs()->end(); ++iter) {
             Constraint* c = (*iter).constr;
-            if (c->isBinary()) {
+            if (c->isBinary() && !c->isDuplicate()) {
                 cij = (VACBinaryConstraint*)c;
                 xj = (VACVariable*)cij->getVarDiffFrom(xi);
                 for (EnumeratedVariable::iterator iti = xi->begin(); iti != xi->end(); ++iti) {
@@ -600,7 +634,7 @@ void VACExtension::enforcePass2()
     Cost tmplambda;
     Value v;
 
-    //if (ToulBar2::verbose > 0)  cout << "VAC Enforce Pass 2" << endl;
+    // if (ToulBar2::verbose > 0)  cout << "VAC Enforce Pass 2" << endl;
 
     assert(i0 >= 0);
     xi0 = (VACVariable*)wcsp->getVar(i0);
@@ -626,10 +660,11 @@ void VACExtension::enforcePass2()
         if (xi->isMarked(v, nbIterations)) {
             j = xi->getKiller(v);
             xj = (VACVariable*)wcsp->getVar(j);
-            queueR->push(pair<int, int>(i, v));
-            cij = (VACBinaryConstraint*)xi->getConstr(xj);
+            queueR->push(pair<int, Value>(i, v));
+            cij = (VACBinaryConstraint*)xi->getConstrNotDuplicate(xj);
             assert(cij);
-            //if (ToulBar2::verbose > 6) cout << "x" << xi->wcspIndex << "," << v << "   killer: " << xj->wcspIndex << endl;
+            assert(!cij->isDuplicate());
+            // if (ToulBar2::verbose > 6) cout << "x" << xi->wcspIndex << "," << v << "   killer: " << xj->wcspIndex << endl;
 
             for (EnumeratedVariable::iterator itj = xj->begin();
                  itj != xj->end(); ++itj) {
@@ -644,7 +679,7 @@ void VACExtension::enforcePass2()
                             wcsp->getUb())) {
                         if ((costij / tmpK) < minlambda) {
                             minlambda = costij / tmpK;
-                            if (minlambda < UNIT_COST) { //A cost function bottleneck here !
+                            if (minlambda < UNIT_COST) { // A cost function bottleneck here !
                                 bneckCost = costij;
                                 bneckCF = cij;
                                 bneckVar = -1;
@@ -693,9 +728,9 @@ void VACExtension::enforcePass2()
             }
         }
     }
-    //if (maxK == 0) {
-    //  maxK = wcsp->getUb() - wcsp->getLb();
-    //}
+    // if (maxK == 0) {
+    //   maxK = wcsp->getUb() - wcsp->getLb();
+    // }
     if (ToulBar2::verbose > 1)
         cout << "minLambda: " << minlambda << "\t\t (dualb = " << std::fixed << std::setprecision(ToulBar2::decimalPoint) << wcsp->getDDualBound() << ", primalb = " << wcsp->getDPrimalBound() << ")" << std::setprecision(DECIMAL_POINT) << endl;
 }
@@ -704,13 +739,13 @@ bool VACExtension::enforcePass3()
 {
     bool util = (minlambda >= UNIT_COST);
     /*if(util) {
-	   Cost ub = wcsp->getUb();
-	   Cost lb = wcsp->getLb();
-	   util = ( (ub - lb)/(10000*ToulBar2::costMultiplier) ) < minlambda;
-	   } */
+           Cost ub = wcsp->getUb();
+           Cost lb = wcsp->getLb();
+           util = ( (ub - lb)/(10000*ToulBar2::costMultiplier) ) < minlambda;
+           } */
     Cost lambda = minlambda;
 
-    //if (ToulBar2::verbose > 2) cout << "VAC Enforce Pass 3.   minlambda " << minlambda << " , var: " << inconsistentVariable << endl;
+    // if (ToulBar2::verbose > 2) cout << "VAC Enforce Pass 3.   minlambda " << minlambda << " , var: " << inconsistentVariable << endl;
     int i, j;
     VACVariable *xi, *xj;
     VACBinaryConstraint* cij;
@@ -758,8 +793,9 @@ bool VACExtension::enforcePass3()
         xj = (VACVariable*)wcsp->getVar(j);
         i = xj->getKiller(w);
         xi = (VACVariable*)wcsp->getVar(i);
-        cij = (VACBinaryConstraint*)xi->getConstr(xj);
+        cij = (VACBinaryConstraint*)xi->getConstrNotDuplicate(xj);
         assert(cij);
+        assert(!cij->isDuplicate());
 
         int xjk = xj->getK(w, nbIterations);
         if (maxk < xjk)
@@ -829,14 +865,14 @@ void VACExtension::iniSingleton()
     for (unsigned int i = 0; i < wcsp->numberOfVariables(); i++) {
         int size = wcsp->getDomainSize(i);
         for (int a = 0; a < size; a++)
-            singletonI.insert(MAX_DOMAIN_SIZE * i + a);
+            singletonI.insert(wcsp->getMaxDomainSize() * i + a);
     }
 }
 
 void VACExtension::updateSingleton()
 {
-    set<int>& s1 = singleton;
-    set<int> s2(singletonI);
+    set<Long>& s1 = singleton;
+    set<Long> s2(singletonI);
     singletonI.clear();
     set_intersection(s1.begin(), s1.end(),
         s2.begin(), s2.end(),
@@ -846,11 +882,11 @@ void VACExtension::updateSingleton()
 
 void VACExtension::removeSingleton()
 {
-    set<int>& s = singletonI;
-    set<int>::iterator it = s.begin();
+    set<Long>& s = singletonI;
+    set<Long>::iterator it = s.begin();
     while (it != s.end()) {
-        int ivar = *it / MAX_DOMAIN_SIZE;
-        Value a = *it % MAX_DOMAIN_SIZE;
+        int ivar = *it / wcsp->getMaxDomainSize();
+        Value a = *it % wcsp->getMaxDomainSize();
         Variable* var = wcsp->getVar(ivar);
         var->remove(a);
         var->queueNC();
@@ -901,7 +937,7 @@ void VACExtension::queueSeekSupport(DLink<VariableWithTimeStamp>* link)
 void VACExtension::printStat(bool ini)
 {
     if (ToulBar2::verbose >= 1) {
-        long double mean = to_double(sumlb) / (long double)nlb;
+        Double mean = to_double(sumlb) / (Double)nlb;
         cout << "VAC mean lb/incr: " << std::setprecision(DECIMAL_POINT) << mean << "     total increments: " << nlb
              << "     cyclesize: " << (double)sumvars / (double)nlb << "     k: " << (double)sumk / (double)nlb << " (mean), " << theMaxK << " (max)" << endl;
     }
@@ -915,15 +951,15 @@ void VACExtension::printStat(bool ini)
         cout << "Number of VAC iterations: " << nbIterations << endl;
         cout << "Number of times is VAC: " << ToulBar2::nbTimesIsVAC << " Number of times isvac and itThreshold > 1: " << ToulBar2::nbTimesIsVACitThresholdMoreThanOne << endl;
     }
-    //sort(heap.begin(), heap.end(), cmp_function);
+    // sort(heap.begin(), heap.end(), cmp_function);
     /*cout << "Vars: ";
-	   vector<tVACStat*>::iterator it = heap.begin();
-	   while(it != heap.end()) {
-	   tVACStat* v = *it;
-	   if(v->sumlb != MIN_COST) cout << "(" << v->var << "," << v->sumlb << ") ";
-	   ++it;
-	   }
-	   cout << endl; */
+           vector<tVACStat*>::iterator it = heap.begin();
+           while(it != heap.end()) {
+           tVACStat* v = *it;
+           if(v->sumlb != MIN_COST) cout << "(" << v->var << "," << v->sumlb << ") ";
+           ++it;
+           }
+           cout << endl; */
 
     sumk = 0;
     theMaxK = 0;
@@ -983,7 +1019,7 @@ void VACExtension::minsumDiffusion()
                     }
                 }
             ntimes++;
-            //cout << "it " << ntimes << "   changed: " << nchanged << endl;
+            // cout << "it " << ntimes << "   changed: " << nchanged << endl;
         }
         if (ToulBar2::verbose >= 0)
             cout << "   done iterations: " << ntimes << endl;
