@@ -7,9 +7,6 @@
 
 #include "utils/tb2queue.hpp"
 
-#define INCREMENTALVAC
-//#define AC2001
-
 class tVACStat;
 class VACVariable;
 class VACBinaryConstraint;
@@ -88,8 +85,8 @@ public:
     void histogram(Cost c);
     void histogram(); /**< Computes the ScaleVAC splitting the cost scale in 20 buckets or less */
 
-    set<int> singletonI;
-    set<int> singleton;
+    set<Long> singletonI;
+    set<Long> singleton;
 
     void iniSingleton();
     void updateSingleton();
