@@ -10,7 +10,7 @@
 
 /*
  * Constructors and misc.
- * 
+ *
  */
 
 Variable::Variable(WCSP* w, string n, Value iinf, Value isup)
@@ -24,15 +24,14 @@ Variable::Variable(WCSP* w, string n, Value iinf, Value isup)
     , fulleac((isup > iinf) ? 0 : 1)
     , constrs(&Store::storeConstraint)
     ,
-    //triangles(&Store::storeConstraint),
+    // triangles(&Store::storeConstraint),
     maxCost(MIN_COST)
     , maxCostValue(iinf)
     , NCBucket(-1)
-    , cluster(-1)
 {
     if (Store::getDepth() > 0) {
         cerr << "You cannot create a variable during the search!" << endl;
-        exit(EXIT_FAILURE);
+        throw InternalError();
     }
     w->link(this);
 
@@ -44,6 +43,7 @@ Variable::Variable(WCSP* w, string n, Value iinf, Value isup)
     linkIncDecQueue.content.incdec = NOTHING_EVENT;
     linkEliminateQueue.content.var = this;
     linkEliminateQueue.content.timeStamp = -1;
+    cluster = -1;
     isSep_ = false;
 }
 
@@ -77,62 +77,21 @@ void Variable::setCurrentVarId(int idx)
     timestamp = wcsp->getNbNodes();
 }
 
-int cmpConstraint(const void* p1, const void* p2)
-{
-    DLink<ConstraintLink>* c1 = *((DLink<ConstraintLink>**)p1);
-    DLink<ConstraintLink>* c2 = *((DLink<ConstraintLink>**)p2);
-    int v1 = c1->content.constr->getSmallestVarIndexInScope(c1->content.scopeIndex);
-    int v2 = c2->content.constr->getSmallestVarIndexInScope(c2->content.scopeIndex);
-    if (v1 < v2)
-        return -1;
-    else if (v1 > v2)
-        return 1;
-    else
-        return 0;
-}
-
-int cmpConstraintDAC(const void* p1, const void* p2)
-{
-    DLink<ConstraintLink>* c1 = *((DLink<ConstraintLink>**)p1);
-    DLink<ConstraintLink>* c2 = *((DLink<ConstraintLink>**)p2);
-    int v1 = c1->content.constr->getSmallestDACIndexInScope(c1->content.scopeIndex);
-    int v2 = c2->content.constr->getSmallestDACIndexInScope(c2->content.scopeIndex);
-    if (v1 < v2)
-        return 1;
-    else if (v1 > v2)
-        return -1;
-    else
-        return 0;
-}
-
-int cmpConstraintTightness(const void* p1, const void* p2)
-{
-    DLink<ConstraintLink>* c1 = *((DLink<ConstraintLink>**)p1);
-    DLink<ConstraintLink>* c2 = *((DLink<ConstraintLink>**)p2);
-    double v1 = c1->content.constr->getTightness();
-    double v2 = c2->content.constr->getTightness();
-    if (v1 < v2)
-        return 1;
-    else if (v1 > v2)
-        return -1;
-    else
-        return 0;
-}
-
 void Variable::sortConstraints()
 {
-    int size = constrs.getSize();
-    DLink<ConstraintLink>** sorted = new DLink<ConstraintLink>*[size]; // replace size by MAX_DOMAIN_SIZE in case of compilation problem
-    int i = 0;
+    vector<DLink<ConstraintLink>*> sorted;
     for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
-        sorted[i++] = iter.getElt();
+        sorted.push_back(iter.getElt());
     }
-    qsort(sorted, size, sizeof(DLink<ConstraintLink>*), cmpConstraintDAC);
-    for (int i = 0; i < size; i++) {
+    if (abs(ToulBar2::constrOrdering) == CONSTR_ORDER_RANDOM) {
+        shuffle(sorted.begin(), sorted.end(), myrandom_generator);
+    } else {
+        stable_sort(sorted.begin(), sorted.end(), Constraint::cmpConstraintLink);
+    }
+    for (unsigned int i = 0; i < sorted.size(); i++) {
         constrs.erase(sorted[i], true);
         constrs.push_back(sorted[i], true);
     }
-    delete[] sorted;
 }
 
 void Variable::deconnect(DLink<ConstraintLink>* link, bool reuse)
@@ -152,6 +111,14 @@ void Variable::deconnect(DLink<ConstraintLink>* link, bool reuse)
     }
 }
 
+void Variable::deconnect()
+{
+    for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
+        (*iter).constr->deconnect();
+    }
+    assign(getSupport());
+}
+
 int Variable::getTrueDegree()
 {
     //	if (constrs.getSize() >= ToulBar2::weightedDegree) return getDegree(); ///\warning returns an approximate degree if the constraint list is too large!
@@ -214,15 +181,6 @@ Long Variable::getWeightedDegree()
     return res;
 }
 
-void Variable::resetWeightedDegree()
-{
-    for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
-        if ((*iter).constr->isSep())
-            continue;
-        (*iter).constr->resetConflictWeight();
-    }
-}
-
 void Variable::conflict()
 {
     wcsp->conflict();
@@ -230,7 +188,7 @@ void Variable::conflict()
 
 /*
  * Propagation methods
- * 
+ *
  */
 
 void Variable::queueNC()
@@ -257,7 +215,7 @@ void Variable::changeNCBucket(int newBucket)
 {
     if (NCBucket != newBucket) {
         if (ToulBar2::verbose >= 3)
-            cout << "changeNCbucket " << getName() << ": " << NCBucket << " -> " << newBucket << endl;
+            cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] changeNCbucket " << getName() << ": " << NCBucket << " -> " << newBucket << endl;
         wcsp->changeNCBucket(NCBucket, newBucket, &linkNCBucket);
         NCBucket = newBucket;
     }
@@ -282,7 +240,7 @@ void Variable::projectLB(Cost cost)
     if (cost == MIN_COST)
         return;
     if (ToulBar2::verbose >= 2)
-        cout << "lower bound increased " << wcsp->getLb() << " -> " << wcsp->getLb() + cost << endl;
+        cout << "[" << Store::getDepth() << ",W" << wcsp->getIndex() << "] lower bound increased " << wcsp->getLb() << " -> " << wcsp->getLb() + cost << endl;
     if (cost < MIN_COST) {
         wcsp->decreaseLb(cost);
     } else {
@@ -307,6 +265,17 @@ void Variable::propagateIncDec(int incdec)
     }
 }
 
+// returns true if there is a global cost function currently linked to this variable
+bool Variable::isGlobal()
+{
+    for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
+        if ((*iter).constr->isGlobal()) {
+            return true;
+        }
+    }
+    return false;
+}
+
 // Looks for the constraint that links this variable with x
 BinaryConstraint* Variable::getConstr(Variable* x)
 {
@@ -337,6 +306,36 @@ BinaryConstraint* Variable::getConstr(Variable* x)
     return NULL;
 }
 
+// Looks for the constraint that links this variable with x and which is not a duplicated constraint
+BinaryConstraint* Variable::getConstrNotDuplicate(Variable* x)
+{
+    BinaryConstraint* ctr2;
+    TernaryConstraint* ctr3;
+    for (ConstraintList::iterator iter = constrs.begin(); iter != constrs.end(); ++iter) {
+        if ((*iter).constr->isDuplicate() || (*iter).constr->isSep() || (*iter).constr->isGlobal())
+            continue;
+
+        if ((*iter).constr->isBinary()) {
+            ctr2 = (BinaryConstraint*)(*iter).constr;
+            if (ctr2->getIndex(x) >= 0)
+                return ctr2;
+        } else if ((*iter).constr->isTernary()) {
+            ctr3 = (TernaryConstraint*)(*iter).constr;
+            int idx = ctr3->getIndex(x);
+            if (idx >= 0) {
+                int idt = (*iter).scopeIndex;
+                if ((0 != idx) && (0 != idt))
+                    return ctr3->yz;
+                else if ((1 != idx) && (1 != idt))
+                    return ctr3->xz;
+                else
+                    return ctr3->xy;
+            }
+        }
+    }
+    return NULL;
+}
+
 BinaryConstraint* Variable::getConstr(Variable* x, int cid)
 {
     BinaryConstraint* res;
@@ -507,6 +506,32 @@ double Variable::strongLinkedby(Variable*& strvar, TernaryConstraint*& tctr1max,
     return maxtight;
 }
 
+// take into account the current tree decomposition of adaptive BTD
+bool Variable::isSep()
+{
+    if (ToulBar2::heuristicFreedom) {
+        isSep_ = false;
+
+        TSepLink::iterator it;
+
+        it = clusters.begin();
+
+        while (it != clusters.end()) {
+            int c = (*it).first;
+            if (wcsp->getTreeDec()->getCluster(c)->getIsCurrInTD()) {
+                isSep_ = true;
+                return true;
+            } else {
+                ++it;
+            }
+        }
+
+        return false;
+    } else {
+        return isSep_;
+    }
+}
+
 ostream& operator<<(ostream& os, Variable& var)
 {
     os << var.name; // << " #" << var.dac;
