@@ -34,16 +34,16 @@ class RegularFlowConstraint;
 
 class WCSP FINAL : public WeightedCSP {
     static int wcspCounter; ///< count the number of instances of WCSP class
+    static vector<Cost> pow10Cache; ///< precomputed powers of 10 as Costs
     int instance; ///< instance number
     string name; ///< problem name
     void* solver; ///< special hook to access solver information
     StoreCost lb; ///< current problem lower bound
     Cost ub; ///< current problem upper bound
     StoreCost negCost; ///< shifting value to be added to problem lowerbound when computing the partition function
+    StoreValue reentrant; ///< avoid recursive calls in WCSP::propagate
     vector<Variable*> vars; ///< list of all variables
     vector<Variable*> divVariables; ///< list of variables submitted to diversity requirements
-    vector<map<int, int>> divVarsId; // vector[j][idx] = index of the dual variable that encodes the diversity constraint on sol j at position idx
-    vector<map<int, int>> divHVarsId; // vector[j][idx] = index of the hidden variable that encodes the diversity constraint on sol j between idx/idx+1
     vector<Value> bestValues; ///< hint for some value ordering heuristics (ONLY used by RDS)
     vector<Value> solution; ///< remember last solution found
     vector<pair<Double, vector<Value>>> solutions; ///< remember all solutions found
@@ -65,7 +65,7 @@ class WCSP FINAL : public WeightedCSP {
     Long nbNodes; ///< current number of calls to propagate method (roughly equal to number of search nodes), used as a time-stamp by Queue methods
     Long nbDEE; ///< number of value removals due to DEE
     Constraint* lastConflictConstr; ///< hook for last conflict variable heuristic
-    int maxdomainsize; ///< maximum initial domain size found in all variables
+    unsigned int maxdomainsize; ///< maximum initial domain size found in all variables
     vector<GlobalConstraint*> globalconstrs; ///< a list of all original global constraints (also inserted in constrs)
     vector<int> delayedNaryCtr; ///< a list of all original nary constraints in extension (also inserted in constrs)
     bool isDelayedNaryCtr; ///< postpone naryctr propagation after all variables have been created
@@ -79,6 +79,7 @@ class WCSP FINAL : public WeightedCSP {
     friend class VACExtension;
 
 public:
+    static map<int, WCSP*> CollectionOfWCSP;
     /// \brief variable elimination information used in backward phase to get a solution during search
     /// \warning restricted to at most two neighbor variables
     typedef struct {
@@ -91,13 +92,13 @@ public:
         Constraint* ctr; ///< corresponding sum/join cost function if using generic variable elimination
     } elimInfo;
 
-    /// \brief used for preprocesssTernary to make a list of potential ternary constraints with assciated information
+    /// \brief used for preprocesssTernary to make a list of potential ternary constraints with associated information
     struct TripleVarCostSize {
         EnumeratedVariable* x;
         EnumeratedVariable* y;
         EnumeratedVariable* z;
         float meancost;
-        long unsigned int size;
+        size_t size;
 
         bool operator<(const TripleVarCostSize& a) const
         {
@@ -116,6 +117,9 @@ public:
 
     VACExtension* vac; ///< link to VAC management system
 
+    vector<set<int>> varsBLP; ///< internal sets of variables inside problems (Problem1, Problem2, NegProblem2) waiting for tree decomposition initialization in bilevel optimization
+    vector<vector<int>> delayedCtrBLP; ///< internal lists of deconnected ternary constraints between clusters waiting for tree decomposition initialization in bilevel optimization
+
 #ifdef XMLFLAG
     map<int, int> varsDom; ///< structures for solution translation: we don't have to parse the XML file again
     vector<vector<int>> Doms; ///< structures for solution translation: we don't have to parse the XML file again
@@ -129,19 +133,22 @@ public:
     // General API for weighted CSP global constraint
 
     int getIndex() const { return instance; } ///< \brief instantiation occurrence number of current WCSP object
-    string getName() const { return (name.size()>0)?name:"problem"; }
+    string getName() const { return (name.size() > 0) ? name : "problem"; }
     void setName(const string& problem) { name = problem; }
     void* getSolver() const { return solver; }
+    void setSolver(void* solver) { this->solver = solver; }
 
     Cost getLb() const { return lb; } ///< \brief gets problem internal lower bound
     Cost getUb() const { return ub; } ///< \brief gets problem internal upper bound
 
-    Double getDDualBound() const { return Cost2ADCost(lb); } ///< \brief gets problem dual bound as a Double representing a decimal cost (upper resp. lower bound for minimization resp. maximization)
-    Double getDPrimalBound() const { return Cost2ADCost(ub); } ///< \brief gets problem primal bound as a Double representing a decimal cost (lower resp. upper bound for minimization resp. maximization)
+    Double getDDualBound() const { return Cost2ADCost(lb); } ///< \brief gets problem dual bound as a Double representing a decimal cost (lower resp. upper bound for minimization resp. maximization)
+    Double getDPrimalBound() const { return Cost2ADCost(ub); } ///< \brief gets problem primal bound as a Double representing a decimal cost (upper resp. lower bound for minimization resp. maximization)
 
     Double getDUb() const { return (ToulBar2::costMultiplier < 0 ? Cost2ADCost(lb) : Cost2ADCost(ub)); } ///< \brief gets problem upper bound as a Double representing a decimal cost
     Double getDLb() const { return (ToulBar2::costMultiplier < 0 ? Cost2ADCost(ub) : Cost2ADCost(lb)); } ///< \brief gets problem lower bound as a Double representing a decimal cost
 
+    void updateDUb(Double newDUb) { updateUb(DoubletoCost(newDUb)); } ///< \brief sets problem upper bound as a Double representing a decimal cost
+
     void setLb(Cost newLb) { lb = newLb; } ///< \internal sets problem lower bound
     void setUb(Cost newUb) { ub = newUb; } ///< \internal sets problem upper bound
 
@@ -202,6 +209,9 @@ public:
     /// \warning to be used in preprocessing only
     void setInfiniteCost();
 
+    /// \brief returns true if any complete assignment using current domains is a valid tuple with finite cost (i.e., cost strictly less than the problem upper bound minus the lower bound)
+    bool isfinite() const;
+
     void decreaseLb(Cost cost)
     {
         negCost += cost;
@@ -211,7 +221,16 @@ public:
     bool enumerated(int varIndex) const { return vars[varIndex]->enumerated(); } ///< \brief true if the variable has an enumerated domain
 
     string getName(int varIndex) const { return vars[varIndex]->getName(); } ///< \note by default, variables names are integers, starting at zero
-    unsigned int getVarIndex(const string& s) const { int i = std::distance(vars.begin(), find_if(vars.begin(), vars.end(), [&s](const Variable *var){return (var->getName()==s);})); assert (i >= 0); return static_cast<unsigned>(i); }
+    unsigned int getVarIndex(const string& s) const
+    {
+        int i = std::distance(vars.begin(), find_if(vars.begin(), vars.end(), [&s](const Variable* var) { return (var->getName() == s); }));
+        assert(i >= 0);
+        if (ToulBar2::bilevel == 3 && i >= (int)varsBLP[0].size()) { // Does not find variable names specific to Problem2 when reading NegProblem2
+            return INT_MAX;
+        } else {
+            return static_cast<unsigned int>(i);
+        }
+    }
     Value getInf(int varIndex) const { return vars[varIndex]->getInf(); } ///< \brief minimum current domain value
     Value getSup(int varIndex) const { return vars[varIndex]->getSup(); } ///< \brief maximum current domain value
     Value getValue(int varIndex) const { return vars[varIndex]->getValue(); } ///< \brief current assigned value \warning undefined if not assigned yet
@@ -251,7 +270,7 @@ public:
     {
         assert(vars[varIndex]->enumerated());
         return ((EnumeratedVariable*)vars[varIndex])->toIndex(valueName);
-    }///< \brief gets index from value name (warning! assumes EnumeratedVariable)
+    } ///< \brief gets index from value name (warning! assumes EnumeratedVariable)
     int getDACOrder(int varIndex) const { return vars[varIndex]->getDACOrder(); } ///< \brief index of the variable in the DAC variable ordering
     void updateCurrentVarsId(); ///< \brief determines the position of each variable in the current list of unassigned variables (see \ref WCSP::dump)
 
@@ -309,7 +328,17 @@ public:
             propagate();
     }
 
-    Cost getUnaryCost(int varIndex, Value v) const { return vars[varIndex]->getCost(v); } ///< \brief unary cost associated to a domain value
+    /// \brief deconnects a set of variables from the rest of the problem and assigns them to their support value (used by Incremental Search)
+    /// \param varIndexes vector of variable indexes as returned by makeXXXVariable
+    void deconnect(vector<int>& varIndexes)
+    {
+        for (unsigned int index : varIndexes) {
+            assert(index >= 0 && index < numberOfVariables());
+            vars[index]->deconnect();
+        }
+    }
+
+    Cost getUnaryCost(int varIndex, Value v) const { return (vars[varIndex]->canbe(v)) ? vars[varIndex]->getCost(v) : MAX_COST; } ///< \brief unary cost associated to a domain value
     Cost getMaxUnaryCost(int varIndex) const { return vars[varIndex]->getMaxCost(); } ///< \brief maximum unary cost in the domain
     Value getMaxUnaryCostValue(int varIndex) const { return vars[varIndex]->getMaxCostValue(); } ///< \brief a value having the maximum unary cost in the domain
     Value getSupport(int varIndex) const { return vars[varIndex]->getSupport(); } ///< \brief unary (NC/EAC) support value
@@ -321,14 +350,16 @@ public:
     int getDegree(int varIndex) const { return vars[varIndex]->getDegree(); } ///< \brief approximate degree of a variable (\e ie number of active cost functions, see \ref varelim)
     int getTrueDegree(int varIndex) const { return vars[varIndex]->getTrueDegree(); } ///< \brief degree of a variable
     Long getWeightedDegree(int varIndex) const { return vars[varIndex]->getWeightedDegree(); } ///< \brief weighted degree heuristic
-    void resetWeightedDegree(int varIndex) { vars[varIndex]->resetWeightedDegree(); } ///< \brief initialize weighted degree heuristic
+    void resetWeightedDegree(); ///< \brief initialize weighted degree heuristic
+    void resetTightness(); ///< \brief initialize constraint tightness used by some heuristics (including weighted degree)
+    void resetTightnessAndWeightedDegree(); ///< \brief initialize tightness and weighted degree heuristics
     void revise(Constraint* c) { lastConflictConstr = c; } ///< \internal last conflict heuristic
     /// \internal last conflict heuristic
     void conflict()
     {
         if (lastConflictConstr) {
             if (ToulBar2::verbose >= 2)
-                cout << "Last conflict on " << *lastConflictConstr << endl;
+                cout << "[" << Store::getDepth() << ",W" << getIndex() << "] Last conflict on " << *lastConflictConstr << endl;
             lastConflictConstr->incConflictWeight(lastConflictConstr);
             lastConflictConstr = NULL;
         }
@@ -364,27 +395,51 @@ public:
     //  }
 
     void whenContradiction(); ///< \brief after a contradiction, resets propagation queues and increases \ref WCSP::nbNodes
-    void propagate(); ///< \brief propagates until a fix point is reached (or throws a contradiction) and then increases \ref WCSP::nbNodes
-    bool verify(); ///< \brief checks the propagation fix point is reached \warning might change EAC supports
+    void deactivatePropagate() { reentrant = true; } ///< \brief forbids propagate calls
+    bool isactivatePropagate() { return reentrant == false; } ///< \brief are propagate calls authorized?
+    void reactivatePropagate() { reentrant = false; } ///< \brief re-authorizes propagate calls
+    void propagate(bool fromscratch = false); ///< \brief (if authorized) propagates until a fix point is reached (or throws a contradiction) and then increases \ref WCSP::nbNodes. If fromscratch is true then propagates every cost function at least once.
+    bool verify(); ///< \brief checks the propagation fix point is correctly reached \warning might change EAC supports
+    bool propagated(); ///< \brief returns true if the propagation fix point is reached
+
+    bool isFullEAC(int varIndex) const { return vars[varIndex]->isFullEAC(); } ///< \brief returns true if variable varIndex is full EAC
+    /// \brief returns true if all unassigned variables are full EAC, i.e., a valid solution can be directly extracted from EAC supports with cost equal to current lower bound (plus VAC thresholds if VAC is used)
+    bool isFullEAC() const
+    {
+        for (unsigned int i = 0; i < vars.size(); i++) {
+            if (unassigned(i) && !isFullEAC(i)) {
+                return false;
+            }
+        }
+        return true;
+    }
 
     unsigned int numberOfVariables() const { return vars.size(); } ///< \brief current number of created variables
     /// \brief returns current number of unassigned variables
     unsigned int numberOfUnassignedVariables() const
     {
-        int res = 0;
+        unsigned int res = 0;
         for (unsigned int i = 0; i < vars.size(); i++)
             if (unassigned(i))
                 res++;
         return res;
     }
+    unsigned int numberOfHiddenVariables() const
+    {
+        int res = 0;
+        for (unsigned int i = 0; i < vars.size(); i++)
+            if (getName(i).rfind(HIDDEN_VAR_TAG, 0) == 0)
+                res++;
+        return res;
+    }
     unsigned int numberOfConstraints() const { return constrs.size(); } ///< \brief initial number of cost functions
     unsigned int numberOfConnectedConstraints() const; ///< \brief current number of cost functions
     unsigned int numberOfConnectedBinaryConstraints() const; ///< \brief current number of binary cost functions
     unsigned int medianDomainSize() const; ///< \brief median current domain size of variables
     unsigned int medianDegree() const; ///< \brief median current degree of variables
     unsigned int medianArity() const; ///< \brief median arity of current cost functions
-    int getMaxDomainSize() const { return maxdomainsize; } ///< \brief maximum initial domain size found in all variables
-    int getMaxCurrentDomainSize() const; ///< \brief maximum current domain size found in all variables
+    unsigned int getMaxDomainSize() const { return maxdomainsize; } ///< \brief maximum initial domain size found in all variables
+    unsigned int getMaxCurrentDomainSize() const; ///< \brief maximum current domain size found in all variables
     unsigned int getDomainSizeSum() const; ///< \brief total sum of current domain sizes
     /// \brief Cartesian product of current domain sizes
     /// \param cartesianProduct result obtained by the GNU Multiple Precision Arithmetic Library GMP
@@ -408,66 +463,98 @@ public:
 #endif
 
     int makeEnumeratedVariable(string n, Value iinf, Value isup);
-    int makeEnumeratedVariable(string n, Value* d, int dsize);
+    int makeEnumeratedVariable(string n, vector<Value>& dom);
     void addValueName(int xIndex, const string& name);
+    const string& getValueName(int xIndex, Value value) { return vars[xIndex]->getValueName((vars[xIndex]->enumerated()) ? toIndex(xIndex, value) : WRONG_VAL); }
     int makeIntervalVariable(string n, Value iinf, Value isup);
 
+    // a limited number of cost functions accept floating-point costs directly and are able to disappear when backtrack occurs (used in incremental search)
     void postNullaryConstraint(Double cost);
+    void postUnaryConstraint(int xIndex, vector<Double>& costs, bool incremental = false);
+    int postBinaryConstraint(int xIndex, int yIndex, vector<Double>& costs, bool incremental = false);
+    int postTernaryConstraint(int xIndex, int yIndex, int zIndex, vector<Double>& costs, bool incremental = false);
+
     void postNullaryConstraint(Cost cost);
     void postUnary(int xIndex, vector<Cost>& costs);
     int postUnary(int xIndex, Value* d, int dsize, Cost penalty);
-    void postUnaryConstraint(int xIndex, vector<Double>& costs, bool incremental = false);
     void postUnaryConstraint(int xIndex, vector<Cost>& costs) { postUnary(xIndex, costs); }
     void postIncrementalUnaryConstraint(int xIndex, vector<Cost>& costs) { postUnary(xIndex, costs); }
     int postUnaryConstraint(int xIndex, Value* d, int dsize, Cost penalty) { return postUnary(xIndex, d, dsize, penalty); }
     int postSupxyc(int xIndex, int yIndex, Value cst, Value deltamax = MAX_VAL - MIN_VAL);
     int postDisjunction(int xIndex, int yIndex, Value cstx, Value csty, Cost penalty);
     int postSpecialDisjunction(int xIndex, int yIndex, Value cstx, Value csty, Value xinfty, Value yinfty, Cost costx, Cost costy);
-    int postBinaryConstraint(int xIndex, int yIndex, vector<Double>& costs, bool incremental = false);
     int postBinaryConstraint(int xIndex, int yIndex, vector<Cost>& costs);
     int postIncrementalBinaryConstraint(int xIndex, int yIndex, vector<Cost>& costs);
-    int postTernaryConstraint(int xIndex, int yIndex, int zIndex, vector<Double>& costs, bool incremental = false);
     int postTernaryConstraint(int xIndex, int yIndex, int zIndex, vector<Cost>& costs);
     int postIncrementalTernaryConstraint(int xIndex, int yIndex, int zIndex, vector<Cost>& costs);
-    int postNaryConstraintBegin(vector<int>& scope, Cost defval, Long nbtuples = 0, bool forcenary = false) { return postNaryConstraintBegin(scope.data(), scope.size(), defval, nbtuples, forcenary); }
-    int postNaryConstraintBegin(int* scopeIndex, int arity, Cost defval, Long nbtuples = 0, bool forcenary = false); /// \warning must call postNaryConstraintEnd after giving cost tuples ; \warning it may create a WeightedClause instead of NaryConstraint
+    int postNaryConstraintBegin(vector<int>& scope, Cost defval, Long nbtuples = 0, bool forcenary = !NARY2CLAUSE) { return postNaryConstraintBegin(scope.data(), scope.size(), defval, nbtuples, forcenary); }
+    int postNaryConstraintBegin(int* scopeIndex, int arity, Cost defval, Long nbtuples = 0, bool forcenary = !NARY2CLAUSE); /// \warning must call postNaryConstraintEnd after giving cost tuples ; \warning it may create a WeightedClause instead of NaryConstraint
     void postNaryConstraintTuple(int ctrindex, vector<Value>& tuple, Cost cost) { postNaryConstraintTuple(ctrindex, tuple.data(), tuple.size(), cost); }
     void postNaryConstraintTuple(int ctrindex, Value* tuple, int arity, Cost cost);
-    void postNaryConstraintTuple(int ctrindex, const Tuple& tuple, Cost cost);
+    void postNaryConstraintTupleInternal(int ctrindex, const Tuple& tuple, Cost cost);
     void postNaryConstraintEnd(int ctrindex);
 
     // -----------------------------------------------------------
     // Methods for diverse solutions
     // -----------------------------------------------------------
 
-    void addDivConstraint(const vector<Value> solution, int sol_id, Cost cost); // to look for the (j+1)-th solution, with j = sol_id
-    void addHDivConstraint(const vector<Value> solution, int sol_id, Cost cost);
-    void addTDivConstraint(const vector<Value> solution, int sol_id, Cost cost);
+    vector<map<int, int>> divVarsId; // vector[j][idx] = index of the dual variable that encodes the diversity constraint on sol j at variable index idx
+    vector<map<int, int>> divHVarsId; // vector[j][idx] = index of the hidden variable that encodes the diversity constraint on sol j at variable index idx
+    void addDivConstraint(vector<Variable*>& divVars, unsigned int distance, map<int, Value>& solution, map<int, int>& divVarsIdMap, bool incremental = false); // to look for the (j+1)-th solution, with j = sol_id
+    void addHDivConstraint(vector<Variable*>& divVars, unsigned int distance, map<int, Value>& solution, map<int, int>& divVarsIdMap, map<int, int>& divHVarsIdMap, bool incremental = false);
+    void addTDivConstraint(vector<Variable*>& divVars, unsigned int distance, map<int, Value>& solution, map<int, int>& divHVarsIdMap, bool incremental = false);
     void addMDDConstraint(Mdd mdd, int relaxed);
     void addHMDDConstraint(Mdd mdd, int relaxed);
     void addTMDDConstraint(Mdd mdd, int relaxed);
 
-    const vector<Variable*>& getDivVariables()
+    vector<Variable*>& getDivVariables()
     {
         return divVariables;
     }
 
-    int postCliqueConstraint(vector<int>& scope, const string& arguments)
+    void initDivVariables()
     {
-        istringstream file(arguments);
+        divVariables.clear();
+        for (auto var : vars) {
+            if (var->unassigned() && var->getName().rfind(IMPLICIT_VAR_TAG, 0) != 0) {
+                if (var->enumerated()) {
+                    divVariables.push_back(var);
+                } else {
+                    cerr << "Error: cannot control diversity of non enumerated variable: " << var->getName() << endl;
+                    throw BadConfiguration();
+                }
+            }
+        }
+    }
+
+    int postCliqueConstraint(vector<int> scope, const string& arguments)
+    {
+        std::istringstream file(arguments);
         return postCliqueConstraint(scope.data(), scope.size(), file);
     }
-    int postCliqueConstraint(int* scopeIndex, int arity, istream& file);
+    int postCliqueConstraint(int* scopeIndex, int arity, istream& file); // warning! scopeIndex may be modified internally.
 
-    int postKnapsackConstraint(vector<int>& scope, const string& arguments)
+#ifdef BOOST
+    void addAMOConstraints();
+#endif
+
+    int postKnapsackConstraint(vector<int> scope, const string& arguments, bool isclique = false, int kp = 0, bool conflict = false)
     {
         istringstream file(arguments);
-        return postKnapsackConstraint(scope.data(), scope.size(), file);
+        return postKnapsackConstraint(scope.data(), scope.size(), file, isclique, kp, conflict);
     }
-    int postKnapsackConstraint(int* scopeIndex, int arity, istream& file);
-    int postGlobalConstraint(int* scopeIndex, int arity, const string& gcname, istream& file, int* constrcounter = NULL, bool mult = true); ///< \deprecated should use WCSP::postGlobalCostFunction instead \warning does not work for arity below 4 (use binary or ternary cost functions instead)
+    int postKnapsackConstraint(int* scopeIndex, int arity, istream& file, bool isclique, int kp, bool conflict); // warning! scopeIndex may be modified internally.
 
+    int postWeightedCSPConstraint(vector<int> scope, WeightedCSP* problem, WeightedCSP* negproblem, Cost lb = MIN_COST, Cost ub = MAX_COST, bool duplicateHard = false, bool strongDuality = false);
+
+    int postGlobalConstraint(int* scopeIndex, int arity, const string& gcname, istream& file, int* constrcounter = NULL, bool mult = true); ///< \deprecated should use WCSP::postGlobalCostFunction instead \warning does not work for arity below 4 (use binary or ternary cost functions instead)
     GlobalConstraint* postGlobalCostFunction(int* scopeIndex, int arity, const string& name, int* constrcounter = NULL);
+    void postGlobalFunction(vector<int> scope, const string& gcname, const string& arguments)
+    {
+        istringstream file(arguments);
+        postGlobalFunction(scope.data(), scope.size(), gcname, file);
+    }
+    void postGlobalFunction(int* scopeIndex, int arity, const string& gcname, istream& file, int* constrcounter = NULL, bool mult = true);
 
     int postWAmong(vector<int>& scope, const string& semantics, const string& propagator, Cost baseCost, const vector<Value>& values, int lb, int ub) { return postWAmong(scope.data(), scope.size(), semantics, propagator, baseCost, values, lb, ub); } ///< \brief post a soft among cost function
     int postWAmong(int* scopeIndex, int arity, const string& semantics, const string& propagator, Cost baseCost, const vector<Value>& values, int lb, int ub); ///< \deprecated
@@ -485,7 +572,8 @@ public:
         const vector<WeightedObjInt>& accepting_States,
         const vector<DFATransition>& Wtransitions); ///< \deprecated
     void postWRegular(int* scopeIndex, int arity, int nbStates, vector<pair<int, Cost>> initial_States, vector<pair<int, Cost>> accepting_States, int** Wtransitions, vector<Cost> transitionsCosts); ///< \deprecated post a weighted regular cost function decomposed as a cost function network
-    int postWAllDiff(int* scopeIndex, int arity, const string& semantics, const string& propagator, Cost baseCost); ///< \brief post a soft alldifferent cost function
+    int postWAllDiff(vector<int>& scope, const string& semantics, const string& propagator, Cost baseCost) { return postWAllDiff(scope.data(), scope.size(), semantics, propagator, baseCost); } ///< \brief post a soft alldifferent cost function
+    int postWAllDiff(int* scopeIndex, int arity, const string& semantics, const string& propagator, Cost baseCost); ///< \deprecated
     void postWAllDiff(int* scopeIndex, int arity, string semantics, Cost baseCost); ///< \deprecated post a soft alldifferent cost function decomposed as a cost function network
     int postWGcc(int* scopeIndex, int arity, const string& semantics, const string& propagator, Cost baseCost,
         const vector<BoundedObjValue>& values); ///< \brief post a soft global cardinality cost function
@@ -502,9 +590,14 @@ public:
         const vector<WeightedVarValPair> weightFunction); ///< \brief post a weighted max cost function (maximum cost of a set of unary cost functions associated to a set of variables)
     void postWSum(int* scopeIndex, int arity, string semantics, Cost baseCost, string comparator, int rightRes); ///< \brief post a soft linear constraint with unit coefficients
     void postWVarSum(int* scopeIndex, int arity, string semantics, Cost baseCost, string comparator, int varIndex); ///< \brief post a soft linear constraint with unit coefficients and variable right-hand side
-    void postWOverlap(int* scopeIndex, int arity, string semantics, Cost baseCost, string comparator, int rightRes); /// \brief post a soft overlap cost function (a group of variables being point-wise equivalent -- and not equal to zero -- to another group with the same size)
+    void postWOverlap(int* scopeIndex, int arity, string semantics, Cost baseCost, string comparator, int rightRes); ///< \brief post a soft overlap cost function (a group of variables being point-wise equivalent -- and not equal to zero -- to another group with the same size)
+    void postWDivConstraint(vector<int>& scope, unsigned int distance, vector<Value>& values, int method = 0);
 
+    bool isKnapsack(); ///< \brief true if there are knapsack constraints defined in the problem
     bool isGlobal() { return (globalconstrs.size() > 0); } ///< \brief true if there are soft global cost functions defined in the problem
+#ifdef ILOGCPLEX
+    bool isPLPS();
+#endif
 
     Cost read_wcsp(const char* fileName); ///< \brief load problem in any of all formats managed by tb2. Return the global UB known to the solver at start (file and command line).
     void read_uai2008(const char* fileName); ///< \brief load problem in UAI 2008 format (see http://graphmod.ics.uci.edu/uai08/FileFormat and http://www.cs.huji.ac.il/project/UAI10/fileFormat.php) \warning UAI10 evidence file format not recognized by toulbar2 as it does not allow multiple evidence (you should remove the first value in the file)
@@ -512,7 +605,8 @@ public:
     void read_wcnf(const char* fileName); ///< \brief load problem in (w)cnf format (see http://www.maxsat.udl.cat/08/index.php?disp=requirements)
     void read_qpbo(const char* fileName); ///< \brief load quadratic pseudo-Boolean optimization problem in unconstrained quadratic programming text format (first text line with n, number of variables and m, number of triplets, followed by the m triplets (x,y,cost) describing the sparse symmetric nXn cost matrix with variable indexes such that x <= y and any positive or negative real numbers for costs)
     void read_opb(const char* fileName); ///< \brief load pseudo-Boolean optimization problem
-    void read_legacy(const char* fileName); ///< \brief common ending section for all readers
+    void read_legacy(const char* fileName); ///< \brief load problem in wcsp format
+    void read_legacy(istream& file); ///< \brief load problem in wcsp format (internal use)
 
     void read_XML(const char* fileName); ///< \brief load problem in XML format (see http://www.cril.univ-artois.fr/~lecoutre/benchmarks.html)
     void solution_XML(bool opt = false); ///< \brief output solution in Max-CSP 2008 output format
@@ -538,90 +632,115 @@ public:
                 setBestValue(i, v);
             solution[i] = ((ToulBar2::sortDomains && ToulBar2::sortedDomains.find(i) != ToulBar2::sortedDomains.end()) ? ToulBar2::sortedDomains[i][toIndex(i, v)].value : v);
         }
-        solutions.push_back(make_pair(Cost2ADCost(solutionCost), solution));
+        if (!ToulBar2::allSolutions || ToulBar2::allSolutions < LONGLONG_MAX) {
+            solutions.push_back(make_pair(Cost2ADCost(solutionCost), solution));
+        }
     }
     void printSolution()
     {
         for (unsigned int i = 0; i < numberOfVariables(); i++) {
+            bool printed = false;
             if (enumerated(i) && ((EnumeratedVariable*)getVar(i))->isValueNames()) {
                 EnumeratedVariable* myvar = (EnumeratedVariable*)getVar(i);
                 Value myvalue = solution[i];
                 string valuelabel = myvar->getValueName(myvar->toIndex(myvalue));
                 string varlabel = myvar->getName();
 
-                switch (ToulBar2::showSolutions) {
-                case 1:
-                    cout << myvalue;
-                    break;
-                case 2:
-                    cout << valuelabel;
-                    break;
-                case 3:
-                    cout << varlabel << "=" << valuelabel;
-                    break;
-                default:
-                    break;
+                if (ToulBar2::showHidden || (varlabel.rfind(HIDDEN_VAR_TAG, 0) != 0)) {
+                    switch (ToulBar2::showSolutions) {
+                    case 1:
+                        printed = true;
+                        cout << myvalue;
+                        break;
+                    case 2:
+                        printed = true;
+                        cout << valuelabel;
+                        break;
+                    case 3:
+                        printed = true;
+                        cout << varlabel << "=" << valuelabel;
+                        break;
+                    default:
+                        break;
+                    }
                 }
             } else {
+                printed = true;
                 cout << solution[i];
             }
-            cout << (i < numberOfVariables() - 1 ? " " : "");
+            if (printed && i < numberOfVariables() - 1)
+                cout << " ";
         }
     }
     void printSolution(ostream& os)
     {
         for (unsigned int i = 0; i < numberOfVariables(); i++) {
+            bool printed = false;
             if (enumerated(i) && ((EnumeratedVariable*)getVar(i))->isValueNames()) {
                 EnumeratedVariable* myvar = (EnumeratedVariable*)getVar(i);
                 Value myvalue = solution[i];
                 string valuelabel = myvar->getValueName(myvar->toIndex(myvalue));
                 string varlabel = myvar->getName();
 
-                switch (ToulBar2::writeSolution) {
-                case 1:
-                    os << myvalue;
-                    break;
-                case 2:
-                    os << valuelabel;
-                    break;
-                case 3:
-                    os << varlabel << "=" << valuelabel;
-                    break;
-                default:
-                    break;
+                if (ToulBar2::showHidden || (varlabel.rfind(HIDDEN_VAR_TAG, 0) != 0)) {
+                    switch (ToulBar2::writeSolution) {
+                    case 1:
+                        printed = true;
+                        os << myvalue;
+                        break;
+                    case 2:
+                        printed = true;
+                        os << valuelabel;
+                        break;
+                    case 3:
+                        printed = true;
+                        os << varlabel << "=" << valuelabel;
+                        break;
+                    default:
+                        break;
+                    }
                 }
             } else {
+                printed = true;
                 os << solution[i];
             }
-            os << (i < numberOfVariables() - 1 ? " " : "");
+            if (printed && i < numberOfVariables() - 1)
+                os << " ";
         }
     }
     void printSolution(FILE* f)
     {
         for (unsigned int i = 0; i < numberOfVariables(); i++) {
+            bool printed = false;
             if (enumerated(i) && ((EnumeratedVariable*)getVar(i))->isValueNames()) {
                 EnumeratedVariable* myvar = (EnumeratedVariable*)getVar(i);
                 Value myvalue = solution[i];
                 string valuelabel = myvar->getValueName(myvar->toIndex(myvalue));
                 string varlabel = myvar->getName();
 
-                switch (ToulBar2::writeSolution) {
-                case 1:
-                    fprintf(f, "%d", myvalue);
-                    break;
-                case 2:
-                    fprintf(f, "%s", valuelabel.c_str());
-                    break;
-                case 3:
-                    fprintf(f, "%s=%s", varlabel.c_str(), valuelabel.c_str());
-                    break;
-                default:
-                    break;
+                if (ToulBar2::showHidden || (varlabel.rfind(HIDDEN_VAR_TAG, 0) != 0)) {
+                    switch (ToulBar2::writeSolution) {
+                    case 1:
+                        printed = true;
+                        fprintf(f, "%d", myvalue);
+                        break;
+                    case 2:
+                        printed = true;
+                        fprintf(f, "%s", valuelabel.c_str());
+                        break;
+                    case 3:
+                        printed = true;
+                        fprintf(f, "%s=%s", varlabel.c_str(), valuelabel.c_str());
+                        break;
+                    default:
+                        break;
+                    }
                 }
             } else {
+                printed = true;
                 fprintf(f, "%d", solution[i]);
             }
-            if (i < numberOfVariables() - 1)
+            if (printed && i < numberOfVariables() - 1)
                 fprintf(f, " ");
         }
     }
@@ -714,6 +833,10 @@ public:
     /// \warning to be done \b only before the search
     void preprocessing();
 
+    /// \brief shrink domains to current domain bounds, reducing cost function memory usage
+    /// \warning to be done \b only before the search (not backtrackable) and preferably at the end of the preprocessing
+    void shrink();
+
     // -----------------------------------------------------------
     // Methods for Variable Elimination
 
@@ -745,10 +868,12 @@ public:
     void ternaryCompletion();
     bool kconsistency(int xIndex, int yIndex, int zIndex, BinaryConstraint* xy, BinaryConstraint* yz, BinaryConstraint* xz);
 
+    pair<vector<EnumeratedVariable*>, vector<BinaryConstraint*>> hiddenEncoding(); ///< \brief returns dual variables and channeling constraints for pairwise consistency
+
     // -----------------------------------------------------------
     // Data and methods for Virtual Arc Consistency
 
-    void histogram(Cost c); /// \brief initializes histogram of costs used by Virtual Arc Consistency to speed up its convergence (Bool\f$_\theta\f$ of P)
+    void histogram(Cost c); ///< \brief initializes histogram of costs used by Virtual Arc Consistency to speed up its convergence (Bool\f$_\theta\f$ of P)
     void iniSingleton();
     void updateSingleton();
     void removeSingleton();
@@ -763,6 +888,7 @@ public:
     void buildTreeDecomposition();
     void elimOrderFile2Vector(char* elimVarOrderFilename, vector<int>& elimVarOrder); ///< \brief returns a reverse topological order from a variable elimination order
     void treeDecFile2Vector(char* treeDecFilename, vector<int>& elimVarOrder); ///< \brief returns a reverse topological order from a tree decomposition
+    vector<int> getBergeDecElimOrder(); ///< \brief return an elimination order compatible with Berge acyclic decomposition of global decomposable cost functions (if possible keep reverse of previous DAC order)
     void setDACOrder(vector<int>& elimVarOrder); ///< \brief change DAC order and propagate from scratch
 
     // dac order reordering when Berge acyclic gobal constraint are present in the wcsp
@@ -772,11 +898,17 @@ public:
     // -----------------------------------------------------------
     // Functions dealing with all representations of Costs
     // warning: ToulBar2::NormFactor has to be initialized
-
+    pair<Cost, int> Decimal2Cost(const string& decimalToken, const unsigned int lineNumber) const;
     Cost decimalToCost(const string& decimalToken, const unsigned int lineNumber) const;
-    Cost DoubletoCost(const Double& c) const { return Round(c * powl(10.0, ToulBar2::decimalPoint)) + negCost; }
+    Cost DoubletoCost(const Double& c) const { return (Cost)min((Double)(MAX_COST - negCost), roundl(c * pow10Cache[ToulBar2::decimalPoint])) + negCost; }
     Double Cost2ADCost(const Cost& c) const { return Cost2RDCost(c - negCost); } // Absolute costs
-    Double Cost2RDCost(const Cost& c) const { return ((Double)(c) / Exp10(ToulBar2::decimalPoint) / ToulBar2::costMultiplier); } //Relative costs
+    Double Cost2RDCost(const Cost& c) const { return ((Double)(c) / pow10Cache[ToulBar2::decimalPoint] / ToulBar2::costMultiplier); } // Relative costs
+    std::string DCost2Decimal(const Double& c)
+    {
+        std::stringstream ss;
+        ss << std::fixed << std::setprecision(ToulBar2::decimalPoint) << c << std::setprecision(DECIMAL_POINT);
+        return ss.str();
+    }
     Cost Prob2Cost(TProb p) const;
     TProb Cost2Prob(Cost c) const;
     TLogProb Cost2LogProb(Cost c) const;
