@@ -31,7 +31,7 @@ void GlobalCardinalityConstraint::buildIndex()
         mapval[*i] = arity_ + (int)(i - D.begin()) + 1;
     }
     nDistinctDomainValue = D.size();
-    //graph.setSize(arity_+D.size()+4);
+    // graph.setSize(arity_+D.size()+4);
 }
 
 void GlobalCardinalityConstraint::read(istream& file, bool mult)
@@ -41,24 +41,24 @@ void GlobalCardinalityConstraint::read(istream& file, bool mult)
     // "wdec" => sigmadec
     string str;
     int nvalues;
-    //int sumlow = 0, sumhigh = 0;
+    // int sumlow = 0, sumhigh = 0;
     file >> str;
-    //JP Start// alteration
+    // JP Start// alteration
     /*mode = EMPTY;
-	if (strcmp(str.c_str(), "var") 	== 0) mode = VAR;
-	if (strcmp(str.c_str(), "dec") 	== 0) mode = VALUE;
-	if (strcmp(str.c_str(), "wdec") == 0) mode = WVALUE;
-	if (mode == EMPTY) {
-		cerr << "Error occurred in reading gcc() : No violation measure" << endl;
-		exit(1);
-	}*/
+        if (strcmp(str.c_str(), "var") 	== 0) mode = VAR;
+        if (strcmp(str.c_str(), "dec") 	== 0) mode = VALUE;
+        if (strcmp(str.c_str(), "wdec") == 0) mode = WVALUE;
+        if (mode == EMPTY) {
+                cerr << "Error occurred in reading gcc() : No violation measure" << endl;
+                throw WrongFileFormat();
+        }*/
     setSemantics(str);
-    //JP End//
+    // JP End//
     file >> def;
     if (mult)
         def *= ToulBar2::costMultiplier;
     file >> nvalues;
-    //JP End//
+    // JP End//
     for (int i = 0; i < nvalues; i++) {
         int d, high, low;
         file >> d >> low >> high;
@@ -66,7 +66,7 @@ void GlobalCardinalityConstraint::read(istream& file, bool mult)
             cerr << "Error occurred in reading gcc: upper bound " << high << " smaller than lower bound " << low << endl;
             THROWCONTRADICTION;
         }
-        //JP Start//
+        // JP Start//
         Cost wshortage = def;
         Cost wexcess = def;
         if (mode == WVALUE) {
@@ -76,11 +76,11 @@ void GlobalCardinalityConstraint::read(istream& file, bool mult)
                 wexcess *= ToulBar2::costMultiplier;
             }
         }
-        //JP End//
+        // JP End//
         bound[d] = make_pair(high, low);
         weights[d] = make_pair(wshortage, wexcess);
-        //sumlow += low;
-        //sumhigh += high;
+        // sumlow += low;
+        // sumhigh += high;
     }
 }
 
@@ -129,25 +129,25 @@ Cost GlobalCardinalityConstraint::evalOriginal(const Tuple& s)
     }
     for (map<Value, pair<int, int>>::iterator i = bound.begin(); i != bound.end(); i++) {
         if (appear[i->first] < i->second.lower_bound) {
-            //JP Start// Alteration
+            // JP Start// Alteration
             Cost lshortage = i->second.lower_bound - appear[i->first];
             shortage += lshortage;
             cost += lshortage * weights[i->first].first;
-            //JP End//
+            // JP End//
         }
         if (appear[i->first] > i->second.upper_bound) {
-            //JP Start// Alteration
+            // JP Start// Alteration
             Cost lexcess = appear[i->first] - i->second.upper_bound;
             excess += lexcess;
             cost += lexcess * weights[i->first].second;
-            //JP End//
+            // JP End//
         }
     }
-    //JP Start// Alteration
+    // JP Start// Alteration
     if (mode == VAR) {
         cost = (excess > shortage) ? excess * def : shortage * def;
     }
-    //JP End//
+    // JP End//
     return cost;
 }
 
@@ -165,7 +165,7 @@ void GlobalCardinalityConstraint::buildGraph(Graph& g)
     int ss = n - 1;
     int st = n - 2;
 
-    //g.clearEdge();
+    // g.clearEdge();
     g.addEdge(t, 0, 0, INF);
     g.addEdge(0, st, 0, arity_);
 
@@ -197,11 +197,11 @@ void GlobalCardinalityConstraint::buildGraph(Graph& g)
         }
     } else {
         for (map<Value, int>::iterator i = mapval.begin(); i != mapval.end(); i++) {
-            //JP Start// Alteration
+            // JP Start// Alteration
             if (bound[i->first].lower_bound > 0)
                 g.addEdge(0, i->second, weights[i->first].first, bound[i->first].lower_bound);
             g.addEdge(i->second, t, weights[i->first].second, arity_);
-            //JP End//
+            // JP End//
         }
     }
 }
@@ -209,53 +209,53 @@ void GlobalCardinalityConstraint::buildGraph(Graph& g)
 Cost GlobalCardinalityConstraint::constructFlow(Graph& g)
 {
 
-    //cout << "use the one\n";
+    // cout << "use the one\n";
     /*pair<int, bool> result;
-	int n = g.size();
-	int ss = n-1;
-	int st = n-2;
-	int fcost = 0;
-	//int fcost = -projectedCost;
+        int n = g.size();
+        int ss = n-1;
+        int st = n-2;
+        int fcost = 0;
+        //int fcost = -projectedCost;
 
-	do {
-		int minc = 0;
-		result = g.augment(ss, st, true, minc);
-		if (result.second) fcost += minc*result.first;
-	} while (result.second);*/
-    //checker(g, fcost);
+        do {
+                int minc = 0;
+                result = g.augment(ss, st, true, minc);
+                if (result.second) fcost += minc*result.first;
+        } while (result.second);*/
+    // checker(g, fcost);
 
     pair<int, Cost> result = g.minCostFlow(g.size() - 1, g.size() - 2);
     return result.second;
-    //return fcost;
+    // return fcost;
 }
 
 /*void GlobalCardinalityConstraint::getDomainFromGraph(Graph &graph, int varindex, vector<int> &domain) {
 
-	domain.clear();
-	for (vector<List_Node >::iterator k = graph[varindex+1].begin();
-			k != graph[varindex+1].end(); k++) {
-		if (k->adj > 0) {
-			for (map<Value, int>::iterator i = mapval.begin();i !=
-					mapval.end();i++) {
-				if (i->second == k->adj) domain.push_back(i->first);
-			}
-		}
-	}
-	for (map<Value, int>::iterator i = mapval.begin();i !=
-			mapval.end();i++) {
-		for (vector<List_Node >::iterator k = graph[i->second].begin();
-				k != graph[i->second].end(); k++) {
-			if (k->adj == varindex+1) {
-				domain.push_back(i->first);
-			}
-		}
-	}
+        domain.clear();
+        for (vector<List_Node >::iterator k = graph[varindex+1].begin();
+                        k != graph[varindex+1].end(); k++) {
+                if (k->adj > 0) {
+                        for (map<Value, int>::iterator i = mapval.begin();i !=
+                                        mapval.end();i++) {
+                                if (i->second == k->adj) domain.push_back(i->first);
+                        }
+                }
+        }
+        for (map<Value, int>::iterator i = mapval.begin();i !=
+                        mapval.end();i++) {
+                for (vector<List_Node >::iterator k = graph[i->second].begin();
+                                k != graph[i->second].end(); k++) {
+                        if (k->adj == varindex+1) {
+                                domain.push_back(i->first);
+                        }
+                }
+        }
 
 }*/
 
 void GlobalCardinalityConstraint::dump(ostream& os, bool original)
 {
-    assert(original); //TODO: case original is false
+    assert(original); // TODO: case original is false
     int nvalues = 0;
     if (original) {
         os << arity_;
@@ -302,26 +302,26 @@ string GlobalCardinalityConstraint::getName()
     return name;
 }
 
-//void GlobalCardinalityConstraint::print(ostream& os) {
-//    int nvalues = 0;
+// void GlobalCardinalityConstraint::print(ostream& os) {
+//     int nvalues = 0;
 //
-//    os << "sgcc(";
-//    for(int i = 0; i < arity_;i++) {
-//        os << scope[i]->wcspIndex;
-//        if(i < arity_-1) os << ",";
-//    }
-//    for (map<Value, pair<int,int> >::iterator i = bound.begin(); i !=	bound.end();i++) nvalues++;
-//    os << ")[" ;
-//    if (mode == VAR   ) os << "var";
-//    if (mode == VALUE ) os << "dec";
-//    if (mode == WVALUE) os << "wdec";
-//    os << "," << def << "," << nvalues;
-//    for (map<Value, pair<int,int> >::iterator i = bound.begin(); i !=	bound.end();i++) {
-//        os << "," << i->first << "," << i->second.lower_bound << "," << i->second.upper_bound;
-//        if (mode == WVALUE) os << "," << weights[i->first].first << "," << weights[i->first].second;
-//    }
-//    os << "]";
-//}
+//     os << "sgcc(";
+//     for(int i = 0; i < arity_;i++) {
+//         os << scope[i]->wcspIndex;
+//         if(i < arity_-1) os << ",";
+//     }
+//     for (map<Value, pair<int,int> >::iterator i = bound.begin(); i !=	bound.end();i++) nvalues++;
+//     os << ")[" ;
+//     if (mode == VAR   ) os << "var";
+//     if (mode == VALUE ) os << "dec";
+//     if (mode == WVALUE) os << "wdec";
+//     os << "," << def << "," << nvalues;
+//     for (map<Value, pair<int,int> >::iterator i = bound.begin(); i !=	bound.end();i++) {
+//         os << "," << i->first << "," << i->second.lower_bound << "," << i->second.upper_bound;
+//         if (mode == WVALUE) os << "," << weights[i->first].first << "," << weights[i->first].second;
+//     }
+//     os << "]";
+// }
 
 /* Local Variables: */
 /* c-basic-offset: 4 */
