@@ -44,13 +44,14 @@ void LPSConstraint::read(istream& file, bool mult)
     cout << "Warning! slinear global cost function skipped... (recompile ToulBar2 with ILOGCPLEX flag)" << endl;
     deconnect();
     if (arity_ != (int)wcsp->numberOfVariables())
-        exit(1);
+        throw WrongFileFormat();
     return;
 #endif
     nrows = 0;
     nslacks = 0;
     windowVars = (int**)malloc(sizeof(int*) * nwindows);
     group = (int**)malloc(sizeof(int*) * nwindows);
+    kpcoefs = (int***)malloc(sizeof(int**) * nwindows);
 
     string typeID, defstr;
     for (int i = 0; i < nwindows; i++) {
@@ -64,17 +65,19 @@ void LPSConstraint::read(istream& file, bool mult)
         file >> d;
         if (d != -1) {
             cerr << "Error occurred in reading slinear" << endl;
-            exit(1);
+            throw WrongFileFormat();
         } else {
             file >> typeID;
             windowType.push_back(typeID);
-            file >> defstr;
-            if (strcmp(defstr.c_str(), "def") == 0 || strcmp(defstr.c_str(), "var") == 0) {
-                file >> d;
-                subdef.push_back(d);
-            } else {
-                cerr << "Error occurred in reading slinear def|var" << endl;
-                exit(1);
+            if (strcmp(windowType[i].c_str(), "knapsack") != 0 && strcmp(windowType[i].c_str(), "knapsackp") != 0) {
+                file >> defstr;
+                if (strcmp(defstr.c_str(), "def") == 0 || strcmp(defstr.c_str(), "var") == 0) {
+                    file >> d;
+                    subdef.push_back(d);
+                } else {
+                    cerr << "Error occurred in reading slinear def|var" << endl;
+                    throw WrongFileFormat();
+                }
             }
             if (strcmp(windowType[i].c_str(), "salldiff") == 0) {
                 nrows += domainSize;
@@ -87,7 +90,7 @@ void LPSConstraint::read(istream& file, bool mult)
                 file >> low >> high >> nvalues;
                 if (high < low) {
                     cerr << "Error occurred in reading samong" << endl;
-                    exit(1);
+                    throw WrongFileFormat();
                 }
                 sumlow.push_back(low);
                 sumhigh.push_back(high);
@@ -109,7 +112,7 @@ void LPSConstraint::read(istream& file, bool mult)
                     file >> nvalues >> low >> high;
                     if (high < low) {
                         cerr << "Error occurred in reading sgcc" << endl;
-                        exit(1);
+                        throw WrongFileFormat();
                     }
                     group[i] = (int*)malloc(sizeof(int) * count2);
                     for (int k = 0; k < count2; k++) {
@@ -148,7 +151,7 @@ void LPSConstraint::read(istream& file, bool mult)
                                 group[i][k] = 0;
                             } else {
                                 cerr << "Error occurred in reading ssame" << endl;
-                                exit(1);
+                                throw WrongFileFormat();
                             }
                             break;
                         }
@@ -162,7 +165,7 @@ void LPSConstraint::read(istream& file, bool mult)
                                 group[i][k] = 1;
                             } else {
                                 cerr << "Error occurred in reading ssame" << endl;
-                                exit(1);
+                                throw WrongFileFormat();
                             }
                             break;
                         }
@@ -174,7 +177,7 @@ void LPSConstraint::read(istream& file, bool mult)
                 file >> low >> high >> nvalues;
                 if (high < low) {
                     cerr << "Error occurred in reading sum" << endl;
-                    exit(1);
+                    throw WrongFileFormat();
                 }
                 sumlow.push_back(low);
                 sumhigh.push_back(high);
@@ -186,11 +189,28 @@ void LPSConstraint::read(istream& file, bool mult)
                     file >> d;
                     group[i][j] = d;
                 }
+            } else if (strcmp(windowType[i].c_str(), "knapsack") == 0) {
+                nrows += 1;
+                file >> low;
+                sumlow.push_back(low);
+                sumhigh.push_back(std::numeric_limits<int>::max());
+                group[i] = NULL;
+                kpcoefs[i] = (int**)malloc(sizeof(int*) * windowSize[i]);
+                for (int j = 0; j < windowSize[i]; j++) {
+                    kpcoefs[i][j] = (int*)malloc(sizeof(int) * count2);
+                    for (int d = 0; d < count2; d++) {
+                        kpcoefs[i][j][d] = 0;
+                    }
+                }
+                for (int j = 0; j < windowSize[i]; j++) {
+                    file >> d;
+                    kpcoefs[i][j][1] = d;
+                }
             } else if (strcmp(windowType[i].c_str(), "segcc") == 0) {
                 nrows += 1;
                 nslacks += 2;
                 file >> d >> nvalues;
-                sumlow.push_back(d); //abused the sumlow array
+                sumlow.push_back(d); // abused the sumlow array
                 sumhigh.push_back(0);
                 group[i] = (int*)malloc(sizeof(int) * count2);
                 for (int j = 0; j < count2; j++) {
@@ -204,7 +224,7 @@ void LPSConstraint::read(istream& file, bool mult)
                 nrows += count2;
                 nslacks += count2;
                 file >> d;
-                sumlow.push_back(d); //abused the sumlow array
+                sumlow.push_back(d); // abused the sumlow array
                 sumhigh.push_back(0);
                 group[i] = (int*)malloc(sizeof(int) * d * 2);
                 for (int j = 0; j < d; j++) {
@@ -214,7 +234,7 @@ void LPSConstraint::read(istream& file, bool mult)
                 }
             } else {
                 cerr << "Error occurred in reading slinear: no linearization method for: " << typeID << endl;
-                exit(1);
+                throw WrongFileFormat();
             }
         }
     }
@@ -252,7 +272,7 @@ Cost LPSConstraint::evalOriginal(const Tuple& s)
                     appear[s[windowVars[i][j]]] -= subdef[i];
                 } else {
                     cerr << "Error occurred in reading ssame()" << endl;
-                    exit(1);
+                    throw WrongFileFormat();
                 }
             }
             int sum = 0;
@@ -270,11 +290,19 @@ Cost LPSConstraint::evalOriginal(const Tuple& s)
             } else if (tmpSum > sumhigh[i]) {
                 cost += subdef[i] * (tmpSum - sumhigh[i]);
             }
+        } else if (strcmp(windowType[i].c_str(), "knapsack") == 0) {
+            int tmpSum = 0;
+            for (int j = 0; j < windowSize[i]; j++) {
+                tmpSum += kpcoefs[i][j][s[windowVars[i][j]]];
+            }
+            if (tmpSum < sumlow[i]) {
+                cost = wcsp->getUb();
+            }
         } else if (strcmp(windowType[i].c_str(), "segcc") == 0) {
             int appear = 0;
-            int tmpSum = s[windowVars[i][sumlow[i]]]; //abused the sumlow array
+            int tmpSum = s[windowVars[i][sumlow[i]]]; // abused the sumlow array
             for (int j = 0; j < windowSize[i]; j++) {
-                //cout << s[j];
+                // cout << s[j];
                 if (group[i][s[windowVars[i][j]]]) {
                     appear++;
                 }
@@ -301,7 +329,7 @@ Cost LPSConstraint::evalOriginal(const Tuple& s)
             }
         } else {
             cerr << "Error occurred in evaloriginal: Unknown ID" << endl;
-            exit(1);
+            throw WrongFileFormat();
         }
     }
     return cost;
@@ -415,6 +443,19 @@ Cost LPSConstraint::buildMIP(MIP& mip)
             mip.rowCoeff(rowCount, count + nslacks, idxs, vars);
             mip.rowBound(rowCount, sumlow[i], sumhigh[i]);
             rowCount++;
+        } else if (strcmp(windowType[i].c_str(), "knapsack") == 0) {
+            for (int j = 0; j < count + nslacks; j++) {
+                vars[j] = 0;
+            }
+            for (int j = 0; j < windowSize[i]; j++) {
+                EnumeratedVariable* x = (EnumeratedVariable*)getVar(windowVars[i][j]);
+                for (EnumeratedVariable::iterator v = x->begin(); v != x->end(); ++v) {
+                    vars[mapvar[windowVars[i][j]][*v]] = kpcoefs[i][j][x->toIndex(*v)];
+                }
+            }
+            mip.rowCoeff(rowCount, count + nslacks, idxs, vars);
+            mip.rowBound(rowCount, sumlow[i], sumhigh[i]);
+            rowCount++;
         } else if (strcmp(windowType[i].c_str(), "segcc") == 0) {
             for (int j = 0; j < count + nslacks; j++) {
                 vars[j] = 0;
@@ -473,7 +514,7 @@ Cost LPSConstraint::buildMIP(MIP& mip)
             }
         } else {
             cerr << "Error occurred in building mip: Unknown ID" << windowType[i] << endl;
-            exit(1);
+            throw WrongFileFormat();
         }
     }
 
@@ -505,8 +546,6 @@ Cost LPSConstraint::solveMIP(MIP& mip)
 
 void LPSConstraint::dump(ostream& os, bool original)
 {
-    int nvalues = 1;
-
     if (original) {
         os << arity_;
         for (int i = 0; i < arity_; i++)
@@ -517,10 +556,26 @@ void LPSConstraint::dump(ostream& os, bool original)
             if (scope[i]->unassigned())
                 os << " " << scope[i]->getCurrentVarId();
     }
-    os << " -1 slinear" << endl
-       << ((mode == VAR) ? "var" : "dec") << " " << def << " " << nvalues << endl;
-
-    os << "bounds " << sumlow[0] << " " << sumhigh[0] << endl;
+    os << " -1 slinear " << nwindows << endl;
+    for (int i = 0; i < nwindows; i++) {
+        os << windowSize[i];
+        for (int j = 0; j < windowSize[i]; j++) {
+            os << " " << windowVars[i][j];
+        }
+        os << " -1 " << windowType[i];
+        if (windowType[i] == "knapsack") {
+            os << " " << sumlow[i];
+            for (int j = 0; j < windowSize[i]; j++) {
+                os << " " << kpcoefs[i][j][1];
+            }
+        } else {
+            os << " " << subdef[i] << " " << sumlow[i] << " " << sumhigh[i];
+            cerr << endl
+                 << "sorry, dump function for slinear not fully implemented!!!" << endl;
+            throw WrongFileFormat();
+        }
+        os << endl;
+    }
 }
 
 void LPSConstraint::print(ostream& os)
@@ -539,6 +594,8 @@ void LPSConstraint::print(ostream& os)
        << "bounds"
        << "," << sumlow[0] << "," << sumhigh[0];
 
+    os << "," << nwindows << "," << nrows << "," << nslacks;
+
     os << "]";
 }
 
