@@ -1,15 +1,15 @@
 /** \file tb2lpsconstr.hpp
- *  \brief Linear programming based global cost function : ssum, samong, sgcc, ssame, segcc, sdisjunction
+ *  \brief Linear programming based global cost function : ssum, samong, sgcc, ssame, segcc, sdisjunction, knapsack, knapsackp
  */
 
 #ifndef TB2LPSCONSTR_HPP_
 #define TB2LPSCONSTR_HPP_
 
-//#include "stddef.h"
+// #include "stddef.h"
 #include "tb2linearconstr.hpp"
 
-//#define upper_bound first
-//#define lower_bound second
+// #define upper_bound first
+// #define lower_bound second
 
 class LPSConstraint : public LinearConstraint {
 private:
@@ -18,7 +18,8 @@ private:
     vector<string> windowType;
     int** windowVars;
     int** group;
-    int count2;
+    int*** kpcoefs;
+    int count2; // number of different domain values
     int* wcspconstrcounter;
     void buildIndex();
     Cost buildMIP(MIP& mip);
@@ -27,15 +28,14 @@ private:
 public:
     static const int VALUE = 1;
     static const int VAR = 0;
-    LPSConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int
-                                                                 arity_in,
+    LPSConstraint(WCSP* wcsp, EnumeratedVariable** scope_in, int arity_in,
         int* constrcounter);
 
     ~LPSConstraint()
     {
     }
 
-    string getName() { return "LPS constraint"; }
+    string getName() { return "slinear_" + to_string(nwindows) + "_" + to_string(nrows) + "_" + to_string(nslacks); }
     Cost evalOriginal(const Tuple& s);
     virtual void read(istream& file, bool mult = true);
 
