@@ -10,6 +10,10 @@ using namespace std;
 #include <iostream>
 #include <fstream>
 
+#include "core/tb2naryconstr.hpp"
+#include "core/tb2wcsp.hpp"
+#include "search/tb2solver.hpp"
+
 #include "incop.h"
 #include "incoputil.h"
 #include "csproblem.h"
@@ -20,16 +24,22 @@ extern ofstream* ofile; // le fichier de sortie
 
 extern Stat_GWW* Statistiques;
 
-#include "search/tb2solver.hpp"
-#include "core/tb2wcsp.hpp"
-
 INCOP::NaryCSProblem::NaryCSProblem(int nbvar, int nbconst)
     : CSProblem(nbvar, nbconst)
 {
     ;
 }
 
-INCOP::NaryConstraint::NaryConstraint(int arit) { arity = arit; }
+INCOP::NaryConstraint::NaryConstraint(int arit)
+{
+    arity = arit;
+    constr = NULL;
+}
+INCOP::NaryConstraint::NaryConstraint(int arit, Constraint* ctr)
+{
+    arity = arit;
+    constr = ctr;
+}
 
 INCOP::NaryVariable::NaryVariable() { ; }
 
@@ -77,10 +87,18 @@ Long INCOP::NaryCSProblem::config_evaluation(Configuration* configuration)
 
 Long INCOP::NaryConstraint::constraint_value(Configuration* configuration)
 {
-    int index = 0;
-    for (int i = 0; i < arity; i++)
-        index += configuration->config[constrainedvariables[i]] * multiplyers[i];
-    return tuplevalues[index];
+    if (!constr) {
+        int index = 0;
+        for (int i = 0; i < arity; i++)
+            index += configuration->config[constrainedvariables[i]] * multiplyers[i];
+        return tuplevalues[index];
+    } else {
+        for (int i = 0; i < arity; i++) {
+            tuple[i] = index2index[i][configuration->config[constrainedvariables[i]]];
+        }
+        assert(constr->isNary() || !constr->extension());
+        return ((AbstractNaryConstraint*)constr)->eval(tuple);
+    }
 }
 
 void INCOP::NaryCSProblem::incr_update_conflicts(IncrCSPConfiguration* configuration, Move* move)
@@ -232,10 +250,11 @@ int wcspdata_constraint_read(WCSP* wcsp, int nbconst, vector<INCOP::NaryVariable
     vector<int>* connexions, vector<Value>* tabdomaines)
 {
     static Tuple tuple;
+    assert(wcsp->getUb() > wcsp->getLb());
     Cost gap = wcsp->getUb() - wcsp->getLb();
     int nbconst_ = 0;
     for (unsigned int i = 0; i < wcsp->numberOfConstraints(); i++) {
-        if (wcsp->getCtr(i)->connected() && !wcsp->getCtr(i)->isSep() && !wcsp->getCtr(i)->isGlobal() && wcsp->getCtr(i)->arity() <= ToulBar2::preprocessNary) {
+        if (wcsp->getCtr(i)->connected() && !wcsp->getCtr(i)->isSep() && wcsp->getCtr(i)->arity() <= ToulBar2::preprocessNary && wcsp->getCtr(i)->getDomainSizeProduct() <= MAX_NB_TUPLES) {
             int arity = 0;
             for (int j = 0; j < wcsp->getCtr(i)->arity(); j++)
                 if (wcsp->getCtr(i)->getVar(j)->unassigned())
@@ -257,6 +276,30 @@ int wcspdata_constraint_read(WCSP* wcsp, int nbconst, vector<INCOP::NaryVariable
                 ct->tuplevalues.push_back(min(gap, cost));
             }
             nbconst_++;
+        } else if (wcsp->getCtr(i)->connected() && !wcsp->getCtr(i)->isSep() && (wcsp->getCtr(i)->isNary() || !wcsp->getCtr(i)->extension())) {
+            int arity = 0;
+            for (int j = 0; j < wcsp->getCtr(i)->arity(); j++)
+                if (wcsp->getCtr(i)->getVar(j)->unassigned())
+                    arity++;
+            INCOP::NaryConstraint* ct = new INCOP::NaryConstraint(arity, wcsp->getCtr(i));
+            vct->push_back(ct);
+            int numvar = 0;
+            for (int j = 0; j < wcsp->getCtr(i)->arity(); j++) {
+                EnumeratedVariable* myvar = (EnumeratedVariable*)wcsp->getCtr(i)->getVar(j);
+                if (myvar->unassigned()) {
+                    numvar = myvar->getCurrentVarId();
+                    ct->constrainedvariables.push_back(numvar);
+                    (*vv)[numvar]->constraints.push_back(ct);
+                    ct->index2index.push_back(vector<tValue>());
+                    for (auto iter = myvar->begin(); iter != myvar->end(); ++iter) {
+                        ct->index2index.back().push_back(myvar->toIndex(*iter));
+                    }
+                }
+            }
+            assert(ct->constrainedvariables.size() == (unsigned int)arity);
+            assert(ct->index2index.size() == (unsigned int)arity);
+            ct->tuple.resize(arity, 0);
+            nbconst_++;
         }
     }
     for (int i = 0; i < wcsp->getElimBinOrder(); i++) {
@@ -340,14 +383,14 @@ int split(char* str, char c, char*** arr)
 
     *arr = (char**)malloc(sizeof(char*) * count);
     if (*arr == NULL)
-        exit(1);
+        throw InternalError();
 
     p = str;
     while (*p != '\0') {
         if (*p == c) {
             (*arr)[i] = (char*)malloc(sizeof(char) * token_len);
             if ((*arr)[i] == NULL)
-                exit(EXIT_FAILURE);
+                throw InternalError();
 
             token_len = 0;
             i++;
@@ -357,7 +400,7 @@ int split(char* str, char c, char*** arr)
     }
     (*arr)[i] = (char*)malloc(sizeof(char) * token_len);
     if ((*arr)[i] == NULL)
-        exit(EXIT_FAILURE);
+        throw InternalError();
 
     i = 0;
     p = str;
@@ -401,10 +444,13 @@ void removeSpaces(string& str)
 /// \return best solution cost found
 /// \param cmd command line argument for narycsp INCOP local search solver (cmd format: lowerbound randomseed nbiterations method nbmoves neighborhoodchoice neighborhoodchoice2 minnbneighbors maxnbneighbors  neighborhoodchoice3 autotuning tracemode)
 /// \param solution best solution assignment found (MUST BE INITIALIZED WITH A DEFAULT ASSIGNMENT)
-/// \warning cannot solve problems with global cost functions
 Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
 {
     Long result = MAX_COST;
+    Cost initialUpperBound = wcsp->getUb();
+    if (wcsp->isKnapsack() || wcsp->isGlobal()) {
+        wcsp->setUb(MAX_COST / MEDIUM_COST / MEDIUM_COST / MEDIUM_COST);
+    }
 
     string filename = "/dev/stdin";
     string outputfile = "/dev/stdout";
@@ -415,7 +461,7 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
     int argc = 0;
 
     // remove leading space  from incop command line
-    cmd.erase(cmd.begin(), std::find_if(cmd.begin(), cmd.end(), std::bind1st(std::not_equal_to<char>(), ' ')));
+    cmd.erase(0, cmd.find_first_not_of(" \t\n\r\f\v"));
 
     // remove multiples space in cmd
     removeSpaces(cmd);
@@ -438,7 +484,7 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
 
     // les divers arguments lus dans la ligne de commande
     int nbvar, nbconst, domsize;
-    Long lbound;
+    Long lbound = 0;
     int taille, nbessais;
     int graine1;
     int narg = 2; // compteur des arguments
@@ -524,8 +570,12 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
     else {
         // boucle sur les essais
         for (int nessai = 0; nessai < nbessais; nessai++) {
+            if (ToulBar2::interrupted) {
+                throw TimeOut();
+            }
             executer_essai(problem, algo, population, taille, graine1, nessai, &initconfig);
-            if (wcsp->getLb() + problem->best_config->valuation < upperbound) {
+            if (wcsp->getLb() + problem->best_config->valuation < min(upperbound, initialUpperBound)) {
+                Cost previousUb = MIN_COST;
                 int depth = Store::getDepth();
                 try {
                     Store::store();
@@ -533,6 +583,11 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
                     for (int i = 0; i < problem->best_config->nbvar; i++) {
                         solution[i] = tabdomaines[i][problem->best_config->config[i]];
                     }
+                    if (wcsp->getUb() > initialUpperBound) {
+                        previousUb = wcsp->getUb();
+                        wcsp->setUb(initialUpperBound);
+                        wcsp->enforceUb();
+                    }
                     wcsp->assignLS(tabvars, solution);
                     newSolution();
                     result = wcsp->getUb();
@@ -545,6 +600,9 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
                     wcsp->whenContradiction();
                 }
                 Store::restore(depth);
+                if (previousUb > MIN_COST) {
+                    wcsp->setUb(previousUb);
+                }
             }
         }
         // ecriture statistiques
@@ -555,8 +613,13 @@ Cost Solver::narycsp(string cmd, vector<Value>& bestsolution)
     delete[] tabdomaines;
     delete[] connexions;
 
-    wcsp->enforceUb();
-    wcsp->propagate();
+    if (wcsp->getUb() < initialUpperBound) {
+        wcsp->enforceUb();
+        wcsp->propagate();
+    } else {
+        assert(wcsp->getUb() == initialUpperBound || wcsp->isKnapsack() || wcsp->isGlobal());
+        wcsp->setUb(initialUpperBound);
+    }
 
     return result;
 }
