@@ -15,7 +15,7 @@ struct WTransition {
     unsigned int symbol;
     Cost weight;
 
-    WTransition(unsigned int _start, unsigned int _end, unsigned int _symbol, int _weight)
+    WTransition(unsigned int _start, unsigned int _end, unsigned int _symbol, Cost _weight)
     {
         start = _start;
         end = _end;
@@ -32,19 +32,19 @@ struct WTransition {
 class WFA {
 private:
     unsigned int nbStates;
-    list<pair<int, Cost> > initialStates;
-    list<pair<int, Cost> > acceptingStates;
+    list<pair<int, Cost>> initialStates;
+    list<pair<int, Cost>> acceptingStates;
     list<WTransition*> transitions;
 
 public:
     WFA();
     WFA(int _nbStates);
-    WFA(istream& file);
+    WFA(istream& file, bool mult = true);
     WFA(int nbSymbol, string forbiddenPattern, Cost cost);
 
     inline unsigned int getNbStates() { return nbStates; }
-    inline list<pair<int, Cost> >& getInitialStates() { return initialStates; }
-    inline list<pair<int, Cost> >& getAcceptingStates() { return acceptingStates; }
+    inline list<pair<int, Cost>>& getInitialStates() { return initialStates; }
+    inline list<pair<int, Cost>>& getAcceptingStates() { return acceptingStates; }
     inline list<WTransition*>& getTransitions() { return transitions; }
 
     void display();
