@@ -25,6 +25,8 @@ public:
     void generateNaryCtr(vector<int>& indexs, long nogoods, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
     void generateTernCtr(int i, int j, int k, long p, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
     void generateBinCtr(int i, int j, long p, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateNotEqual(int i, int j, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
+    void generateVertexCover(int i, int j);
     void generateSubModularBinCtr(int i, int j, Cost costMin = SMALL_COST, Cost costMax = LARGE_COST);
     void Input(int in_n, int in_m, vector<int>& p, bool forceSubModular = false, string globalname = "");
 
