@@ -9,108 +9,6 @@
 #include "core/tb2wcsp.hpp"
 #ifdef OPENMPI
 
-// Conversion Tools
-// Solution to Message
-void ReplicatedParallelDGVNS::SolToMsg(
-    MPIEnv& env0, int cluster, int k, int discrepancy, Cost bestUb,
-    map<int, Value>& bestSolution)
-{
-    int i = 0;
-    env0.sendbuff[i] = cluster;
-    i++;
-    env0.sendbuff[i] = k;
-    i++;
-    env0.sendbuff[i] = discrepancy;
-    i++;
-    stringstream nfile;
-    nfile << bestUb;
-    string temp = nfile.str();
-    env0.sendbuff[i] = temp.size();
-    i++;
-    for (string::iterator it = temp.begin(); it != temp.end(); ++it) {
-        env0.sendbuff[i] = *it - '0';
-        i++;
-    }
-    nfile.clear();
-    for (map<int, Value>::iterator it = bestSolution.begin();
-         it != bestSolution.end(); ++it) {
-        env0.sendbuff[i] = it->second;
-        i++;
-    }
-}
-
-void ReplicatedParallelDGVNS::SolToMsg2(
-    MPIEnv& env0, Cost bestUb, map<int, Value>& bestSolution)
-{
-    int i = 0;
-    stringstream ss_bestUb;
-    ss_bestUb << bestUb;
-    string temp = ss_bestUb.str();
-    env0.sendbuff[i] = temp.size();
-    i++;
-    for (string::iterator it = temp.begin(); it != temp.end(); ++it) {
-        env0.sendbuff[i] = *it - '0';
-        i++;
-    }
-    ss_bestUb.clear();
-    for (map<int, Value>::iterator it = bestSolution.begin();
-         it != bestSolution.end(); ++it) {
-        env0.sendbuff[i] = it->second;
-        i++;
-    }
-}
-
-//Message to solution
-void ReplicatedParallelDGVNS::MsgToSol(
-    MPIEnv& env0, int nov, int& cluster, int& k, int& discrepancy,
-    Cost& bestUb, map<int, Value>& bestSolution)
-{
-    int j = 0;
-    cluster = env0.recvbuff[j];
-    j++;
-    k = env0.recvbuff[j];
-    j++;
-    discrepancy = env0.recvbuff[j];
-    j++;
-    uint size = env0.recvbuff[j];
-    j++;
-    bestUb = env0.recvbuff[j];
-    j++;
-    for (uint it = 1; it < size; it++) {
-        bestUb = (bestUb * 10) + env0.recvbuff[j];
-        j++;
-    }
-    for (int i = 0; i < nov; i++) {
-        bestSolution[i] = env0.recvbuff[j];
-        j++;
-    }
-}
-
-void ReplicatedParallelDGVNS::MsgToSol2(
-    MPIEnv& env0, int nov, Cost& bestUb, map<int, Value>& bestSolution)
-{
-    int j = 0;
-    uint size = env0.recvbuff[0];
-    j++;
-    bestUb = env0.recvbuff[1];
-    j++;
-    for (uint it = 1; it < size; it++) {
-        bestUb = (bestUb * 10) + env0.recvbuff[j];
-        j++;
-    }
-    for (int i = 0; i < nov; i++) {
-        bestSolution[i] = env0.recvbuff[j];
-        j++;
-    }
-}
-//---------------- Class Definition --------------------------//
-void timeOut()
-{
-    MPI_Abort(MPI_COMM_WORLD, 0);
-    //    MPI_Finalize();
-    //    exit(0);
-}
-
 bool ReplicatedParallelDGVNS::solve(bool first)
 {
     // Initialization
@@ -126,20 +24,18 @@ bool ReplicatedParallelDGVNS::solve(bool first)
         }
     } catch (const Contradiction&) {
         wcsp->whenContradiction();
-        if (env0.myrank == 0) {
+        if (world.rank() == MASTER) {
             if (lastUb < MAX_COST)
                 wcsp->setSolution(lastUb, &lastSolution);
             endSolve(lastUb < MAX_COST, lastUb, true);
         }
-        /* Shut down MPI */
-        MPI_Finalize();
         return (lastUb < MAX_COST);
     }
 
-    assert((int)wcsp->numberOfUnassignedVariables() == unassignedVars->getSize());
+    assert((int)wcsp->numberOfUnassignedVariables() >= unassignedVars->getSize());
     ToulBar2::vnsLDSmax = min(ToulBar2::vnsLDSmax, (int)wcsp->getDomainSizeSum() - (int)wcsp->numberOfUnassignedVariables());
     ToulBar2::vnsLDSmin = min(ToulBar2::vnsLDSmin, ToulBar2::vnsLDSmax);
-    ToulBar2::vnsKmax = min(ToulBar2::vnsKmax, (int)wcsp->numberOfUnassignedVariables());
+    ToulBar2::vnsKmax = min(ToulBar2::vnsKmax, unassignedVars->getSize());
     ToulBar2::vnsKmin = min(ToulBar2::vnsKmin, ToulBar2::vnsKmax);
     assert(ToulBar2::vnsLDSmin >= 0);
     assert(ToulBar2::vnsLDSmax >= 0);
@@ -150,16 +46,13 @@ bool ReplicatedParallelDGVNS::solve(bool first)
 
     bool complete = false;
 
-    mysrand(abs(ToulBar2::seed) + env0.myrank);
+    mysrand(abs(ToulBar2::seed) + world.rank());
 
-    MPI_Barrier(MPI_COMM_WORLD); /* IMPORTANT */
-    startTime = MPI_Wtime();
-    ToulBar2::timeOut = timeOut;
+    world.barrier(); /* IMPORTANT */
 
-    env0.buffsize = (int)wcsp->numberOfVariables() + 108; // 3 : cluster + k + cost, second time, msecond time, localtime,the rest is the size of solution
-    env0.sendbuff = new int[env0.buffsize];
-    env0.recvbuff = new int[env0.buffsize];
-    if (env0.myrank == 0) {
+    ToulBar2::startRealTimeAfterPreProcessing = realTime();
+
+    if (world.rank() == MASTER) {
         if (!ToulBar2::vnsParallelSync) {
             complete = radgvns();
             endSolve(bestUb < MAX_COST, bestUb, complete);
@@ -179,16 +72,15 @@ bool ReplicatedParallelDGVNS::solve(bool first)
         ToulBar2::vnsOutput.close(); // close the output file
 
     /* Shut down MPI */
-    MPI_Barrier(MPI_COMM_WORLD); /* IMPORTANT */
-    double elapsedTime = MPI_Wtime() - startTime;
+    world.barrier(); /* IMPORTANT */
+    double elapsedTime = realTime() - ToulBar2::startRealTimeAfterPreProcessing;
     double elapsedCPUTime = cpuTime() - ToulBar2::startCpuTime;
     double totalCPUTime = 0.;
-    MPI_Reduce(&elapsedCPUTime, &totalCPUTime, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
-    if (ToulBar2::verbose >= 0 && env0.myrank == 0) { /* use time on master node */
-        cout << "Total CPU time = " << totalCPUTime << " seconds" << endl;
-        cout << "Solving real-time = " << elapsedTime << " seconds (not including preprocessing time)" << endl;
+    mpi::reduce(world, elapsedCPUTime, totalCPUTime, std::plus<double>(), 0);
+    if (ToulBar2::verbose >= 0 && world.rank() == MASTER) { /* use time on master node */
+        cout << "Total CPU time = " << totalCPUTime << " seconds." << endl;
+        cout << "Solving real-time = " << elapsedTime << " seconds (not including reading and preprocessing time)." << endl;
     }
-    MPI_Finalize();
 
     return (bestUb < MAX_COST);
 }
@@ -211,26 +103,23 @@ bool ReplicatedParallelDGVNS::radgvns()
         cout << "Problem decomposition in " << ch->getSize() << " clusters with size distribution: min: " << ch->getMinClusterSize() << " median: " << ch->getMedianClusterSize() << " mean: " << ch->getMeanClusterSize() << " max: " << ch->getMaxClusterSize() << endl;
     }
 
-    // MPI data
-    MPI_Status status;
-
     // verify the number of processes and number of clusters
-    env0.processes = env0.ntasks - 1;
-    //    cout << "number of processes=" << env0.processes << endl;
+    int processes = world.size() - 1;
+    //    cout << "number of processes=" << processes << endl;
     // verify the number of processes and number of clusters
     int npr = 0;
     if (ToulBar2::vnsParallelLimit) {
-        if (env0.ntasks > h->getSize()) {
+        if (world.size() > h->getSize()) {
             npr = h->getSize();
         } else {
-            npr = env0.processes;
+            npr = processes;
         }
     } else {
-        npr = env0.processes;
+        npr = processes;
     }
 
     if (ToulBar2::vnsOutput)
-        ToulBar2::vnsOutput << "#param " << ToulBar2::vnsNeighborVarHeur << " " << ToulBar2::vnsKmin << " " << ToulBar2::vnsKmax << " " << ToulBar2::vnsLDSmin << " " << ToulBar2::vnsInitSol << " " << env0.processes << " " << env0.ntasks << " " << h->getSize() << endl;
+        ToulBar2::vnsOutput << "#param " << ToulBar2::vnsNeighborVarHeur << " " << ToulBar2::vnsKmin << " " << ToulBar2::vnsKmax << " " << ToulBar2::vnsLDSmin << " " << ToulBar2::vnsInitSol << " " << processes << " " << world.size() << " " << h->getSize() << endl;
 
     // Generation of initial Solution
     file = h->getClustersIndex();
@@ -245,42 +134,42 @@ bool ReplicatedParallelDGVNS::radgvns()
     // Get all clusters from the tree decomposition of constraint graph
     int c = 0;
     if (!complete && bestUb > ToulBar2::vnsOptimum) {
-        for (int p = 1; p < npr + 1; ++p) {
-            pr pr_p = pr();
-            pr_p.cl = c;
-            pr_p.k = ToulBar2::vnsKmin;
-            pr_p.lds = ToulBar2::vnsLDSmin;
-            pr_p.synch = false;
-            vecPR.push_back(pr_p);
-            SolToMsg(env0, pr_p.cl, pr_p.k, pr_p.lds, bestUb, bestSolution);
-            MPI_Send(&env0.sendbuff[0], env0.buffsize, MPI_INT, p, WORKTAG, MPI_COMM_WORLD);
-            c = (c + 1) % file.size();
-        }
+        for (int p = 0; p < npr + 1; ++p)
+            if (p != MASTER) {
+                pr pr_p = pr();
+                pr_p.cl = c;
+                pr_p.k = ToulBar2::vnsKmin;
+                pr_p.lds = ToulBar2::vnsLDSmin;
+                pr_p.synch = false;
+                vecPR.push_back(pr_p);
+                SolMsg solmsg(pr_p.cl, pr_p.k, pr_p.lds, bestUb, bestSolution);
+                world.send(p, WORKTAG, solmsg);
+                c = (c + 1) % file.size();
+            }
     }
     Cost pbestUb = MAX_COST;
     map<int, Value> pbestSolution;
     while (npr && !complete && bestUb > ToulBar2::vnsOptimum) {
-        MPI_Recv(&env0.recvbuff[0], env0.buffsize, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
-        int pindex = status.MPI_SOURCE - 1;
-        complete = (status.MPI_TAG == WORKTAG);
+        SolMsg2 solmsg;
+        mpi::status status = world.recv(mpi::any_source, mpi::any_tag, solmsg);
+        int pindex = status.source() - 1;
+        complete = (status.tag() == WORKTAG);
         if ((vecPR[pindex].lds >= ToulBar2::vnsLDSmax || ToulBar2::restart == 1) && vecPR[pindex].k >= ToulBar2::vnsKmax)
             npr = 0;
-        MsgToSol2(env0, wcsp->numberOfVariables(), pbestUb, pbestSolution);
+        solmsg.get(pbestUb, pbestSolution);
         NeighborhoodChange(ToulBar2::vnsNeighborChange, pindex, c, ToulBar2::vnsKmin, ((ClustersNeighborhoodStructure*)h)->getMaxClusterSize() + ((ClustersNeighborhoodStructure*)h)->getSize() - 1, ToulBar2::vnsKmax, ToulBar2::vnsLDSmin, ToulBar2::vnsLDSmax, ToulBar2::vnsNeighborSizeSync, pbestUb, pbestSolution);
         //        if (ToulBar2::restart==1 && find(clusterKmax.begin(), clusterKmax.end(), false) == clusterKmax.end()) npr = 0;
         if (!complete && bestUb > ToulBar2::vnsOptimum) {
-            SolToMsg(env0, vecPR[pindex].cl, vecPR[pindex].k, vecPR[pindex].lds, bestUb, bestSolution);
-            MPI_Send(&env0.sendbuff[0], env0.buffsize, MPI_INT, status.MPI_SOURCE, WORKTAG, MPI_COMM_WORLD);
+            SolMsg solmsg(vecPR[pindex].cl, vecPR[pindex].k, vecPR[pindex].lds, bestUb, bestSolution);
+            world.send(status.source(), WORKTAG, solmsg);
         }
     }
-
-    MPI_Request requests[env0.ntasks];
-    for (int p = 1; p < env0.ntasks; ++p) {
-        //        MPI_Isend(0, 0, MPI_INT, p, DIETAG, MPI_COMM_WORLD);
-
-        MPI_Isend(NULL, 0, MPI_INT, p, DIETAG, MPI_COMM_WORLD, &requests[p]);
-    }
-
+    vector<mpi::request> reqs;
+    for (int p = 0; p < world.size(); ++p)
+        if (p != MASTER) {
+            reqs.push_back(world.isend(p, DIETAG, SolMsg()));
+        }
+    mpi::wait_all(reqs.begin(), reqs.end());
     return complete && (bestUb < MAX_COST);
 }
 
@@ -301,30 +190,27 @@ bool ReplicatedParallelDGVNS::rsdgvns()
         cout << "Problem decomposition in " << ch->getSize() << " clusters with size distribution: min: " << ch->getMinClusterSize() << " median: " << ch->getMedianClusterSize() << " mean: " << ch->getMeanClusterSize() << " max: " << ch->getMaxClusterSize() << endl;
     }
 
-    // MPI data
-    MPI_Status status;
-
     // Initialization
     map<int, Value> bestInterSolution;
     Cost bestInterUb = MAX_COST;
 
     // verify the number of processes and number of clusters
-    env0.processes = env0.ntasks - 1;
-    cout << "number of processes=" << env0.processes << endl;
+    int processes = world.size() - 1;
+    cout << "number of processes=" << processes << endl;
     // verify the number of processes and number of clusters
     int npr = 0;
     if (ToulBar2::vnsParallelLimit) {
-        if (env0.ntasks > h->getSize()) {
+        if (world.size() > h->getSize()) {
             npr = h->getSize();
         } else {
-            npr = env0.processes;
+            npr = processes;
         }
     } else {
-        npr = env0.processes;
+        npr = processes;
     }
 
     if (ToulBar2::vnsOutput)
-        ToulBar2::vnsOutput << "#param " << ToulBar2::vnsNeighborVarHeur << " " << ToulBar2::vnsKmin << " " << ToulBar2::vnsKmax << " " << ToulBar2::vnsLDSmin << " " << ToulBar2::vnsInitSol << " " << env0.processes << " " << env0.ntasks << " " << h->getSize() << endl;
+        ToulBar2::vnsOutput << "#param " << ToulBar2::vnsNeighborVarHeur << " " << ToulBar2::vnsKmin << " " << ToulBar2::vnsKmax << " " << ToulBar2::vnsLDSmin << " " << ToulBar2::vnsInitSol << " " << processes << " " << world.size() << " " << h->getSize() << endl;
 
     // Generation of initial Solution
     file = h->getClustersIndex();
@@ -354,20 +240,21 @@ bool ReplicatedParallelDGVNS::rsdgvns()
         int k = ToulBar2::vnsKmin;
         while (npr && !complete && k <= ToulBar2::vnsKmax && bestUb > ToulBar2::vnsOptimum) {
 
-            for (int p = 1; p < npr + 1; ++p) {
-                SolToMsg(env0, c, k, lds, bestUb, bestSolution);
-                MPI_Send(&env0.sendbuff[0], env0.buffsize, MPI_INT, p, WORKTAG, MPI_COMM_WORLD);
-                c = (c + 1) % file.size();
-            }
+            for (int p = 0; p < npr + 1; ++p)
+                if (p != MASTER) {
+                    SolMsg solmsg(c, k, lds, bestUb, bestSolution);
+                    world.send(p, WORKTAG, solmsg);
+                    c = (c + 1) % file.size();
+                }
 
             int finished = 0;
-            //S"
             Cost pBestUb;
             map<int, Value> pBestSolution;
 
             while (finished < npr) {
-                MPI_Recv(&env0.recvbuff[0], env0.buffsize, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
-                MsgToSol2(env0, wcsp->numberOfVariables(), pBestUb, pBestSolution);
+                SolMsg2 solmsg;
+                world.recv(mpi::any_source, mpi::any_tag, solmsg);
+                solmsg.get(pBestUb, pBestSolution);
                 if (pBestUb < bestInterUb) {
                     bestInterUb = pBestUb;
                     for (int v = 0; v < (int)wcsp->numberOfVariables(); v++) {
@@ -408,7 +295,7 @@ bool ReplicatedParallelDGVNS::rsdgvns()
                         break;
                     default:
                         cerr << "Unknown neighborhood size increment strategy inside VNS (see option -kinc)!" << endl;
-                        exit(EXIT_FAILURE);
+                        throw BadConfiguration();
                     }
                     k = min(k, ToulBar2::vnsKmax);
                 } else
@@ -432,7 +319,7 @@ bool ReplicatedParallelDGVNS::rsdgvns()
                         break;
                     default:
                         cerr << "Unknown LDS increment strategy inside VNS (see option -linc)!" << endl;
-                        exit(EXIT_FAILURE);
+                        throw BadConfiguration();
                     }
                     lds = min(lds, ToulBar2::vnsLDSmax);
                 }
@@ -440,13 +327,12 @@ bool ReplicatedParallelDGVNS::rsdgvns()
                 stop = true;
         }
     }
-
-    MPI_Request requests[env0.ntasks];
-    for (int p = 1; p < env0.ntasks; ++p) {
-        //        MPI_Isend(0, 0, MPI_INT, p, DIETAG, MPI_COMM_WORLD);
-        MPI_Isend(NULL, 0, MPI_INT, p, DIETAG, MPI_COMM_WORLD, &requests[p]);
-    }
-
+    vector<mpi::request> reqs;
+    for (int p = 0; p < world.size(); ++p)
+        if (p != MASTER) {
+            reqs.push_back(world.isend(p, DIETAG, SolMsg()));
+        }
+    mpi::wait_all(reqs.begin(), reqs.end());
     return complete && (bestUb < MAX_COST);
 }
 
@@ -456,40 +342,39 @@ bool ReplicatedParallelDGVNS::slave()
     ParallelRandomClusterChoice* h = new ParallelRandomClusterChoice();
     h->init(wcsp, this);
 
-    // MPI data
-    MPI_Status status;
-
     // loop for getting new initial solution to initialize the search process
     bool complete = false;
     while (true) {
-        MPI_Recv(&env0.recvbuff[0], env0.buffsize, MPI_INT, 0, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
-        if (status.MPI_TAG == DIETAG)
+        SolMsg solmsg;
+        mpi::status status = world.recv(0, mpi::any_tag, solmsg);
+        if (status.tag() == DIETAG)
             return complete; // warning! always wait for DIETAG before stopping
         /* launch Vns/Lds+cp */
-        complete = complete || VnsLdsCP(env0, h);
+        SolMsg2 solmsg2;
+        complete = complete || VnsLdsCP(solmsg, h, solmsg2);
         /* Send the result back */
-        MPI_Request request;
-        MPI_Isend(&env0.sendbuff[0], env0.buffsize, MPI_INT, 0, complete, MPI_COMM_WORLD, &request);
+        mpi::request req = world.isend(0, complete, solmsg2);
+        while (!req.test().is_initialized() && !MPI_interrupted())
+            ;
     }
     return complete;
 }
 
 //------------------- Base functions -----------------------//
 
-bool ReplicatedParallelDGVNS::VnsLdsCP(MPIEnv& env0, ParallelRandomClusterChoice* h)
+bool ReplicatedParallelDGVNS::VnsLdsCP(SolMsg& solmsg, ParallelRandomClusterChoice* h, SolMsg2& solmsg2)
 {
     int cluster, k, discrepancy;
-    MsgToSol(env0, wcsp->numberOfVariables(), cluster, k, discrepancy, bestUb,
-        bestSolution);
+    solmsg.get(cluster, k, discrepancy, bestUb, bestSolution);
     for (map<int, Value>::iterator it = bestSolution.begin();
          it != bestSolution.end(); ++it)
         lastSolution[(*it).first] = (*it).second;
     lastUb = bestUb;
-    //vns/lds+cp
+    // vns/lds+cp
     set<int> neighborhood = h->SlaveGetNeighborhood(cluster, k); // based shuffle
     vector<int> variables;
     variables.reserve(unassignedVars->getSize());
-    vector<int> values;
+    vector<Value> values;
     values.reserve(unassignedVars->getSize());
     for (BTList<Value>::iterator iter = unassignedVars->begin(); iter != unassignedVars->end(); ++iter) {
         int v = *iter;
@@ -499,13 +384,13 @@ bool ReplicatedParallelDGVNS::VnsLdsCP(MPIEnv& env0, ParallelRandomClusterChoice
         }
     }
     if (ToulBar2::verbose >= 1) {
-        cout << env0.myrank << ": LDS " << discrepancy << " Neighborhood " << k << ": ";
+        cout << world.rank() << ": LDS " << discrepancy << " Neighborhood " << k << ": ";
         for (set<int>::iterator it = neighborhood.begin(); it != neighborhood.end(); it++)
             cout << " " << *it;
         cout << endl;
     }
 
-    //repair
+    // repair
     bool complete = false;
     ToulBar2::vnsKcur = k;
     ToulBar2::vnsLDScur = (ToulBar2::lds) ? discrepancy : -1;
@@ -513,7 +398,7 @@ bool ReplicatedParallelDGVNS::VnsLdsCP(MPIEnv& env0, ParallelRandomClusterChoice
         complete = repair_recursiveSolve(discrepancy, variables, values, bestUb);
     else
         complete = repair_recursiveSolve(variables, values, bestUb);
-    SolToMsg2(env0, lastUb, lastSolution);
+    solmsg2 = SolMsg2(lastUb, lastSolution);
     return complete;
 }
 
@@ -582,7 +467,7 @@ void ReplicatedParallelDGVNS::ChangeClusterAlways(
                     break;
                 default:
                     cerr << "Unknown neighborhood size increment strategy inside VNS (see option -kinc)!" << endl;
-                    exit(EXIT_FAILURE);
+                    throw BadConfiguration();
                 }
                 vecPR[p].k = min(vecPR[p].k, kmax);
             } else if (ToulBar2::restart > 1) { // Warning, unbounded number of restarts...
@@ -598,7 +483,7 @@ void ReplicatedParallelDGVNS::ChangeClusterAlways(
                         break;
                     default:
                         cerr << "Unknown LDS increment strategy inside VNS (see option -linc)!" << endl;
-                        exit(EXIT_FAILURE);
+                        throw BadConfiguration();
                     }
                     vecPR[p].lds = min(vecPR[p].lds, ToulBar2::vnsLDSmax);
                 }
@@ -655,7 +540,7 @@ void ReplicatedParallelDGVNS::ChangeClusterWhenNotImproved(
                     break;
                 default:
                     cerr << "Unknown neighborhood size increment strategy inside VNS (see option -kinc)!" << endl;
-                    exit(EXIT_FAILURE);
+                    throw BadConfiguration();
                 }
                 vecPR[p].k = min(vecPR[p].k, kmax);
             } else if (ToulBar2::restart > 1) { // Warning, unbounded number of restarts...
@@ -671,7 +556,7 @@ void ReplicatedParallelDGVNS::ChangeClusterWhenNotImproved(
                         break;
                     default:
                         cerr << "Unknown LDS increment strategy inside VNS (see option -linc)!" << endl;
-                        exit(EXIT_FAILURE);
+                        throw BadConfiguration();
                     }
                     vecPR[p].lds = min(vecPR[p].lds, ToulBar2::vnsLDSmax);
                 }
@@ -688,8 +573,9 @@ void ReplicatedParallelDGVNS::DumpBestSol(bool improved)
     // Save
     wcsp->setSolution(bestUb, &bestSolution);
     if (ToulBar2::vnsOutput) {
-        //cout << "# ------------------------------------------" << endl;
-        ToulBar2::vnsOutput << "InstanceVnsBestTime " << MPI_Wtime() - startTime << endl;
+        // cout << "# ------------------------------------------" << endl;
+        double elapsedTime = realTime() - ToulBar2::startRealTime;
+        ToulBar2::vnsOutput << "InstanceVnsBestTime " << elapsedTime << endl;
         ToulBar2::vnsOutput << "Cost " << std::fixed << std::setprecision(ToulBar2::decimalPoint) << wcsp->Cost2ADCost(bestUb) << std::setprecision(DECIMAL_POINT) << endl;
         ToulBar2::vnsOutput << "Solution ";
         for (map<int, Value>::iterator it = bestSolution.begin();
@@ -709,23 +595,25 @@ void ReplicatedParallelDGVNS::DumpBestSol(bool improved)
         //        cout << endl;
         //        cout << "# ------------------------------------------" << endl;
     }
-    if (improved && ToulBar2::verbose >= 0) {
+    if (improved && (ToulBar2::verbose >= 0 || ToulBar2::showSolutions)) {
+        double elapsedTime = realTime() - ToulBar2::startRealTime;
         if (!ToulBar2::bayesian)
-            cout << "New solution: " << std::fixed << std::setprecision(ToulBar2::decimalPoint) << wcsp->Cost2ADCost(bestUb) << std::setprecision(DECIMAL_POINT) << " in " << (MPI_Wtime() - startTime) << " seconds." << endl;
+            cout << "New solution: " << std::fixed << std::setprecision(ToulBar2::decimalPoint) << wcsp->Cost2ADCost(bestUb) << std::setprecision(DECIMAL_POINT) << " in " << elapsedTime << " seconds." << endl;
         else
-            cout << "New solution: " << bestUb << " energy: " << -(wcsp->Cost2LogProb(bestUb) + ToulBar2::markov_log) << " prob: " << wcsp->Cost2Prob(bestUb) * Exp(ToulBar2::markov_log) << " in " << MPI_Wtime() - startTime << " seconds." << endl;
+            cout << "New solution: " << bestUb << " energy: " << -(wcsp->Cost2LogProb(bestUb) + ToulBar2::markov_log) << " prob: " << std::scientific << wcsp->Cost2Prob(bestUb) * Exp(ToulBar2::markov_log) << std::fixed << " in " << elapsedTime << " seconds." << endl;
     }
     if (improved && ToulBar2::showSolutions) {
         wcsp->printSolution();
         cout << endl;
     }
-    if (ToulBar2::writeSolution && ToulBar2::solutionFile != NULL) {
+    if (!ToulBar2::uaieval && ToulBar2::writeSolution && ToulBar2::solutionFile != NULL) {
         fseek(ToulBar2::solutionFile, ToulBar2::solutionFileRewindPos, SEEK_SET);
         wcsp->printSolution(ToulBar2::solutionFile);
         fprintf(ToulBar2::solutionFile, "\n");
     }
     if (ToulBar2::xmlflag) {
-        cout << "o " << bestUb << endl;
+        cout << "o " << std::fixed << std::setprecision(0) << wcsp->Cost2ADCost(bestUb) << std::setprecision(DECIMAL_POINT) << endl; //" ";
+        ((WCSP*)wcsp)->solution_XML(false);
     }
     if (ToulBar2::maxsateval) {
         cout << "o " << bestUb << endl;
