@@ -10,6 +10,7 @@
 #include "tb2vns.hpp"
 #include "core/tb2wcsp.hpp"
 #include "search/tb2clusters.hpp"
+#include <random>
 
 using namespace boost;
 
@@ -20,8 +21,8 @@ void ClustersNeighborhoodStructure::load_decomposition()
         bool cover = false;
         if (ToulBar2::clusterFile.find(".cov") != string::npos)
             cover = true;
-        fstream file(ToulBar2::clusterFile.c_str());
-        //file >> clustersNumber;
+        std::fstream file(ToulBar2::clusterFile.c_str());
+        // file >> clustersNumber;
         set<int> nbvars;
         set<int> nbunvars;
         while (!file.eof()) {
@@ -35,19 +36,19 @@ void ClustersNeighborhoodStructure::load_decomposition()
                     ss >> clusterid;
                     ss >> parentid;
                 }
-                do {
+                while (ss.good()) {
                     unsigned int var;
                     ss >> var;
                     nbvars.insert(var);
                     if (var >= wcsp->numberOfVariables()) {
                         cerr << "Error: cluster decomposition contains bad variable index!" << endl;
-                        exit(EXIT_FAILURE);
+                        throw WrongFileFormat();
                     }
                     if (wcsp->unassigned(var)) {
                         tmp.insert(var);
                         nbunvars.insert(var);
                     }
-                } while (ss.good());
+                };
                 if (tmp.size() > 0) {
                     TDCluster c = add_vertex(m_graph);
                     m_graph[c].vars = tmp;
@@ -55,9 +56,8 @@ void ClustersNeighborhoodStructure::load_decomposition()
             }
         }
         file.close();
-        if (nbvars.size() != wcsp->numberOfVariables()) {
-            cerr << "Error: cluster decomposition has missing variables! (" << nbvars.size() << "!=" << wcsp->numberOfVariables() << ")" << endl;
-            exit(EXIT_FAILURE);
+        if (nbvars.size() < wcsp->numberOfVariables()) {
+            cout << "Warning: cluster decomposition has missing variables! (" << nbvars.size() << "!=" << wcsp->numberOfVariables() << ")" << endl;
         }
         assert(nbunvars.size() == wcsp->numberOfUnassignedVariables());
         TCDGraph::vertex_iterator v, vend, v2;
@@ -102,7 +102,7 @@ void ClustersNeighborhoodStructure::load_decomposition()
         for (int i = 0; i < nc; i++) {
             Cluster* ct = td->getCluster(i);
             if (ct->getSep())
-                ct->getSep()->deconnect(); //deconnect separator constraints
+                ct->getSep()->deconnect(); // deconnect separator constraints
             set<int> unassignedvars;
             set<int> cvars = ct->getVars();
             for (TVars::iterator iter = cvars.begin(); iter != cvars.end(); ++iter)
@@ -138,7 +138,7 @@ void ClustersNeighborhoodStructure::load_decomposition()
         m_graph[c].name = to_string(0);
         m_graph[c].vars = l->getUnassignedVars();
     }
-    if (fabs(ToulBar2::boostingBTD) > 0. && fabs(ToulBar2::boostingBTD) < 1.) {
+    if (std::abs(ToulBar2::boostingBTD) > 0. && std::abs(ToulBar2::boostingBTD) < 1.) {
         TCDGraph abs_graph; // graph absorption
         cluster_graph_absorption(m_graph, abs_graph);
         m_graph = abs_graph;
@@ -177,11 +177,12 @@ const zone RandomNeighborhoodChoice::getNeighborhood(size_t neighborhood_size)
     zone neighborhood;
     vector<int> z(l->unassignedVars->getSize());
     unsigned int j = 0;
+
     for (BTList<Value>::iterator iter = l->unassignedVars->begin(); iter != l->unassignedVars->end(); ++iter) {
         z[j] = *iter;
         ++j;
     }
-    random_shuffle(z.begin(), z.end());
+    shuffle(z.begin(), z.end(), myrandom_generator);
     assert(neighborhood_size <= z.size());
     neighborhood.insert(z.begin(), z.begin() + neighborhood_size);
     return neighborhood;
@@ -190,7 +191,8 @@ const zone RandomNeighborhoodChoice::getNeighborhood(size_t neighborhood_size, z
 {
     zone neighborhood;
     vector<int> zv(z.begin(), z.end());
-    random_shuffle(zv.begin(), zv.end());
+
+    shuffle(zv.begin(), zv.end(), myrandom_generator);
     assert(neighborhood_size <= zv.size());
     neighborhood.insert(zv.begin(), zv.begin() + neighborhood_size);
     return neighborhood;
@@ -198,7 +200,6 @@ const zone RandomNeighborhoodChoice::getNeighborhood(size_t neighborhood_size, z
 
 void RandomClusterChoice::init(WeightedCSP* wcsp_, LocalSearch* l_)
 {
-
     this->l = l_;
     this->wcsp = wcsp_;
     maxClusterSize = 0;
@@ -235,20 +236,18 @@ void RandomClusterChoice::init(WeightedCSP* wcsp_, LocalSearch* l_)
         clusters.pop_back();
     }
     file = clusters;
-    random_shuffle(file.begin(), file.end());
+    shuffle(file.begin(), file.end(), myrandom_generator);
     insideHeuristic = new RandomNeighborhoodChoice();
-    precK = -1;
     insideHeuristic->init(wcsp, l);
 }
 
 const zone RandomClusterChoice::getNeighborhood(size_t neighborhood_size)
 {
     assert(neighborhood_size <= wcsp->numberOfUnassignedVariables());
-    precK = neighborhood_size;
     set<int> selclusters;
     if (file.size() == 0) {
         file = clusters;
-        random_shuffle(file.begin(), file.end());
+        shuffle(file.begin(), file.end(), myrandom_generator);
     }
     assert(file.size() > 0);
     int c = file.back();
@@ -257,7 +256,7 @@ const zone RandomClusterChoice::getNeighborhood(size_t neighborhood_size)
     if (ToulBar2::verbose >= 1)
         cout << "Select cluster " << c << endl;
     zone z = m_graph[c].vars;
-    //if variables are missing
+    // if variables are missing
     if (z.size() < neighborhood_size) {
         queue<int> fifo;
         TCDGraph::adjacency_iterator v, vend;
@@ -267,9 +266,9 @@ const zone RandomClusterChoice::getNeighborhood(size_t neighborhood_size)
             tie(v, vend) = adjacent_vertices(currclu, m_graph);
             if (v != vend) {
                 vector<int> neighbors(v, vend);
-                //merge neighbors of current cluster
-                random_shuffle(neighbors.begin(), neighbors.end());
-                //add them to list
+                // merge neighbors of current cluster
+                shuffle(neighbors.begin(), neighbors.end(), myrandom_generator);
+                // add them to list
                 for (vector<int>::iterator it = neighbors.begin();
                      it != neighbors.end(); ++it) {
                     if (selclusters.count(*it) == 0) {
@@ -281,7 +280,7 @@ const zone RandomClusterChoice::getNeighborhood(size_t neighborhood_size)
             if (fifo.size() == 0) {
                 assert(selclusters.size() < clusters.size());
                 while (selclusters.count(i) > 0) {
-                    i = (i + 1) % clusters.size(); //TODO: use file instead in order to shuffle clusters randomly
+                    i = (i + 1) % clusters.size(); // TODO: use file instead in order to shuffle clusters randomly
                 }
                 fifo.push(clusters[i]);
                 selclusters.insert(clusters[i]);
@@ -305,7 +304,7 @@ bool RandomClusterChoice::incrementK()
 {
     if (file.size() == 0) {
         file = clusters;
-        random_shuffle(file.begin(), file.end());
+        shuffle(file.begin(), file.end(), myrandom_generator);
         return true;
     }
 
@@ -350,21 +349,19 @@ void ParallelRandomClusterChoice::init(WeightedCSP* wcsp_, LocalSearch* l_)
         clusters.pop_back();
     }
     file = clusters;
-    random_shuffle(file.begin(), file.end());
+    shuffle(file.begin(), file.end(), myrandom_generator);
     insideHeuristic = new RandomNeighborhoodChoice();
-    precK = -1;
     insideHeuristic->init(wcsp, l);
 }
 
 const zone ParallelRandomClusterChoice::getNeighborhood(size_t neighborhood_size)
 {
-    precK = neighborhood_size;
     set<int> selclusters;
     int c = file.back();
     file.pop_back();
     selclusters.insert(c);
     zone z = m_graph[c].vars;
-    //if variables are missing
+    // if variables are missing
     if (z.size() < neighborhood_size) {
         queue<int> fifo;
         TCDGraph::adjacency_iterator v, vend;
@@ -374,9 +371,9 @@ const zone ParallelRandomClusterChoice::getNeighborhood(size_t neighborhood_size
             tie(v, vend) = adjacent_vertices(currclu, m_graph);
             if (v != vend) {
                 vector<int> neighbors(v, vend);
-                //merge neighbors of current cluster
-                random_shuffle(neighbors.begin(), neighbors.end());
-                //add them to list
+                // merge neighbors of current cluster
+                shuffle(neighbors.begin(), neighbors.end(), myrandom_generator);
+                // add them to list
                 for (vector<int>::iterator it = neighbors.begin();
                      it != neighbors.end(); ++it) {
                     if (selclusters.count(*it) == 0) {
@@ -388,7 +385,7 @@ const zone ParallelRandomClusterChoice::getNeighborhood(size_t neighborhood_size
             if (fifo.size() == 0) {
                 assert(selclusters.size() < clusters.size());
                 while (selclusters.count(i) > 0) {
-                    i = (i + 1) % clusters.size(); //TODO: use file instead in order to shuffle clusters randomly
+                    i = (i + 1) % clusters.size(); // TODO: use file instead in order to shuffle clusters randomly
                 }
                 fifo.push(clusters[i]);
                 selclusters.insert(clusters[i]);
@@ -410,11 +407,10 @@ const zone ParallelRandomClusterChoice::getNeighborhood(size_t neighborhood_size
 
 const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int CurrentCluster, size_t neighborhood_size)
 {
-    precK = neighborhood_size;
     set<int> selclusters;
     selclusters.insert(CurrentCluster);
     zone z = m_graph[CurrentCluster].vars;
-    //if variables are missing
+    // if variables are missing
     if (z.size() < neighborhood_size) {
         queue<int> fifo;
         TCDGraph::adjacency_iterator v, vend;
@@ -424,9 +420,9 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
             tie(v, vend) = adjacent_vertices(currclu, m_graph);
             if (v != vend) {
                 vector<int> neighbors(v, vend);
-                //merge neighbors of current cluster
-                random_shuffle(neighbors.begin(), neighbors.end());
-                //add them to list
+                // merge neighbors of current cluster
+                shuffle(neighbors.begin(), neighbors.end(), myrandom_generator);
+                // add them to list
                 for (vector<int>::iterator it = neighbors.begin(); it != neighbors.end(); ++it) {
                     if (selclusters.count(*it) == 0) {
                         fifo.push(*it);
@@ -437,7 +433,7 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
             if (fifo.size() == 0) {
                 assert(selclusters.size() < clusters.size());
                 while (selclusters.count(i) > 0) {
-                    i = (i + 1) % clusters.size(); //TODO: use file instead in order to shuffle clusters randomly
+                    i = (i + 1) % clusters.size(); // TODO: use file instead in order to shuffle clusters randomly
                 }
                 fifo.push(clusters[i]);
                 selclusters.insert(clusters[i]);
@@ -452,11 +448,10 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
 
 const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int CurrentCluster, uint number, size_t NeighborhoodSize)
 {
-    precK = NeighborhoodSize;
     set<int> selclusters;
     selclusters.insert(CurrentCluster);
     zone z = m_graph[CurrentCluster].vars;
-    //if variables are missing
+    // if variables are missing
     if (z.size() < NeighborhoodSize && number > 0) {
         queue<int> fifo;
         TCDGraph::adjacency_iterator v, vend;
@@ -467,9 +462,9 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
             tie(v, vend) = adjacent_vertices(currclu, m_graph);
             if (v != vend) {
                 vector<int> neighbors(v, vend);
-                //merge neighbors of current cluster
-                random_shuffle(neighbors.begin(), neighbors.end());
-                //add them to list
+                // merge neighbors of current cluster
+                shuffle(neighbors.begin(), neighbors.end(), myrandom_generator);
+                // add them to list
                 for (vector<int>::iterator it = neighbors.begin();
                      it != neighbors.end(); ++it) {
                     if (selclusters.count(*it) == 0) {
@@ -481,7 +476,7 @@ const zone ParallelRandomClusterChoice::SlaveGetNeighborhood(unsigned int Curren
             if (fifo.size()) {
                 assert(selclusters.size() < clusters.size());
                 while (selclusters.count(i) > 0) {
-                    i = (i + 1) % clusters.size(); //TODO: use file instead in order to shuffle clusters randomly
+                    i = (i + 1) % clusters.size(); // TODO: use file instead in order to shuffle clusters randomly
                 }
                 fifo.push(clusters[i]);
                 selclusters.insert(clusters[i]);
@@ -504,7 +499,7 @@ bool ParallelRandomClusterChoice::incrementK()
 {
     if (file.size() == 0) {
         file = clusters;
-        random_shuffle(file.begin(), file.end());
+        shuffle(file.begin(), file.end(), myrandom_generator);
         return true;
     }
 
@@ -530,9 +525,9 @@ uint ParallelRandomClusterChoice::getClustersSize(uint c, uint number)
             tie(v, vend) = adjacent_vertices(currclu, m_graph);
             if (v != vend) {
                 vector<int> neighbors(v, vend);
-                //merge neighbors of current cluster
-                random_shuffle(neighbors.begin(), neighbors.end());
-                //add them to list
+                // merge neighbors of current cluster
+                shuffle(neighbors.begin(), neighbors.end(), myrandom_generator);
+                // add them to list
                 for (vector<int>::iterator it = neighbors.begin();
                      it != neighbors.end(); ++it) {
                     if (selclusters.count(*it) == 0) {
@@ -544,7 +539,7 @@ uint ParallelRandomClusterChoice::getClustersSize(uint c, uint number)
             if (fifo.size() == 0) {
                 assert(selclusters.size() < clusters.size());
                 while (selclusters.count(i) > 0) {
-                    i = (i + 1) % clusters.size(); //TODO: use file instead in order to shuffle clusters randomly
+                    i = (i + 1) % clusters.size(); // TODO: use file instead in order to shuffle clusters randomly
                 }
                 fifo.push(clusters[i]);
                 selclusters.insert(clusters[i]);
