@@ -15,12 +15,12 @@
 using namespace CSPXMLParser;
 
 /*
- * A simple demo to illustrate how to use the XML CSP parser 
+ * A simple demo to illustrate how to use the XML CSP parser
  */
 
 #define MAXDOMS 100000
 #define MAXDOMSIZE MAX_DOMAIN_SIZE
-#define MAXDOMSIZEZERO 1000
+#define MAXDOMSIZEZERO (MAX_DOMAIN_SIZE/2)
 #define MAX_COST_XML MAX_COST
 
 /**
@@ -82,7 +82,7 @@ private:
     int currentIndexArity;
 
     vector<int> nDoms;
-    vector<vector<int> > DomsToIndex;
+    vector<vector<int>> DomsToIndex;
 
     int maxDomSize;
 
@@ -226,7 +226,7 @@ public:
         preds[name] = p;
         currentP = p;
         intension = true;
-        cerr << "Warning!!! Predicate " << name << " not implemented, just skip it..." << endl;
+        cerr << "Warning! Predicate " << name << " not implemented, just skip it..." << endl;
     }
 
     virtual void addFormalParameter(int pos, const string& name,
@@ -389,14 +389,14 @@ public:
                     assert(domsize >= 0);
                     if (domsize != wcsp->getDomainInitSize(varindex)) {
                         cerr << "wrong domain size " << domsize << " compared to previous one " << wcsp->getDomainInitSize(varindex) << endl;
-                        exit(EXIT_FAILURE);
+                        throw WrongFileFormat();
                     }
                     for (int idx = 0; idx < domsize; idx++) {
                         int v = wcsp->Doms[wcsp->varsDom[varindex]][idx];
                         string vname = "v" + to_string(v);
                         if (((EnumeratedVariable *) wcsp->getVar(varindex))->getValueName(idx) != vname) {
                             cerr << "wrong domain value name " << vname << " compared to previous one " << ((EnumeratedVariable *) wcsp->getVar(varindex))->getValueName(idx) << endl;
-                            exit(EXIT_FAILURE);
+                            throw WrongFileFormat();
                         }
                     }
                 } else {
@@ -479,17 +479,17 @@ public:
                                 strvalues[i] = DomsToIndex[wcsp->varsDom[scopeIndex[i]]][MAXDOMSIZEZERO + t[pos]];
                             }
                             if (r->type == REL_SUPPORT) {
-                                wcsp->postNaryConstraintTuple(ctrIndex, strvalues, MIN_COST);
+                                wcsp->postNaryConstraintTupleInternal(ctrIndex, strvalues, MIN_COST);
                             } else if (r->type == REL_CONFLICT) {
 #ifdef MAXCSP
-                                wcsp->postNaryConstraintTuple(ctrIndex, strvalues, UNIT_COST);
+                                wcsp->postNaryConstraintTupleInternal(ctrIndex, strvalues, UNIT_COST);
 #else
-                                wcsp->postNaryConstraintTuple(ctrIndex, strvalues, MAX_COST_XML);
+                                wcsp->postNaryConstraintTupleInternal(ctrIndex, strvalues, MAX_COST_XML);
 #endif
                             } else if (r->type == REL_SOFT) {
-                                wcsp->postNaryConstraintTuple(ctrIndex, strvalues, t[arity]);
+                                wcsp->postNaryConstraintTupleInternal(ctrIndex, strvalues, t[arity]);
                             } else {
-                                wcsp->postNaryConstraintTuple(ctrIndex, strvalues, MAX_COST_XML);
+                                wcsp->postNaryConstraintTupleInternal(ctrIndex, strvalues, MAX_COST_XML);
                             }
 
                             ++itl;
@@ -520,7 +520,7 @@ public:
                                 int pos = scopeOrder[scopeIndex[i]];
                                 values[i] = DomsToIndex[wcsp->varsDom[scopeIndex[i]]][MAXDOMSIZEZERO + t[pos]];
                             }
-                            int pos = values[0] * y->getDomainInitSize() * z->getDomainInitSize() + values[1] * z->getDomainInitSize() + values[2];
+                            size_t pos = (size_t)values[0] * y->getDomainInitSize() * z->getDomainInitSize() + (size_t)values[1] * z->getDomainInitSize() + values[2];
                             if (r->type == REL_SUPPORT) {
                                 costs[pos] = MIN_COST;
                             } else if (r->type == REL_CONFLICT) {
